// CircularBuffer.cc
// 
// Implementation of circular buffer / FIFO 
// for audgestenv
//
// dpwe@media.mit.edu 1995feb04
//

#include <stdio.h>
#include <assert.h>
#include "CircularBuffer.h"

void CircularBuffer::print(char *tag)
{   /* for debugging, print the contents of the circular buffer to stdout */
    if(tag) 
    	fprintf(stdout,"<CircularBuffer#0x%lx'%s',sz%d,ct%d>:\n", 
		this, tag, bufsize, count);
    else
    	fprintf(stdout,"<CircularBuffer#0x%lx,sz%d,ct%d>:\n", 
		this, bufsize, count);
    int n;
    for(n=0; n<bufsize; ++n)  {
    	if(n==head)  {
    	    if(n==tail)  {
		if(isEmpty)
		    fprintf(stdout, "O");
		else
		    fprintf(stdout, "#");
	    } else {
    		fprintf(stdout, ">");
	    }
    	} else if(n==tail)  {
    	    fprintf(stdout, "<");
    	}
    	fprintf(stdout, "%.1f", buf[n]);
    	fprintf(stdout, " ");
    }
    fprintf(stdout, "\n");
}

int CircularBuffer::Resize(int size)	// Resize the buffer
{ 
    int origavai = Avai();
    if(count == 0)  {
	if(bufsize)
	    delete [] buf; 
	bufsize = size; 
	if(bufsize)  {  
	    buf = new DATA[bufsize]; DATAclear(buf, bufsize); 
	}
    } else if(size)  {
	DATA *newbuf = new DATA[size];
	int commonsize = min(size, min(count, bufsize));
	int newearliest = count-commonsize;
	readFrom(newbuf, commonsize, newearliest);
	int tailcount = TailCount();
	// OK got all the data we want
	// now change the state
	if(bufsize) 	
	    delete [] buf;
	buf = newbuf;
	bufsize = size;
	head = commonsize % bufsize;
	if(tailcount < count-bufsize)	tailcount = count-bufsize;
	SetTail(tailcount);
	if(savedPos < count-bufsize)	savedPos = count-bufsize; // I guess
    } else {	// new size is zero 
	if(bufsize)	
	    delete [] buf;
	bufsize = 0;
    }
//    fprintf(stderr, "CBresize 0x%lx to %d, avai %d..%d\n", this, bufsize, 
//	    origavai, Avai());
    return bufsize; 
}

// ---------------------- CircularBuffer related functions --------------

double CircBufInnerProdFrom(CircularBuffer *cb1, int base1, 
		            CircularBuffer *cb2, int base2, int npoints)
{   /* calculate the inner product (sum of products of elements)
       for the <npoints> from the two circular buffers, starting at 
       the specified indices. */
    DATA *ptr1, *ptr2;
    int done = 0, size1, size2, todo;
    double rslt = 0;
    
    while(done < npoints)  {
    	size1 = cb1->GetTailPtrFrom(&ptr1, base1);
    	size2 = cb2->GetTailPtrFrom(&ptr2, base2);
    	todo = min(size1,size2);
    /*	fprintf(stderr, "bs1 %d, sz1 %d, bs2 %d, sz2 %d\n", 
		base1, size1, base2, size2);  */
    	assert(todo>0);
    	rslt += DATAinnerProduct(ptr1, 1, ptr2, 1, todo);
    	done += todo;
    	base1 += todo;
    	base2 += todo;
    }
    return rslt;
}

int CircBufInputOutputFrom(CircularBuffer *input, int cb1base, int cb1len, 
			   CircularBuffer *output, 
			   DATAprocess *proc)
{   /* apply the DATAprocess to <cb1len> points of input taken from 
       circular buffer <input> starting at item <cb1base>.
       Return number of input samples used, may be less than requested. */
    DATA *inptr, *outptr;
    int insize, outsize, outdone, indone, inrem;
    int done=0, go_on=1;
    
    while(done < cb1len && go_on)  {
    	insize  =input->GetTailPtrFrom(&inptr, cb1base);
    	assert(insize);
    	outsize = output->GetHeadPtr(&outptr);
	assert(outsize);
    	inrem   = min(insize, cb1len-done);
    	indone  = proc->process(inptr, 1, inrem, outptr, 1, outsize, &outdone);
    	output->UpdateHead(outdone);
    	cb1base += indone;
    	done += indone;
    	if(indone == 0 && outdone == 0)  {
	    go_on = 0;
    	}
    }
    return done;
}

double CircBufInnerProd(CircularBuffer *cb1, 
		        CircularBuffer *cb2, int npoints)
{   /* calculate the inner product (sum of products of elements)
       for the <npoints> from the two circular buffers, starting at 
       the tails. */
    DATA *ptr1, *ptr2;
    int done = 0, size1, size2, thistime;
    double rslt = 0;
    
    while(done < npoints)  {
    	size1 = cb1->GetTailPtr(&ptr1);
    	size2 = cb2->GetTailPtr(&ptr2);
    	thistime = min(size1,size2);
    /*	fprintf(stderr, "sz1 %d, sz2 %d\n", 
		size1, size2);	*/
    	assert(thistime>0);
    	rslt += DATAinnerProduct(ptr1, 1, ptr2, 1, thistime);
	cb1->UpdateTail(thistime);
	cb2->UpdateTail(thistime);
    	done += thistime;
    }
    return rslt;
}

int CircBufInputOutput(CircularBuffer *input, CircularBuffer *output, 
		       DATAprocess *proc)
{   /* apply the DATAprocess to all Avai() points from input, write 
       result to output.
       Return number of input samples used, may be less than requested. */
    DATA *inptr, *outptr;
    int insize, outsize, outdone, indone, inrem, todo;
    int done=0, go_on=1;
    
    todo = input->Avai();
    while(done < todo && go_on)  {
    /*	fprintf(stderr, "src tailcount %d, dst count %d\n", 
		input->TailCount(), output->Count());  */
    	insize  =input->GetTailPtr(&inptr);
    	assert(insize);
    	outsize = output->GetHeadPtr(&outptr);
	assert(outsize);
    	inrem   = min(insize, todo-done);
    	indone  = proc->process(inptr, 1, inrem, outptr, 1, outsize, &outdone);
    /*	fprintf(stderr, "insize %d, outsize %d, inrem %d, indone %d\n", 
		insize, outsize, inrem, indone);  */
    	output->UpdateHead(outdone);
    	input->UpdateTail(indone);
    	done += indone;
    	if(indone == 0 && outdone == 0)  {
	    go_on = 0;
    	}
    }
    return done;
}

int CircBufSimpleIO(CircularBuffer *input, CircularBuffer *output, 
		    DATAsimpleFn *fn)
{   /* apply the DATAsimpleFn to all Avai() points from input, write 
       result to output.
       Return number of input samples used, may be less than requested. */
    DATA *inptr, *outptr;
    int  totalDone = 0, thistime = -1;

    while(thistime)  {
    	thistime = min(input->GetTailPtr(&inptr), 
		       output->GetHeadPtr(&outptr));
    	(*fn)(inptr, outptr, thistime);
    	output->UpdateHead(thistime);
    	input->UpdateTail(thistime);
    	totalDone += thistime;
    }
    return totalDone;
}

int CircBufTransfer(CircularBuffer *input, CircularBuffer *output, int n)
{   /* just transfer n points from input to output.  Return actual num */
    DATA *inptr, *outptr;
    int  totalDone = 0, thistime = -1, remain = n;

    while(thistime)  {
    	thistime = min(input->GetTailPtr(&inptr), 
		       output->GetHeadPtr(&outptr));
	thistime = min(thistime, remain);
    	DATAcopy(inptr, outptr, thistime);
    	output->UpdateHead(thistime);
    	input->UpdateTail(thistime);
    	totalDone += thistime;
	remain -= thistime;
    }
    return totalDone;
}


#ifdef MAIN	// test code

void test1(void)
{   /* test the circular buffer code */
    CircularBuffer c(10);
    DATA d[10];
    
    c.print();
    d[0] = 0.1;  d[1] = 0.2;  d[2] = 0.3;
    c.write(d, 3);
    c.print();
    c.write(d, 3);
    c.write(d, 3);
    c.write(d, 3);
    c.print();
    int m,n = c.Read(d,1000);
    fprintf(stdout, "Got %d vals:\n", n);
    for(m=0; m<n; ++m)  {
    	fprintf(stdout, "%.1f ", d[m]);
    }
    fprintf(stdout, "\n");
    c.print();
    c.write(d, 3);
    fprintf(stdout, "size %d, count %d, avai %d, tcount %d\n", 
    	    c.Size(), c.Count(), c.Avai(), c.TailCount());
    DATA *e;
    n = c.GetHeadPtr(&e); // return a pointer to data
    fprintf(stdout, "Writing another %d vals\n", n);
    for(m=0;m<n;++m)  {
        e[m] = 0.1+(float)m;
    }
    c.UpdateHead(n);
    c.print();
    n = c.GetTailPtrFrom(&e, 12);
    fprintf(stdout, "Got %d vals:\n", n);
    for(m=0; m<n; ++m)  {
    	fprintf(stdout, "%.1f ", e[m]);
    }
    c.print();
    c.write(d, 3);
    fprintf(stderr, "SetTail(21) returns %d\n", c.SetTail(21));
    n = c.GetTailPtr(&e);
    fprintf(stdout, "Got %d vals:\n", n);
    for(m=0; m<n; ++m)  {
    	fprintf(stdout, "%.1f ", e[m]);
    }
    c.UpdateTail(n);
    c.print();
}

#include <math.h>

#ifndef PI
#define PI (3.1415926535)
#endif /* !PI */

#define SIZE 100
#define BASE 15
#define BASE2 40

void test2(void)
{
    CircularBuffer c1(SIZE), c2(SIZE);
    int i,j;
    DATA *d;
    DATA e[SIZE];
    
    j = c1.GetHeadPtr(&d);
    fprintf(stdout, "got %d cells\n", j);
    for(i=0; i<SIZE; ++i)  {
//    	d[i] = 0.5*(1.0-cos(2*PI*i/(double)j));
    	e[i] = cos(2*PI*i/(double)j);
    }
    // c1.UpdateHead(j);
    c1.write(e, BASE);
    c1.Flush();
//    c1.PushHead();
    c1.write(e, SIZE);
    c1.PushTail();
    c1.print("c1");
    // spuriously offset the base of c2
    c2.UpdateHead(BASE2);
    c2.Flush();
    // can copy?
    DATAcopyClass *dc = new DATAcopyClass();
    int rc = CircBufInputOutput(&c1, &c2, dc);
    fprintf(stderr, "rc is %d\n", rc);
    c1.print();
    c2.print();
    c2.PushTail();
    double ip = CircBufInnerProdFrom(&c1, BASE, &c2, BASE2, SIZE);
    fprintf(stderr, "ip is %f\n", ip);
    c1.print("before");
    c1.PopTail();
    c2.PopTail();
    c1.print("after");
    ip = CircBufInnerProd(&c1, &c2, SIZE);
    fprintf(stderr, "ip is %f\n", ip);
}

void main(int argc, char **argv)
{
    test2();
}
#endif /* MAIN */    

