// -*- C++ -*-

// Data.h
//
// Common declaration of Data format used in aprl
// dpwe@media.mit.edu 1995feb02
//

#ifndef _DATA_H
#define _DATA_H

typedef float DATA;

#include <string.h> // for memcpy

inline void DATAclear(DATA *dest, int len)
{   memset(dest, '\0', len*sizeof(DATA));	}

inline void DATAset(DATA *dset, DATA val, int len)
{   int i; for(i=0; i<len; ++i) { *dset++ = val; } 	}

inline void DATAcopy(DATA *src, DATA *dest, int len)
{   memcpy(dest, src, len*sizeof(DATA));    }

inline void DATAscale(DATA *src, DATA *dest, int len, double factor)
{   int i=len;    while(i--)  { *src++ = factor * *dest++; }	}

inline void DATAstrideCopy(const DATA *src, int sinc,
	      DATA *dst, int dinc, int len)
{
    if(dst > src)  {	   // start at end to allow overlapped copy forwards
	src += sinc*len;	dst += dinc*len;
	while(len--)  {
	    src -= sinc;	dst -= dinc;
	    *dst = *src;
	}
    } else {		   // moving backwards or something - copy any old how
	while(len--)  {
	    *dst = *src;
	    src += sinc;	dst += dinc;
	}
    }
}

inline double DATAinnerProduct(const DATA *s1, int inc1,
			       const DATA *s2, int inc2, int npts)
{   /* calculate inner product between two DATA vectors */
    int i;
    double rslt = 0.0;
    
    for(i=0; i<npts; ++i)  {
	rslt += *s1 * *s2;
	s1 += inc1;	s2 += inc2;
    }
    return rslt;
}

inline int max(int a, int b)
{
    if(a>b)  return a; else return b;
}

inline int min(int a, int b)
{
    if(a<b)  return a; else return b;
}

class DATAprocess {	// Defines a function to process data sequentially
    // but allows encapsulation of state and
    // constructor, destructor (better than typedef)
public:
    DATAprocess(void)	{};
    virtual ~DATAprocess(void)	{};
    // actually process data between input and output buffers, observing
    //	interleave.  Return number of input data consumed, also
    //	write the number of values written to the output in poutdone
    virtual int process(DATA *input, int istride, int niItems,
			DATA *output, int ostride, int noItems,
			int *poutdone) = 0;
    virtual void reset(void)	    {};	// clear internal state
};

// if the DATAprocess class is too much for you, try this fn instead:
// just to copy n points from in to out (memoryless modification)
typedef void DATAsimpleFn(DATA *in, DATA *out, int n);

// simple degenerate DATAprocess may be useful now and again
class DATAcopyClass : public DATAprocess {
public:
    DATAcopyClass(void) 	{};
    ~DATAcopyClass(void)	{};
    int process(DATA *input, int istride, int niItems, 
		DATA *output, int ostride, int noItems, 
		int *poutdone)
	{  int i,todo = min(niItems, noItems);
	   for(i=0; i<todo; ++i) 
	   	{ *output = *input; input += istride; output += ostride; }
	   *poutdone = todo; return todo; }
};

class DATAcombiner {	// combines two blocks of DATA
public:
    DATAcombiner()		{};
    virtual ~DATAcombiner()		{};
    virtual int combine(DATA *input1, int istep1, int *ni1Items, 
			DATA *input2, int istep2, int *ni2Items, 
			DATA *output, int ostep, int *noItems) = 0;
    // combine the data from inputs 1 and 2, writing to 
    // output.  On entry ni1Items, ni2Items contain the number 
    // of input data in each, and noItems has the number of 
    // free output positions.  On output, <combine> should
    // modify these to reflect how much processing was actually 
    // done.
    // istep can be zero, indicating a single input value
};

#endif /* _DATA_H */
