// -*- C++ -*-

// DataCombiner.h
//
// Class that takes several (two?) streams of data and combines them 
// to a single input.  In the first instance, we'll force them all to
// be at the same sampling rate, but eventually I'd like to support
// constants and maybe resampling as well.
// for Audio Pattern Recognition Language (aprl)
// dpwe@media.mit.edu 1995feb12
//

#ifndef _DATACOMBINER_H
#define _DATACOMBINER_H

#include "DataPipe.h"

// 
// I want the DataStreamCombiner to multiply-inherit from *two* 
// instances of a datasink - but I can't figure out the syntax 
// (if indeed it's possible) and I can't find a C++ book right 
// now, so I'll define a subtlely different second version of 
// DataSink so I can inherit from it to.

class DataSink1 : public DataSink {
public:
    DataSink1(DataSource *in=0)	: DataSink(in)  	
        { };
    ~DataSink1(void)
        { };
};

class DataSink2 : public DataSink {
public:
    DataSink2(DataSource *in=0)	: DataSink(in)  	
        { };
    ~DataSink2(void)
        { };
};

class DataStreamCombiner: public DataSource,public DataSink1,public DataSink2 {
    // a data source with two data sinks
private:
//    NullDataSink *sink1;
//    NullDataSink *sink2;
    void checkit(void);
public:
    DataStreamCombiner(DataSource *in1 = 0, DataSource *in2 = 0)
	: DataSource()
	    , DataSink1(in1), DataSink2(in2)
	{ // sink1 = new NullDataSink(in1); sink2 = new NullDataSink(in2); 
          checkit(); }
    ~DataStreamCombiner(void)
        { // delete sink1; delete sink2;  
	}
    void SetInput1(DataSource *in)
        { // sink1->SetInput(in);
	    DataSink1::SetInput(in);
          checkit(); }
    void SetInput2(DataSource *in)
        { // sink2->SetInput(in);
	    DataSink2::SetInput(in);
          checkit(); }
    void SetInput(DataSource *in)	// Input is synonym for Input1
        { SetInput1(in); }
    DataSource *Input1(void) const  
        { // return sink1->Input(); 
	    return DataSink1::Input();
        }
    DataSource *Input2(void) const  
        { // return sink2->Input(); 
	    return DataSink2::Input();
        }
    DataSource *Input(void) const  	// Input is synonym for Input1
        { return Input1(); }
    DataSource *Input(int n) const
        { DataSource *rslt = NULL;
	  if(n==1)	rslt = Input1();
	  if(n==2)	rslt = Input2();
	  return(rslt); }
    int ThereIsMore(void);
};

class DataProcCombiner : public DataStreamCombiner {  // combine by a proc
protected:
    DATAcombiner	*combFn;
public:
    DataProcCombiner(DATAcombiner *cfn, DataSource *in1=0, DataSource *in2=0)
        : DataStreamCombiner(in1, in2), combFn(cfn)
	{ };
    ~DataProcCombiner(void)
	{ };
    int Update(double /* time */);
};

class DATAadder : public DATAcombiner {	// adds two data, of course
private:
    double k1;	// scale factors
    double k2;
public:
    DATAadder(double s1=1.0, double s2=1.0) : k1(s1), k2(s2)
	{ };
    double& scale1(void)	{ return k1; }
    double& scale2(void)	{ return k2; }
    int combine(DATA *in1, int step1, int *ni1,
		DATA *in2, int step2, int *ni2,
		DATA *out, int stepo, int *no);
};

class DataStreamAdder : public DataProcCombiner {
public:
    DataStreamAdder(DataSource *in1=0, DataSource *in2=0, 
		    double s1=1.0, double s2=1.0)
	: DataProcCombiner(0, in1, in2)
	{ combFn = new DATAadder(s1, s2); };
    ~DataStreamAdder(void)
        { delete combFn; }
    void SetScales(double s1, double s2)
        { ((DATAadder *)combFn)->scale1() = s1;
	  ((DATAadder *)combFn)->scale2() = s2; }
};

class DATAmulter : public DATAcombiner {	// mults two data, of course
private:
    double k1;	// scale factors
    double k2;
public:
    DATAmulter(double s1=1.0, double s2=1.0) : k1(s1), k2(s2)
	{ };
    double& scale1(void)	{ return k1; }
    double& scale2(void)	{ return k2; }
    int combine(DATA *in1, int step1, int *ni1,
		DATA *in2, int step2, int *ni2,
		DATA *out, int stepo, int *no);
};

class DataStreamMulter : public DataProcCombiner {
public:
    DataStreamMulter(DataSource *in1=0, DataSource *in2=0, 
		    double s1=1.0, double s2=1.0)
	: DataProcCombiner(0, in1, in2)
	{ combFn = new DATAmulter(s1, s2); };
    ~DataStreamMulter(void)
        { delete combFn; }
    void SetScales(double s1, double s2)
        { ((DATAmulter *)combFn)->scale1() = s1;
	  ((DATAmulter *)combFn)->scale2() = s2; }
};

#endif  /* _DATACOMBINER_H */

