// -*- C++ -*-

// DecimPipe.cc
//
// DataPipe functions that do decimation and interpolation
// 
// for Audio Pattern Recognition Language (aprl)
// dpwe@media.mit.edu 1995feb02
//

#include <stdio.h>
#include <assert.h>

#include "DecimPipe.h"

int DecimPipe::Update(double /* time */)
{  /* Transfer data, do decimation */
    int thisTime, totalDone = 0;
    if(ThereIsMore())  {
	PushHead();
	Input()->PopTail();	   
	int remain = Input()->Avai();
	DATA *i, *o;
	int  ni, no, noDone;
	int  j;
	while(remain)  {
	    ni = Input()->GetTailPtr(&i);
	    no = GetHeadPtr(&o);
	    /* no output slots can accommodate no*decim input values, 
	       plus a run-in of (decim-base)%decim */
	    assert( thisTime = min( decim*no+((decim-base)%decim),
				   min(remain, ni) )) ;
	    noDone = 0;
	    for(j = base; j < thisTime; j += decim)  {
		*o++ = i[j]; ++noDone;
	    }
	    UpdateHead(noDone);
	    Input()->UpdateTail(thisTime);
	    base = (base + thisTime)%decim;
	    remain -= thisTime;
	    totalDone += thisTime;
	}
    }
    return totalDone;
}

int LinterpPipe::Update(double /* time */)
{   /* do linear interpolation into output */
    int thisTime, totalDone = 0;
    if(ThereIsMore()) {
	PushHead();
	Input()->PopTail();	   
	int remain = Input()->Avai();
	DATA *i, *o;
	int  ni, no, niDone;
	int  j;
	double inc;
	while(remain)  {
	    ni = Input()->GetTailPtr(&i);
	    no = GetHeadPtr(&o);
	    /* Each input will generate <interp> output vals, so 
	       the max number of inputs we can consume is
	       (no - ((interp-base)%interp))+interp-1)/interp */
	    thisTime=min((no-((interp-base)%interp)+interp-1)/interp,
			 min(remain, ni));
	    /* x input vals corresponds to 
	       x*(interp-1)+1 + ((interp-base)%interp) op vals 
	       (-1)+1 because two points allows you to interpolate interp+1 vals */
	    thisTime = min(((interp-base)%interp) + interp*(min(ni, remain)-1)+1, 
			   no);
	    inc = (*i-lastval)/interp;
	    fprintf(stderr, "LP: %d in to %d out (base %d) - take %d\n", 
		    ni, no, base, thisTime);
	    niDone = 0;
	    for(j = base; j < thisTime+base; ++j)  {
		if( (j%interp)==0 )  {
		    lastval = *i++; ++niDone;
		    inc = (*i-lastval)/interp;
		    *o++ = lastval;
		} else {
		    *o++ = lastval + inc * (j%interp);
		}
	    }
	    UpdateHead(thisTime);
	    base = (thisTime+base)%interp;
	    Input()->UpdateTail(niDone);
	    remain -= niDone;
	    totalDone += niDone;
	    assert(thisTime > 0);
	}
    }
    return totalDone;
}

#ifdef MAIN //---------------------------- test code ----------------------

extern "C" {
char *programName;
};

#include "AudioIO.h"
#include "FilterPipe.h"
#include "SoundfileIO.h"
#include "MiscPipes.h"

void testLinDecim(char *infile)
{
    SoundfileSrc  ss(infile);
    SquarePipe	  sp(&ss);
    LowpassFilterPipe lp(500.0, 8, LPF_2NDO, &sp);
    RootPipe      rp(&lp);
    DecimPipe     dp(100, &rp);
    LinterpPipe	  li(100, &dp);
    SoundfileSink sk("tmp.aif", &li);

    Scheduler s;

    s.Enschedule(&ss);
    s.Enschedule(&sp);
    s.Enschedule(&lp);
    s.Enschedule(&rp);
    s.Enschedule(&dp);
    s.Enschedule(&li);
    s.Enschedule(&sk);

    while(ss.ThereIsMore())  {
	s.Update(0.0);
    }
}

void testRms(char *infile)
{
    SoundfileSrc  ss(infile);
    RmsPipe	  li(100, &ss);
    SoundfileSink sk("tmp.aif", &li);

    Scheduler s;

    s.Enschedule(&ss);
    s.Enschedule(&li);
    s.Enschedule(&sk);

    while(ss.ThereIsMore())  {
	s.Update(0.0);
    }
}

void main(int argc, char **argv)
{
    programName = argv[0];
    if(argc < 2)  {
	fprintf(stderr, "Usage: %s infile\n", programName);
	exit(-1);
    }
//    testLinDecim(argv[1]);
    testRms(argv[1]);
}

#endif /* MAIN */
