// -*- C++ -*-

// EventPipe.h
//
// Classes for sources and sinks of data
// Pipe that does one-shotting on a thresholded signal
// *and* makes a callback on each leading edge of its output
//
// for Audio Pattern Recognition Language (aprl)
// dpwe@media.mit.edu 1995feb02
//

#include <stdio.h>
#include <assert.h>
#include "EventPipe.h"

int EventPipe::process(DATA *in, DATA *out, int n, double baseTime, double sr)
{   /* just the guts of the processing - do n samples at in and out */
    int i;
    double d, time;
    int remones;
    int instep = 1, outstep = 1;

    remones = max(0, (int)(sr * (lastTrigTime + pulseTime - baseTime)));
    for(i=0; i<n; ++i)  {
	d = *in;
	if(lastVal <=0 && d > 0)  {	// rising edge in input
	    time = baseTime+(i/sr);
	    if(time > lastTrigTime + holdoffTime)  {
		// If the output is currently low, actually trigger
		if((time > lastTrigTime + pulseTime) && cbFn)  {
		 //   fprintf(stderr, "Doing callback\n");
		    (*cbFn)(time, cbFnData);
		}
		lastTrigTime = time;
		remones = (int)(sr*pulseTime);
	    }
	}
	if(remones)  {
	    *out = 1.0;
	    --remones;
	} else {
	    *out = 0.0;
	}
	in += instep;
	out += outstep;
	lastVal = d;
    }
    return n;
}		

int EventPipe::Update(double time)
{   /* pick out rising edges in input, perhaps call back */
    int totalDone = 0;
    if(ThereIsMore())  {
	PushHead();
	Input()->PopTail();
	int thistime = -1;
	double sr = SampleRate();
	DATA *inptr, *outptr;
	while(thistime)  {
	    thistime = min(Input()->GetTailPtr(&inptr), 
			   GetHeadPtr(&outptr));
	    process(inptr, outptr, thistime, time+(totalDone/sr), sr);
	    UpdateHead(thistime);
	    Input()->UpdateTail(thistime);
	    totalDone += thistime;
	}
    }
    return totalDone;
}
