// -*- C++ -*-

// FilterPipe.h
//
// Classes for sources and sinks of data
// as wrappers around digital filters.
// for Audio Pattern Recognition Language (aprl)
// dpwe@media.mit.edu 1995feb02
//

#include "DataPipe.h"

#include "Filter.h"

class FilterPipe : public DataPipe  {
protected:
    Filter *filt;
public:
    FilterPipe(Filter *f, DataSource *in) : filt(f), DataPipe(in, 1.0)
        { };
    virtual ~FilterPipe(void)
        { };
    int Update(double /* time */)
        {   /* run the filter on our data */
	    int done = 0;
	    if(ThereIsMore()) {
		PushHead();
		assert(Input()->PopTail()>=0);
		int thistime, todo = Input()->Avai();
		int isize, osize;
		DATA *i, *o;
		while(todo)  {
		    isize = Input()->GetTailPtr(&i);
		    osize = GetHeadPtr(&o);
		    assert(thistime = min(todo, min(isize,osize)));
		    filt->run(i, o, thistime);  // was todo!!!
		    todo -= thistime;
		    UpdateHead(thistime);
		    Input()->UpdateTail(thistime);
		    done += thistime;
		}
	    }
	    return done;
	};
};

#include <iostream.h>

class BandpassFilterPipe : public FilterPipe {
private:
    double ctrFrq;
    double Qfct;
    int    type;
    void init(void)
        { filt = new BandpassFilter(Input()->SampleRate(), ctrFrq, Qfct, type); }
public:
    BandpassFilterPipe(double cf, double q, int t, DataSource *in = 0)
	: ctrFrq(cf), Qfct(q), type(t), FilterPipe(0, in)
	{ };
    virtual ~BandpassFilterPipe(void) 
        { if(filt)	delete filt;	};
    int Update(double t)
        { int rc = 0;
	  if(ThereIsMore()) {
	      if(!filt)	init();
	      rc = FilterPipe::Update(t); }
	  return rc; }
    void ChangeFilt(double cf, double q)
        { ((BandpassFilter *)filt)->ChangeFilter(cf, q);
          ctrFrq = cf;  Qfct = q; }
    double CtrFrq(void) const  { return ctrFrq; }
    double Qfactor(void) const { return Qfct;   }
};

class LowpassFilterPipe : public FilterPipe {
private:
    double cutFrq;
    double Qfct;
    int    type;
    void init(void)
        { filt = new LowpassFilter(Input()->SampleRate(), cutFrq, Qfct, type);}
public:
    LowpassFilterPipe(double cf, double q, int t, DataSource *in = 0)
	: cutFrq(cf), Qfct(q), type(t), FilterPipe(0, in)
	{ };
    virtual ~LowpassFilterPipe(void) 
        { if(filt)	delete filt;	};
    int Update(double t)
        { int rc = 0;
	  if(ThereIsMore()) {
	      if(!filt)	init();
	      rc = FilterPipe::Update(t); }
	  return rc; }
    void ChangeFilt(double cf, double q)
        { ((LowpassFilter *)filt)->ChangeFilter(cf, q);
          cutFrq = cf;  Qfct = q; }
    double CutoffFrq(void) const  { return cutFrq; }
    double Qfactor(void)   const  { return Qfct;   }
};

class HighpassFilterPipe : public FilterPipe {
private:
    double cutFrq;
    double Qfct;
    int    type;
    void init(void)
        { filt = new HighpassFilter(Input()->SampleRate(), cutFrq, Qfct, type);}
public:
    HighpassFilterPipe(double cf, double q, int t, DataSource *in = 0)
      : cutFrq(cf), Qfct(q), type(t), FilterPipe(0, in)
      { };
    virtual ~HighpassFilterPipe(void) 
        { if(filt)    delete filt;    };
    int Update(double t)
        { int rc = 0;
        if(ThereIsMore()) {
            if(!filt) init();
            rc = FilterPipe::Update(t); }
        return rc; }
    void ChangeFilt(double cf, double q)
        { ((HighpassFilter *)filt)->ChangeFilter(cf, q);
          cutFrq = cf;  Qfct = q; }
    double CutoffFrq(void) const  { return cutFrq; }
    double Qfactor(void)   const  { return Qfct;   }
};

