// -*- C++ -*-

// PtPipe.cc
//
// Pipe wrapper for billg's pitch tracker
//
// for Audio Pattern Recognition Language (aprl)
// dpwe@media.mit.edu 1995feb25
//
static char rcsid[] = "$Header: /n/crab/da/dpwe/src/aprl/PtPipe.cc,v 1.1 1996/11/18 18:42:29 dpwe Exp $ @media.mit.edu Machine Listening Group";

#include <stdio.h>
#include <assert.h>

#include "PtPipe.h"

int PtPipe::process(DATA *in, DATA *out, int* pinn, int *pmaxout, 
		    double fudge, double ioratio)
{   /* just the guts of the processing - do n samples at in */
    int i, n=*pinn, donein = 0;
    double d;
    int instep = 1, outstep = 1, doneout = 0;
    int stop = 0;

    for(i=0; !stop && i<n; ++i)  {
	d = *in;
	proc_samp((int)(d*32767));

	while( !stop && (fudge + i*ioratio) >= doneout )  {
	    if(doneout == *pmaxout)  {
		/* can't take this one */
		--i;
		--donein;	/* to undo effect of increment below */
		in -= instep;
		stop = 1;
	    } else {
		*out = get_pitch();
		out += outstep;
		++doneout;
	    }
	}
	in += instep;
	++donein;
    }
//    fprintf(stderr, "avai in %d, avai out %d, done in %d, done out %d, ra %f fu %f\n",
//	    *pinn, *pmaxout, donein, doneout, ioratio, fudge);
    *pmaxout = doneout;
    *pinn = donein;
    return doneout;
}		

int PtPipe::Update(double /* time */ )
{   /* pick out rising edges in input, perhaps call back */
    int totalDone = 0;
    if(ThereIsMore())  {
	PushHead();
	Input()->PopTail();
	int thistimein = -1, thistimeout;
	double sr = Input()->SampleRate(), osr = opSampleRate, ioratio;
	if(osr)	ioratio  = osr/sr;
	else	ioratio  = 1.0;
	DATA *inptr, *outptr;
	while(thistimein)  {
	    thistimein = Input()->GetTailPtr(&inptr);
	    thistimeout = GetHeadPtr(&outptr);	
	    process(inptr, outptr, &thistimein, &thistimeout,
		    fudge, ioratio);
	    fudge = (fudge + (thistimein)*ioratio) - thistimeout;
	    assert(fudge < 1.0);
	    UpdateHead(thistimeout);
	    Input()->UpdateTail(thistimein);
	    totalDone += thistimein;
	}
    }
    return totalDone;
}

#ifdef MAIN /* --------------------------------- test code ------------- */

#include "Scheduler.h"
#include "SoundfileIO.h"
#include "MiscPipes.h"

char *programName;

void main(int argc, char **argv) 
{
    programName = argv[0];
    
    Scheduler	s;

//    SoundfileSrc sfs("../../sound/goodbad.aif", 0.2);
//    sfs.SetLoop(1);

    RandSrc sfs(5.0, 3);

    s.Enschedule(&sfs);

    PtPipe pt(3.0, &sfs);

    s.Enschedule(&pt);

    s.Update(0.17);
    s.Update(0.17);
    s.Update(0.17);
    s.Update(0.17);

    s.Update(0.17);
    s.Update(0.17);
    s.Update(0.17);
    s.Update(0.17);
}



#endif /* MAIN */
