#!./aprlwish -f
#
# binau.tcl
#
# Voice monitor, to provide real-time trace of voice status.
# Based on oscope.tcl
#
# dpwe@media.mit.edu 1995may01
#

set tcl_interactive 1
wm minsize . 1 1

scheduler s -docmd 1 -timestep 0.033 -command { .fill config -val [expr "[as filled]<500?500:[as filled]"]; \
	                        after 10 {%s step} }
# -command {.tl config -maxVal %t -minVal [expr "%t-$disptime"] ; \
#           after 10 { %s step }}

# Configure the overall samplerate
set rate 16000
# decimate full-band audio by 4 before displaying - efficiency
set decrate 4
set bufsize $rate
# initialize the global list of display widgets
set displist {}

# Configure the height of each wavedisplay
set wheight 60
set wwidth  500

# Set the offset and scale for the two db display graphs
set dbofs -1
# zero at the bottom of the screen
set dbscale [expr "1.0/25.0"]
# -25 to +25 i.e. 50 dB range

set ch 2

audiosrc as -sampleRate $rate -bufferTime [expr "$bufsize./$rate"] -channel $ch
hpfilter hp -cut 10

audiosink . -sampleRate $rate -channels $ch

decimer . -factor 2

# We're looking for claps - choose a high frequency band
#bpfilter  .     -centerFreq 4000
rms       .     -sampleRate 100
dB        d0    -ref 0.01
# Have one display of the smoothed energy in dB
frame .f
wavesink .f.ws1  -scale 0.05 -height 80 \
	-relief sunken -padX 4 -padY 4

# pack .f.ws1 -side top -fill both -expand 1

# The 'clap' detector samples the energy E(t) at three times (i.e. the 
#   current signal and two delayed versions) and calculates the following 
#   expression:  
#        ( E(t) < -20dB ) & ( E(t-0.1) > 0 dB ) & ( E(t-0.15) < -20dB )
delay     d1    -delayTime 0.1
delay     d2    -delayTime 0.05
# form the logical not of the non-delayed signal, E(t)
# 'nthresh' outputs 1 if the input is *less* than the reference, else 0
nthresh   n0    -input d0 -ref  -20
# threshold E(t-0.1); 'thresh' returns 1 if the input is *above* the reference
thresh    c1    -input d1 -ref    0
# `And' them (multiply)  m1 = delayed(d0) & not(d0)
multer    m1    -input n0 -in2   c1
# multiply in the third term
nthresh   n2    -input d2 -ref  -20
multer    m2    -input n2 -in2   m1

# 'eventer' objects are one-shot flip flops that emit a pulse of
#  length pulseTime every time they see a rising edge in their inputs, 
#  except after triggering, they ignore the input for holdoffTime.
#  Also, whenever their output goes from low to high, they execute 
#  the Tcl expression set by their 'command' option, with the time of 
#  the actual event appended (so this one executes, e.g. "clap 0.151")
eventer   e1    -input n0 -pulseTime 0.05 -holdoffTime 0.5  -command "puts clap"
# Track the output of the one-shot
wavesink  .f.ws2  -input n0 -height 40 -scale 1.8 -offset -1  \
	-relief sunken -padX 4 -padY 4

pack .f .f.ws1 .f.ws2 -padx 2 -pady 2 -fill x
# pack .f .f.ws1 -padx 2 -pady 2 -fill x

## extra vu meter to show input buffer fillation
# max scale from known input buffer size
vumeter .fill -scale [expr "1.0/$bufsize"] -relief flat -height 10 -border 3 \
	-padX 0 -padY 0 -cells 100 -nyellow 0 -ngreen 100 -green red \
	-frame 0 -fg #cdb79e
pack .fill -side bottom -expand yes -fill x

# buttons
frame .bf 
button .bf.pause -text "Pause" -command { if { [lindex [.bf.pause config -text] 4]=="Pause" } { .bf.pause config -text "Resume"; s config -docmd 0 } else { .bf.pause config -text "Pause"; s config -docmd 1; s step }}
button .bf.quit  -text "Quit" -command "exit"

pack .bf.pause .bf.quit -side left -expand y
pack .bf -side bottom -fill both -padx 10 -pady 10

s step

