#!./aprlwish -f
#
# graphic.tcl
#
# Do a graphic EQ in aprl using the VU meter object
#
# dpwe@media.mit.edu 1995feb23
#

set tcl_interactive 1
#wm minsize . 1 1

# initialize the scheduler
scheduler s -timestep 0.1

# standard input process
#audiosrc as -sampleRate 22050
audiosrc as -sampleRate 32000 -channels 2

audiosink ak -sampleRate 32000 -channels 2

decimer as0 -factor 2

# standard execution code
proc cont {} {
    global flag frqs
    s step
    # Update the VU meters
    foreach f $frqs {
	.f$f.vu config -val [dB$f val]
    }
    # Update input buffer fillmeter
    .fill config -val [as filled]
    # Quit if we've run out of input (for timing tests anyway)
    if { ![as more] } { exit }
    if { $flag } {
	after 10 { cont }
    }
}
# 'run' starts execution by setting the flag and starting the chain 
# of 'cont's
proc run {} {
    global flag
    set flag 1
    cont
}
# 'stop' pauses the processing by clearing the flag, so the next time 
#  'cont' executes, it won't setup the follow-on call
proc stop {} {
    global flag
    set flag 0
}

## extra vu meter to show input buffer fillation
# max scale as 10,000, since it's a 16000 sample buffer
vumeter .fill -scale 0.0001 -relief flat -height 10 -border 3 \
	-padX 0 -padY 0 -cells 100 -nyellow 0 -ngreen 100 -green red \
	-frame 0 -fg #cdb79e
pack .fill -side bottom -expand yes -fill x

## standard quit button at bottom of frame
# Make the buttons and display them
frame .bf 
button .bf.b1 -text "Quit"  -command {exit}
pack .bf -side bottom
pack .bf.b1  -side left -expand yes -padx 10 -pady 10


################# define processing

## Do a vertical scale
frame .tlf
label .tlf.l -text "dB"
timeline .tlf.t -isVert 1 -relief sunken -minVal 0 -maxVal -50 \
	-height 300 -width 25
pack .tlf.l .tlf.t -side bottom
pack .tlf -side left -fill y -expand y -padx 3 -pady 3 
set frqs { 100 200 400 800 1600 3200 6400}

proc buildforfrq { frq as } {
    # Build a VU meter element for the specified frequency
    bpfilter bpf$frq  -input $as -centerFreq $frq
    rms      rms$frq  -sampleRate 20
    lpfilter lp$frq   -cutoff 10
    dB       dB$frq   -ref 0.1
    # 50 dB scale for vu meters
    frame .f$frq
    label .f$frq.l -text "$frq"
    vumeter  .f$frq.vu -width 32 -height 300 -orient v -relief sunken \
	    -scale 0.02 -offset 1 
    pack .f$frq.l .f$frq.vu -side bottom
    pack .f$frq -side left -fill y -expand y -padx 3 -pady 3
}

foreach f $frqs { buildforfrq $f as0 }

run
