#!./aprlwish -f
#
# vxmon.tcl
#
# Voice monitor, to provide real-time trace of voice status.
# Based on oscope.tcl
#
# dpwe@media.mit.edu 1995may01
#

set tcl_interactive 1
wm minsize . 1 1

scheduler s -docmd 1 -timestep 0.033 -command { .fill config -val [expr "[as filled]<500?500:[as filled]"]; \
	                        after 10 {%s step} }
# -command {.tl config -maxVal %t -minVal [expr "%t-$disptime"] ; \
#           after 10 { %s step }}

# Configure the overall samplerate
set rate 16000
# decimate full-band audio by 4 before displaying - efficiency
set decrate 4
set bufsize $rate
# initialize the global list of display widgets
set displist {}

# Configure the height of each wavedisplay
set wheight 60
set wwidth  500

# Set the offset and scale for the two db display graphs
set dbofs -1
# zero at the bottom of the screen
set dbscale [expr "1.0/25.0"]
# -25 to +25 i.e. 50 dB range

audiosrc as -sampleRate $rate -bufferTime [expr "$bufsize./$rate"]
hpfilter hp -cut 10

# decimate without lowpass filtering, just for display
decimer  dc  -factor  $decrate
wavesink .ws -height $wheight -width $wwidth
# register wavesink with list that gets its buflen updated
lappend displist [list .ws [expr "$rate/$decrate"]]
timeline .tl -isVertical 0 -tixTopLeft 1 -width $wwidth

lpfilter lb   -cutoff 1000   -input hp
rms      .    -sampleRate 100
dB       .    -ref 0.01
wavesink .lb  -offset $dbofs -scale $dbscale  -height $wheight -width $wwidth
lappend  displist  {.lb 100}

hpfilter hb  -cutoff 1000    -input hp
rms      .   -sampleRate 100
dB       .   -ref 0.01
wavesink .hb  -offset $dbofs -scale $dbscale -height $wheight -width $wwidth
lappend  displist  {.hb 100}

# function to bind to scale widget to adjust scale const in object
proc setSrcScale {src max scale} {
    set val [expr "$scale/$max"]
    .asf.val config -text $val
    $src config -scale $val
}

# from is top of scale, to is bottom
frame .asf -borderwidth 2 -relief groove
scale .asf.sc -to 0 -from 100  -showvalue 0 -command "setSrcScale as 10.0 "
label .asf.label -text "Gain"
label .asf.val -text "0"

proc newTimeLen {time} {
    # adjust display buffer sizes of displays to hold exactly $time secs
    # displist holds a list of the display widgets and their srates
    global displist
    foreach d $displist  {
        set wdg   [lindex $d 0]
        set srate [lindex $d 1]
        $wdg config -buflen [expr "int($srate*$time)"]
    }
}

proc setTimeScale {wsw max scale} {
    global rate
    set scale [expr "int($scale - $max)"]
    set range [expr "$scale/3"]
    set pten  [expr "10.0*pow(10.0,$range)"]
    set arg   [lindex {1.0 2.0 5.0} [expr "$scale - 3*$range"]]
        
#    set disptime [expr "exp(($scale-$max)*0.767528)"]
    set disptime [expr "$arg * $pten"]
    newTimeLen $disptime
    .tsf.val config -text $disptime
#    .tl config -minVal 0 -maxVal $disptime
    .tl config -minVal -$disptime -maxVal 0
}

# from is top of scale, to is bottom
frame .tsf -borderwidth 2 -relief groove
scale .tsf.ts -to 0 -from 10  -showvalue 0 -command "setTimeScale .ws 10.0 "
label .tsf.label -text "Time"
label .tsf.val -text "0"

## extra vu meter to show input buffer fillation
# max scale from known input buffer size
vumeter .fill -scale [expr "1.0/$bufsize"] -relief flat -height 10 -border 3 \
	-padX 0 -padY 0 -cells 100 -nyellow 0 -ngreen 100 -green red \
	-frame 0 -fg #cdb79e
pack .fill -side bottom -expand yes -fill x

# buttons
frame .bf 
button .bf.pause -text "Pause" -command { if { [lindex [.bf.pause config -text] 4]=="Pause" } { .bf.pause config -text "Resume"; s config -docmd 0 } else { .bf.pause config -text "Pause"; s config -docmd 1; s step }}
button .bf.quit  -text "Quit" -command "exit"

pack .bf.pause .bf.quit -side left -expand y
pack .bf -side bottom -fill both -padx 10 -pady 10

pack .asf.label .tsf.label -side top -fill both -padx 5 -pady 5
pack .asf.val .tsf.val -side bottom -fill both -padx 5 -pady 5
pack .asf.sc .tsf.ts       -side top -fill both -expand y -padx 5 -pady 5
pack .asf .tsf -side left -fill both  -padx 5 -pady 5
pack .tl -side bottom -fill both
pack .hb .lb .ws -side top -fill both -expand y

.tsf.ts set 6
.asf.sc set 80

s step

