// $Header: /n/basmati/da/bedk/speech/src/frontend/RCS/Nonlinearity.h,v 1.2 1998/05/29 18:21:00 bedk Exp bedk $
//
// $Log: Nonlinearity.h,v $
// Revision 1.2  1998/05/29 18:21:00  bedk
// Added sqrt compression
//
// Revision 1.1  1998/02/07 00:18:47  bedk
// Initial revision
//
//
// Simple memoryless nonlinearity

#ifndef Nonlinearity_h_INCLUDED
#define Nonlinearity_h_INCLUDED

#include "Blocker.h"
#include "Matrix.h"
#include "Vector.h"

class Nonlinearity : public Blocker
{
public:
  enum NonlMethod
  {
    HALFWAVE = 0,
    SQRT,
    POW,
    SQR
  };

  Nonlinearity(const size_t a_ioblocking,
               const size_t a_nc,
               const NonlMethod a_method)
    : ioblocking(a_ioblocking),
      nc(a_nc),
      method(a_method),
      exponent(0.),
      out_buffer(new float[nc * ioblocking])
  {};

  Nonlinearity(const size_t a_ioblocking,
               const size_t a_nc,
               const NonlMethod a_method,
               const float a_exponent)
    : ioblocking(a_ioblocking),
      nc(a_nc),
      method(a_method),
      exponent((double) a_exponent),
      out_buffer(new float[nc * ioblocking])
  {};

  ~Nonlinearity() { delete [] out_buffer; };

  const CMatrix processBlock(const CMatrix in,
			     bool& lastTime);

private:
  size_t ioblocking;            // max. length input we'll ever see
  
  size_t nc;                    // number of input channels

  const NonlMethod method;      // type of nonlinearity we are

  const double exponent;        // exponent for power nonlinearity

  float* out_buffer;            // output buffer
};

#endif // Nonlinearity_h_INCLUDED
