// $Header$
//
// $Log$
//
// Take in data in the online format --- binary shorts, with 0x8000
// marking the end of each utterance.

//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "OnlineSpeechInput.h"

OnlineSpeechInput::OnlineSpeechInput(const size_t blk,
                                     const char* const fileName)
  : IOBlocking(blk)
{
  if (fileName == NULL)
    {
      fprintf(stderr,"NULL file name, exiting.\n");
      exit(1);
    }
  else if (!strcmp(fileName,"-"))
    {
      file = stdin;
      stin = true;
    }
  else if ((file = fopen(fileName,"r")) == NULL)
    {
      fprintf(stderr,"Can't open file (%s), exiting.\n",fileName);
      exit(1);    
      stin = false;
    }
  result = new Vector(IOBlocking);
  buffer = new short[IOBlocking];
}

OnlineSpeechInput::~OnlineSpeechInput()
{
  if (!stin) { fclose(file); }
  delete(result);
  delete(buffer);
}

const CVector
OnlineSpeechInput::processBlock(bool& EOUtt, bool& EOProc, bool& EOFile)
{
  size_t n;
  size_t nread = 0;
  short* p = buffer;
  short* endp;
  float* dest;

  if (EOUtt)
    { 
      EOProc = true;
      return CVector(0, *result);
    }

  while (nread < IOBlocking)
    {
      n = fread(p, sizeof(short), 1, file);
#ifndef WORDS_BIGENDIAN
      *p = ((*p & 0x00FF) << 8) | (0x00FF & (*p >> 8)); 
#endif /* WORDS_BIGENDIAN */
      if (n == 0)
        {
          if (feof(file))
            {
              EOUtt = true;
              EOProc = true;
              EOFile = true;
              break;
            }
          else
            {
              fprintf(stderr, "Error reading on-line input.\n");
              exit(1);
            }
        }
      else if ((unsigned short) *p == 0x8000)
        {
          EOUtt = true;
          EOProc = true;
          break;
        }
      p++;
      nread++;
    }

  p = buffer;
  endp = buffer + nread;
  dest = result->data;
  while (p != endp) { *dest++ = (float) *p++; }

  return CVector(nread, *result);
}


#ifdef TEST 

#define B 1000

main(int argc, char*argv[])
{
  size_t i;
  if (argc < 2) {
    fprintf(stderr,"Usage: %s filename\n",argv[0]);
    exit(1);
  }
  
  OnlineSpeechInput rsi(B,argv[1]);
  bool lastTime = false;

  while (!lastTime) {
    const CVector res = rsi.processBlock(lastTime);
    for (i=0;i<res.m;i++) {
      printf("%f\n",res.data[i]);
    }
  }

}


#endif
