// $Header: /n/basmati/da/bedk/speech/src/frontend/RCS/SOSFilter.h,v 1.2 1997/08/08 02:09:15 bedk Exp $
//
// $Log: SOSFilter.h,v $
// Revision 1.2  1997/08/08 02:09:15  bedk
// Thought I needed doubles as intermediates, but floats will do.
//
// Revision 1.1  1997/08/08 01:57:43  bedk
// Initial revision
//
//
// General ARMA filter object, implemented as a cascade of
// second-order sections
//

#ifndef SOSFilter_h_INCLUDED
#define SOSFilter_h_INCLUDED

#include <stdlib.h>

#include "Blocker.h"
#include "Vector.h"
#include "Matrix.h"

class SOSFilter : public Blocker
{
public:
  SOSFilter(const size_t a_ioblocking,
             const size_t a_nc,
             const char* const a_filename);

  ~SOSFilter()
  {
    delete c;
    delete state_store;
    delete out_buffer;
  };

  const unsigned int group_delay() { return gd; };

  const CVector processBlock(const CVector in,
                             bool& lastTime);

  const CMatrix processBlock(const CMatrix in,
                             bool& lastTime);

private:
  size_t ioblocking;            // max. length input we'll ever see
  
  size_t nc;                    // number of input channels
  
  size_t N;                     // Number of second-order sections
  
  float sf;                     // filter sampling frequency
  
  unsigned int gd;              // group delay in filter passband
                                // (optional)

  float* c;                     // filter coefficients

  float* state_store;           // storage for filter state between
                                // calls to processBlock

  float* out_buffer;            // output buffer
};

#endif // SOSFilter_h_INCLUDED
