// $Header$
//
// $Log$
//
// Object to do various forms of trimming on a data stream
//

#ifndef Trimming_h_INCLUDED
#define Trimming_h_INCLUDED

#include <stdlib.h>

#include "Blocker.h"
#include "Vector.h"
#include "Matrix.h"

class Trimming : public Blocker
{
public:
  Trimming(const size_t a_ioblocking,
           const size_t a_nc,
           const size_t a_nLeading,
           const size_t a_nTrailing)
    : ioblocking(a_ioblocking),
      nc(a_nc),
      nLeading(a_nLeading),
      nTrailing(a_nTrailing),
      lead_trimmed(0),
      points_in_buffer(0),
      prev_out_len(0),
      buffer(new float[nc * (ioblocking + nTrailing)])
  {};

  ~Trimming() { delete buffer; };

  const CVector processBlock(const CVector in,
                             bool& lastTime);

  const CMatrix processBlock(const CMatrix in,
                             bool& lastTime);

private:
  const size_t ioblocking;      // max. length input we'll ever see

  const size_t nc;              // number of input channels
  
  const size_t nLeading;        // number of leading points to trim

  const size_t nTrailing;       // number of trailing points to trim

  size_t lead_trimmed;          // number of leading points trimmed so
                                // far

  size_t points_in_buffer;      // number of valid points in buffer

  size_t prev_out_len;          // number of points output last time

  float* const buffer;          // output buffer
};

#endif // Trimming_h_INCLUDED
