#include "grammar.h"
#include "parser.h"
#include "db.h"
#include "dbquery.h"

#include <ctype.h>
#include <time.h>

#ifndef SHARED_LIB
#include <icsiargs.h>
#endif /* !SHARED_LIB */

#ifdef DEBUG
#define DBGFPRINTF(a)	fprintf a
#else
#define DBGFPRINTF(a)	/* */
#endif /* DEBUG */

/***************************  DEFINES  **************************/

#define REPEAT
#define SPOKEN

#define GOTWHENDAY  0x1
#define GOTWHERE 0x2
#define GOTWHAT  0x4
#define GOTHOWMUCH 0x8
#define DONEINIT 0x10
#define STARTING 0x40
#define GOTWHENMEAL  0x80

/*************************** AUDIO STUFF ********************************/
// Where to find the feedback soundfiles.  Presumed prepended by 
// $SPEECH_DIR if it doesn't start with "/"
#define DEF_AUDIODIR "demo/berp/data/feedback"
// the environment variable that overrides DEF_AUDIODIR
#define AUDIODIR_ENV "BERP_AUDIODIR"
#define MAX_PATH_LEN 256
// The file extension used on the audio files in the AUDIO_DIR
#define AUDIOFEXT ".aif"
// where the audio files are
char *audiodir = NULL;

/***************************  LOCAL PROTOTYPES  **************************/

inline	int FullTemplate(int flags);
void	HowMuch();
static int	GetSentence(char *, int bufsiz);
void	Welcome();
void	Help();
void	Init();
void	Menu();
void	Recommend();
void	Types();
void	Reservations();
void	Farewell();
void	WhenDay();
void	WhenMeal();
void	What();
void	Where();
void	Confused();
void	Sorry();
int	ProcessSentence(char *outbuf,int * TemplateFlags, char *insent);
void	AskNextQuestion(int *flags);
void	DoQuery(int);
int	CheckQuery();
int	WhichAnswer(char *);
void 	PrintState();
int 	DoOldQuery();
void	Reset();
void SentenceOutput(char *s);
void TemplateOutput(char *s);
void Template2Output(char *s);
void Output(char *s);
void Append(char *s);
char *MakeWhenQuestion(char * Date, char *Meal);
void Log(char *fmt ...);

/***************************  EXTERNAL PROTOTYPES  **************************/

extern		char * 	RestInfo(char * );
extern		void 	PrintFlags(int flags);
extern  	char * 	query(char *);
extern		"C" int play_sparc(char *, char*);
extern		int NotWord(char *);
extern 		"C" void init_dbquery(void);

// defined in backendInit.c (was from gui.h)
extern		"C" int guiproc_strs(...);
extern		"C" int guiproc(char *);

/***************************  GLOBAL VARIABLES  **************************/

//int QuestionFlags[9] = {0,WHENQ,WHEREQ,0,WHATQ,0,0,0,HOWMUCHQ};
FILE 		*logfile = NULL;
int 		LastQ;
static char 	QueryBuf[2000];
static ChartParser	*p;
/*
static Grammar three("s",0);
static ChartParser cp(&three);
*/
static Grammar *pthree = NULL;
static ChartParser *pcp = NULL;
char *StateMsg[5] = {"Type of Food:","Distance:","Cost:","Day:","Meal:"};
int TemplateFlags = 0;
char InputBuf[BUFSIZ];
char OutputBuf[9000];
char *Template[5];
char *OldTemplate[5];
char *RestaurantName;
char *PrintTemplate[5];
char *OldPrintTemplate[5];
int ActionFlags = 0;

char TalkerID[5];
char UtteranceID[5];
char LogFile[100];
double Seconds = 0;

// stream for parse tree window (ptw)
extern FILE *s4ptw;
extern "C" FILE *open_ptw(char *);

/*  These are for finding memory leaks */
int News = 0;
int Deletes = 0;
int NodeDeletes = 0;
int NodeCreates = 0;
int ArcNews = 0;
int ArcDeletes = 0;
int RuleNews = 0;
int RuleDeletes = 0;
int SymbolNews = 0;
int SymbolDeletes = 0;

char *wp_cont = "bigramfile";
FILE *fpIn;
int doprint;
extern int Unify, onlineUnify, do_occurs_check;
int UnifyYes, UnifyNo;
extern int do_cover_sem;
char *grammarfile = "./berp.grammar", *inputfile;

// for new database stuff
char *db_prog  = NULL;
char *database = NULL;

extern "C" void init_db_paths(void) 
{   /* build the static paths if we need to */
static int db_paths_inited=0;
    if (db_paths_inited == 0) {
	db_prog = new char[MAX_PATH_LEN];
	database = new char[MAX_PATH_LEN];
	audiodir = new char[MAX_PATH_LEN];
	maybe_prepend_speechdir(db_prog, DB_PROG);
	maybe_prepend_speechdir(database, DATABASE);
	/* dpwe - audiodir init */
	char audiodir_tmp[MAX_PATH_LEN];
	getenv_default(AUDIODIR_ENV, DEF_AUDIODIR, audiodir_tmp);
	// If the first character isn't a "/", rebuild it, 
	// prepending $SPEECH_DIR
	maybe_prepend_speechdir(audiodir, audiodir_tmp);
	
	db_paths_inited = 1;
   }
}

/***************************  MAIN  **************************/

extern "C" void Tk_Main(int argc, char **argv, int (*Tcl_AppInit)(Tcl_Interp *));

#ifndef SHARED_LIB
extern "C"
main(int argc, char **argv){
  int pt=0;
  char *opt_ptw_name = NULL;
  icsiargtab_t args[] = {
    { "input", ": the sentences file as input", ARG_STR, &inputfile, ARG_OPT},
    { "grammar", ": the SCFG file (defaults to ./berp.grammar)", ARG_STR, &grammarfile, ARG_OPT},
// In Eric's better version
//    { "playcmd", "program used to play audio (.au) files", ARG_STR,&playcmd, ARG_OPT},
    { "audiodir", "directory where the audio feedback files live", ARG_STR, &audiodir, ARG_OPT},
    { "occurs", "do the occurs check? (defaults to FALSE)", ARG_BOOL, &do_occurs_check, ARG_OPT },
    { "unify", "do unification? (defaults to TRUE)", ARG_BOOL, &Unify, ARG_OPT },
    { "online_unify", "do online unification? (defaults to FALSE, TRUE implies unify=TRUE)", ARG_BOOL, &onlineUnify, ARG_OPT },
//    { "paramfile", "what file to expect front-end info for demo in", ARG_STR, &PARAMFILE, ARG_OPT},
//    { "donefile", "where to send information back to frontend", ARG_STR, &FLAGFILE, ARG_OPT},
    { "parsetree", "print out parse trees", ARG_BOOL, &pt, ARG_OPT},
    { "ptprog", "program print out parse trees", ARG_STR, &opt_ptw_name, ARG_OPT},
    { "db_prog", "database program", ARG_STR, &db_prog, ARG_OPT},
    { "database", "restaurant database file", ARG_STR, &database, ARG_OPT},
    { "debug", ": true to print out lots of debug info", ARG_INT, &doprint, ARG_OPT },
    { 0,0  }
  };
  char *appname;

  do_occurs_check=0;
 
  Unify=1;
  onlineUnify=0;
  
  init_db_paths();
  icsiargs(args,&argc, (const char ***)&argv, &appname);
  do_cover_sem=1;
  init_dbquery();

  if (onlineUnify) Unify=1;
 
  if (inputfile == NULL) 
    fpIn = stdin;
  else
    fpIn = fopen(inputfile,"r");
  /* end of argument processing */

  if (pt) {
      s4ptw=open_ptw(opt_ptw_name);
  }
  Tk_Main(argc, argv, Tcl_AppInit);
  return 0;
};
#endif /* !SHARED_LIB */

extern "C" void getenv_default(const char *envname, const char *deflt, 
			       char *out)
{   /* dpwe - Get a string value from the environment using the name *envname, 
       copy it into the pre-allocated string *out.  But if it's not 
       defined in the environment, use the default value *default.
       (prototype in dbquery.h) */
    char *s;
    if((s = getenv(envname)) != NULL) {
	strcpy(out, s);
    } else {
#ifdef WARN_GETENV_DFLT
	fprintf(stderr, "backend.C: getenv_default: '%s' not set, taking default value '%s'\n", envname, deflt);
#endif /* WARN_GETENV_DFLT */
	strcpy(out, deflt);
    }
}

const char *speech_dir(void) 
{   /* dpwe - return the value for $SPEECH_DIR, either from the environment 
       or using a default (prototype in dbquery.h) */
    static char SPEECH_DIR[MAX_PATH_LEN];

    getenv_default(SPEECHDIR_ENV, DEF_SPEECHDIR, SPEECH_DIR);
    return SPEECH_DIR;
}

void maybe_prepend_speechdir(char *out, const char *in)
{   /* dpwe - Take a path as input in *in; if it begins with "/" (indicating 
       an absolute path), copy it straight into the pre-allocated, 
       passed-in string *out (presumed large enough).  If it *doesn't* 
       start with "/" (relative path), prepend $SPEECH_DIR (from 
       speech_dir()) and then catenate *in. (prototype in dbquery.h) */
    if (in[0] == '/') {
	strcpy(out, in);
    } else {
	strcpy(out, speech_dir());
	strcat(out, "/");
	strcat(out, in);
    }
}

/* Caution: You can't call "guiproc()" or "guiproc_strs()", because TCL script
 *		is not processed yet, so "guiprocreg" is not invoked and
 *		"guiproc()" can't invoke any TCL procedure.
 */
extern "C"
int backend_init(void)
{
	int i;
	int CurrentWantFlag = 0;
	FILE *fp;

	LogFile[0] = 0;

	//strcpy(LogFile,"/u/jurafsky/berp/src/logfile");

	DBGFPRINTF((stderr, "be_init\n"));

	OutputBuf[0] = 0;
	Output("");
	TemplateOutput("Type of Food:\nDistance:\nCost:\nDay:\nMeal:");
	Template2Output("NONE\nNONE\nNONE\nNONE\nNONE");
	SentenceOutput(" ");

	// Initialization
	DBGFPRINTF((stderr, "be_init init\n"));

	for (i=0; i < 5; i++)  {
		Template[i] = new char[BUFSIZ];
		Template[i][0] = 0;
		OldTemplate[i] = new char[BUFSIZ];
		OldTemplate[i][0] = 0;
		PrintTemplate[i] = new char[BUFSIZ];
		PrintTemplate[i][0] = 0;
		OldPrintTemplate[i] = new char[BUFSIZ];
		OldPrintTemplate[i][0] = 0;
	}
	RestaurantName = new char[100];

	DBGFPRINTF((stderr, "be_init read grammar\n"));

	if (pthree == NULL) {
	    /* fix for no static inits under g++ shared obj */
	    pthree = new Grammar("s",0);
	    pcp = new ChartParser(pthree);
	}

	p = pcp;

	if (!pthree->ReadFile(grammarfile)) {
	    // error reading grammar
	    return 0;
	}

	DBGFPRINTF((stderr, "be_welcome\n"));
	Welcome();

	DBGFPRINTF((stderr, "be_reset\n"));
	Reset();
	
	TemplateFlags |= DONEINIT | STARTING;
	DBGFPRINTF((stderr, "be_init done\n"));

	return 1;
}


extern "C"
void backend_preloop()
{
    FILE *fp;
    int i;
    /* If we have a full template, build a query out of it,
       send it off to the database, and display the
       result to the user.  We also keep a backup copy
       of the template in OldTemplate so they can
       `show the list again' if they want */
	
    if (FullTemplate(TemplateFlags)) {
	if ( (ActionFlags & FLAG_GETTYPES ) ||
	     (ActionFlags & FLAG_SHOWLIST)) {
	    ActionFlags &= ~FLAG_GETTYPES;
	    ActionFlags &= ~FLAG_SHOWLIST;
	} else {
	    if  (ActionFlags & FLAG_CONFUSED) 
		DoQuery(FALSE);
	    else
		DoQuery(TRUE);
	}
    } 
    ActionFlags &= ~FLAG_CONFUSED;

    /* Now ask the next question (or 
       the first question, if we just finished a query) */

    AskNextQuestion(&TemplateFlags);

    /* Get the user's answer, and go parse it.  This is
       also where we update the slider on the user's
       screen as the RAP gives us tickmarks every
       200 ms */
}

extern "C"
void backend_end()
{
  int i;
	/*** DeInitialization ***/

	Farewell();

	for (i=0; i < 5; i++) 
		delete Template[i];
	exit(0);
}

extern "C"
void backend_postloop(int argc, char *argv[])
{
    char *insent = NULL;
    if(argc > 1) {
	insent = argv[1];
    }

    ActionFlags |= ProcessSentence(InputBuf,&TemplateFlags, insent);

    if (ActionFlags & FLAG_QUIT)
	backend_end(); /* The end !! */

    Output("");
    guiproc("rinfowdown");
    guiproc("rlwindown");
    guiproc("respclr");


    /* Print the current state */

    if (!(ActionFlags & FLAG_CONFUSED)
	&& !(ActionFlags & FLAG_RESTART) 
	&& !(ActionFlags & FLAG_SHOWLIST)) {
	PrintState();
    }
    InputBuf[0] = 0;
		

    /*  Now check for all the various kinds of special
	situations. */

    if (ActionFlags & FLAG_SHOWLIST)  {
	if (!DoOldQuery()) {
	    Append("I'm sorry, I no longer have your earlier list\n");
	    play_sparc(audiodir, "01" AUDIOFEXT);
	}
	//ActionFlags &= ~FLAG_SHOWLIST;
    }
    if (ActionFlags & FLAG_HELP)  {
	Help();
	ActionFlags &= ~FLAG_HELP;
    }
    if (ActionFlags & FLAG_RECOMMEND)  {
	Recommend();
	ActionFlags &= ~FLAG_RECOMMEND;
    }
    if (ActionFlags & FLAG_GETTYPES)  {
	Types();
	//ActionFlags &= ~FLAG_GETTYPES;
    }
    if (ActionFlags & FLAG_MENU)  {
	Menu();
	ActionFlags &= ~FLAG_MENU;
    }
    if (ActionFlags & FLAG_RESERVATIONS)  {
	Reservations();
	ActionFlags &= ~FLAG_RESERVATIONS;
    }
    if (ActionFlags & FLAG_THANKS)  {
	Append("You're welcome!\n");
	play_sparc(audiodir, "04" AUDIOFEXT);
	ActionFlags &= ~FLAG_THANKS;
    }
    if (ActionFlags & FLAG_CONFUSED) {
	Confused();
    }
    if (ActionFlags & FLAG_RESTART)  {
	Reset();
	PrintState();
    }
}

/*********************************************************************

	FUNCTION:	FullTemplate()
	ARGS:		int flags - the current template flags
	RETURNS:	true if template is full OR if
			user has specified a specific restaurant instead

*********************************************************************/

inline int FullTemplate(int flags)
{
	return ( ((flags & 0xF) == 0xF) || RestaurantName[0]);
}

/*********************************************************************

	FUNCTION:	AskNextQuestion()
			Asks the user the appropriate next question.
	ARGS:		int *flags - the current template flags
	RETURNS:	sets DONEINIT and resets STARTING if necessary.

*********************************************************************/

void AskNextQuestion(int *flags)
{
	// Don't ask a question if we just printed startup message
	if (*flags & STARTING) {
		*flags &= ~STARTING;
		return;
	}
	if (!(*flags & DONEINIT)) {
		Init();
		*flags = DONEINIT;
	} else if (!(*flags & GOTWHAT))
	  What();
	else if (!(*flags & GOTWHENDAY))
	  WhenDay();
	else if (!(*flags & GOTWHENMEAL))
	  WhenMeal();
	else if (!(*flags & GOTHOWMUCH))
	  HowMuch();
	else if (!(*flags & GOTWHERE))
	  Where();
}
	

/*********************************************************************

	FUNCTION:	GetSentence()
			Reads the next sentence from stdin, converts to lower
			case and sticks it in buffer.
	ARGS:		char *buf - the data buffer
	RETURNS:	false if buffer is empty, else true

*********************************************************************/


static int GetSentence(char *buf,int bufsiz)
{
    int i;
    buf[0] = 0;
    int DoneRead = FALSE;
    char *r;

    while (TRUE) {
	r = fgets(buf,bufsiz,fpIn);
	if(r==NULL) {
	    /* EOF on input */
	    return FALSE;
	}
	if (buf[0] == '\n')
	    continue;
	if (buf[0] == '#') {
	    if (!DoneRead) {
		DoneRead = TRUE;
		SentenceOutput(" ");
	    }
	} else {
	    DoneRead = FALSE;
	    break;
	}
    }

    for(i = 0; i < strlen(buf); i++) 
	buf[i] = tolower(buf[i]);

    if (buf[0])
	return TRUE;
    else
	return FALSE;
}

void Init()
{
#ifdef SPOKEN
        Append("\nREMEMBER: you must click on the RECORD button before you begin speaking.\n\n\nREADY...\n"); 
#else
	Append("\nReady for a new query.\n\n");
#endif
	fflush(stdout);
};

void Welcome()
{
#ifdef SPOKEN
	Output("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nWelcome to BeRP.  How may I help you?\n\nREMEMBER: you must click on the RECORD button before you begin speaking.\n\n\nREADY...\n");
	play_sparc(audiodir, "05" AUDIOFEXT);
#ifdef ASCII
	fflush(stdout);
#endif
#else
	cout << "\n\nWELCOME!!\n";
	cout << "In this session your task will be to find one or more restaurants in\n";
	cout << "Berkeley. The task of finding a restaurant involves telling the\n";
	cout << "computer the type of food you want to eat, the amount of money you\n";
	cout << "want to spend, etc.\n\n";
	cout << "To use the system you type to the computer one sentence at a time.\n";
	cout << "Since this system will eventually be part of a speech system, try to say your sentences\n";
	cout << "out loud to yourself before you type them.  They should be ``normal'' sentences \n";
        cout << "as if you were speaking to a person.  \n";
	cout << "\n PLEASE CHECK YOUR SPELLING, AND SPELL OUT RESTAURANT NAMES exactly, \n";
	cout << "but SUBSTITUTE DASHES FOR SPACES IN RESTAURANT NAMES.\n\n";
	cout << "The computer's response will take one of several forms:\n";
	cout << "1) A request for additional information to narrow down the list of possible restaurants.\n";
	cout << "2) A request for you to repeat your statement, because the computer\n";
	cout << "could not understand you or a word you used.\n";
	cout << "3) A statement that there are no restaurants listed in the computer\n";
	cout << "data base which match your choices.\n";
	cout << "4) A short list of restaurants which match the choices you made.\n";
	cout << "To start over, you may say 'start over'.\n";
	cout << "Until you say this, the system assumes that you are adding information\n";
	cout << "to something you said previously.  For example, if you previously said\n";
	cout << "that you wanted to find a Chinese restaurant, and now you ask for a\n";
	cout << "French restaurant, the computer will look for Chinese AND French restaurants.\n";
	cout << "\n\nNow go ahead and give me some constraints on your restaurant.\n";
	cout << flush;
#endif
}

void Help()
{
        cout << "Here's a repeat of the instructions:\n";
        cout << "In this session your task will be to find one or more restaurants in\n";
        cout << "Berkeley. The task of finding a restaurant involves telling the\n";
        cout << "computer the type of food you want to eat, the amount of money you\n";
        cout << "want to spend, etc.\n";
        cout << "To use the system you type to the computer one sentence at a time.\n";
        cout << "Since this system will be part of a speech system, try to say your sentences\n";
        cout << "out loud to yourself before you type them.  They should be ``normal'' sentences \n";
        cout << "as if you were speaking to a person.  \n";
	cout << "\n PLEASE CHECK YOUR SPELLING, AND SPELL OUT RESTAURANT NAMES exactly, \n";
	cout << "but SUBSTITUTE DASHES FOR SPACES IN RESTAURANT NAMES.\n\n";
        cout << "The computer's response will take one of several forms:\n";
        cout << "1) A request for additional information to narrow down the list of possible restaurants.\n";
        cout << "2) A request for you to repeat your statement, because the computer\n";
        cout << "could not understand you or a word you used.\n";
        cout << "3) A statement that there are no restaurants listed in the computer\n";
        cout << "data base which match your choices.\n";
        cout << "4) A short list of restaurants which match the choices you made.\n";
	cout << "To start over, you may say 'start over'.\n";
	cout << "Until you say this, the system assumes that you are adding information\n";
	cout << "to something you said previously.  For example, if you previously said\n";
	cout << "that you wanted to find a Chinese restaurant, and now you ask for a\n";
	cout << "French restaurant, the computer will look for Chinese AND French restaurants\n";
	cout << flush;
}


void Farewell()
{
	Append("\nThank you for your participation.\n");
	play_sparc(audiodir, "06" AUDIOFEXT);
}

void Types()
{
	Append("\nHere are the types of food I know about:\n\n"
		"@bold(African    American   South American   Asian)\n"
		"@bold(California Chinese    Cuban            Ethiopian)\n"
		"@bold(European   Fast       French           German)\n"
		"@bold(Greek      Indian     Italian          Japanese)\n"
		"@bold(Mexican    Persian    Russian          Taiwanese)\n"
		"@bold(Thai       Vegetarian Vietnamese       Barbeque)\n");
	play_sparc(audiodir, "18" AUDIOFEXT);
}
void Recommend()
{
	Append("\nI'm sorry, I'm just a computer, I can't make any recommendations.\n");
	play_sparc(audiodir, "07" AUDIOFEXT);
}

void Menu()
{
	Append("\nI'm sorry, I don't have copies of any menus.\n");
	play_sparc(audiodir, "08" AUDIOFEXT);
}

void Reservations()
{
	Append("\nI'm sorry, I don't know anything about reservations.\n");
	play_sparc(audiodir, "09" AUDIOFEXT);
}

void WhenDay()
{
	Append("\nPlease provide more information in order to narrow your choices.\n\nFor example: WHAT DAY WOULD YOU LIKE TO EAT?\n");
	LastQ = WHENDAY;
	play_sparc(audiodir, "10" AUDIOFEXT);
}
void WhenMeal()
{
	Append("\nPlease provide more information in order to narrow your choices.\n\nFor example: WHICH MEAL(S) (BREAKFAST, LUNCH, DINNER) WOULD YOU LIKE TO EAT?\n");
	LastQ = WHENMEAL;
	play_sparc(audiodir, "11" AUDIOFEXT);
}

void HowMuch()
{
        Append("\nPlease provide more information in order to narrow your choices.\n\nFor example: HOW MUCH MONEY DO YOU WANT TO SPEND?\n");
	LastQ = HOWMUCH;
	play_sparc(audiodir, "12" AUDIOFEXT);
}

void What()
{
	Append("\nPlease provide more information in order to narrow your choices.\n\nFor example: WHAT TYPE OF FOOD YOU WOULD LIKE TO EAT?\n");
	LastQ = WHAT;
	play_sparc(audiodir, "13" AUDIOFEXT);
}

void Where()
{
	Append("\nPlease provide more information in order to narrow your choices.\n\nFor example: HOW FAR FROM ICSI ARE YOU WILLING TO TRAVEL?\n");
	LastQ = WHERE;
	play_sparc(audiodir, "14" AUDIOFEXT);
}

void Confused()
{
	Append("\nI'm sorry. I did not understand what you said.\n");
	play_sparc(audiodir, "02" AUDIOFEXT);
}

void Sorry()
{
	Append("\nI'm sorry, that information is not in the database.\n");
	play_sparc(audiodir, "03" AUDIOFEXT);
}	  

char *InsertBold(char *buf,int *guesslist)
{
        static char outbuf[BUFSIZ];
        char *q;
        int i = 0;

	outbuf[0] = 0;
	
	q = strtok(buf," ");
	while (q != NULL) {
		if (NotWord(q)) {
			strcat(outbuf,q);
		} else {
			if (guesslist[i]) {
				strcat(outbuf,"@bold(");
				strcat(outbuf,q);
				strcat(outbuf,")");
			} else 
				strcat(outbuf,q);
			i++; 
		}
		strcat(outbuf," ");
		q = strtok(NULL," ");
	}
	return outbuf;
}

int ProcessSentence(char *outbuf,int * TemplateFlags, char *insent)
{		
	int actionflags;
	char *s;
	char inbuf[BUFSIZ];
	char sbuf[BUFSIZ];
	double pr = 0.0;
	int guessflag = 0;
	int guesslist[MAXGUESS];
	int i;

	for (i=0;i<MAXGUESS;i++)
		guesslist[i] = 0;

	/* dpwe - allow the sentence to be passed in */
	if (insent != NULL) {
	    char *s=insent, *t=inbuf;
	    char c;
	    do {
		*t++ = c = tolower(*s++);
	    } while (c);
	    if (inbuf[0] == '\0') {
		return FLAG_QUIT;
	    }
	} else if (!GetSentence(inbuf,BUFSIZ)) {
		return FLAG_QUIT;
	}

#ifdef ASCII
	printf("This is what I think you said: ``%s''\n",inbuf);
	fflush(stdout);
#endif
	/* Print out the sentence as first recognized.
		We will update this later as the parser highlights
		the understood fragments */

	SentenceOutput(inbuf);

	Log("[Begin Recognized: %s]\n%s\n[End Recognized: %s]\n",UtteranceID,inbuf,UtteranceID);



	/*    Parse the sentence */
        char buf[BUFSIZ];      // BEG/END added by Dan G - why not there before?
        strcpy(buf,"BEG ");
        strcat(buf,inbuf);
        strcat(buf," END");

	s = p->Parser(inbuf,&pr,FALSE,doprint,&guessflag,guesslist);
        if (doprint) 
          printf("%s\n", s);

	if (NULL != s4ptw) fflush(s4ptw);

	if (!s) {
		// Not parseable
		actionflags = FLAG_CONFUSED;
//		SUIT_setText(label1,LABEL,"What I THINK You Said:");
//		SentenceOutput(inbuf);
	} else {
		/* Parseable.  First Xlate the query, then
			outbuf the wordlist with the useful ones
			boldfaced.  (If the whole sentence parsed,
			we boldface the whole thing).  */

		strcpy(sbuf,s);
		actionflags = XlateQuery(sbuf,outbuf,LastQ);
		if (guessflag) {
			SentenceOutput(InsertBold(inbuf,guesslist));
//			SUIT_setText(label1,LABEL,"What I THINK You Said:");
		} else {
			strcpy(sbuf,"@bold(");
			strcat(sbuf,inbuf);
			strcat(sbuf,")");
			SentenceOutput(sbuf);
//			SUIT_setText(label1,LABEL,"What You Said:");
		}
#ifdef DEBUG
		if (!actionflags) {
			if (guessflag)
				printf("I didn't fully understand, but I think your\n  sentence has the following semantics: %s\n",outbuf);
			else
				printf("Your sentence has the following semantics: %s\n",outbuf);
		} else
		fflush(stdout);
#endif
		//if (actionflags)
			//PrintFlags(actionflags);
		*TemplateFlags |= CheckQuery();
	}
	p->Reset();
	return actionflags;
}

/******************************************************************

	CheckQuery() -- returns a flag value with a bit
		set for each slot we have filled in the current
		template.

******************************************************************/

int CheckQuery()
{
	int flags = 0;
	if (PrintTemplate[WHENDAYQ][0]) flags |= GOTWHENDAY;
	if (PrintTemplate[WHENMEALQ][0]) flags |= GOTWHENMEAL;
	if (PrintTemplate[WHATQ][0]) flags |= GOTWHAT;
	if (PrintTemplate[WHEREQ][0]) flags |= GOTWHERE;
	if (PrintTemplate[HOWMUCHQ][0]) flags |= GOTHOWMUCH;
	return flags;
}

/******************************************************************

	DoQuery() -- Build a query from the current template,
		call query() (in q.C) to do the query, and
		then output the result to the output window

******************************************************************/

void DoQuery(int speakflag)
{
    int i;
    int testflag = FALSE;
    int gotflag = FALSE;
    char *p;
    struct timeval tp;
#ifdef NOQ
    return;
#else
    for (i=0;i<5;i++) {
	if (Template[i][0]) testflag = TRUE;
    }
    if (RestaurantName[0]) testflag = TRUE;
    if (!testflag) return;

    if (RestaurantName[0]) {
	p=RestInfo(RestaurantName);
	Output(p);
	if ('S' == p[0]) {
	    guiproc_strs("forresp", p, 0);
	}
	TemplateFlags &= ~DONEINIT;
    } else {
#ifdef POSTGRES
	strcpy(QueryBuf,"retrieve portal rportal (REST.all) where ");
#else /* !POSTGRES */
	/* bug: was accumulating all queries! */
	QueryBuf[0] = '\0';
#endif
	for (i=0;i<3;i++) {
	    if (Template[i][0]) {
		if (gotflag)
		    strcat(QueryBuf, " and ");
		strcat(QueryBuf,Template[i]);
		gotflag = TRUE;
	    }
	}
	if (Template[3][0] || Template[4][0]) {
	    if (gotflag)
		strcat(QueryBuf, " and ");
	    strcat(QueryBuf,MakeWhenQuestion(Template[3],Template[4]));
	}
	gettimeofday(&tp,NULL);
	Log("[Timestamp: Sent query for utterance %s at %s]\n",UtteranceID,ctime(&(tp.tv_sec)));
	Log("[Begin Query: %s]\n%s\n[End Query: %s]\n",UtteranceID,QueryBuf,UtteranceID);
#ifndef POSTGRES
	strcat(QueryBuf,"\n");
#endif
#ifdef ASCII
	printf("query=%s\n", QueryBuf);
	fflush(stdout);
#endif /* ASCII */
	p = query(QueryBuf);
	Output(p);
	if ('S' == p[0]) {
	    guiproc_strs("forresp", p, 0);
	} else {
	    guiproc("rlwinup");
	}
	gettimeofday(&tp,NULL);
	Log("[Timestamp: Sent answer for utterance %s at %s]\n",UtteranceID,ctime(&(tp.tv_sec)));
	Log("[Begin Result: %s]\n%s\n[End Result: %s]\n",UtteranceID,p,UtteranceID);
	for (i=0;i<5;i++) {
	    strcpy(OldTemplate[i],Template[i]);
	    strcpy(OldPrintTemplate[i],PrintTemplate[i]);
	}
    }

    if (speakflag) {
	if (!strcmp(p,"Sorry, there was a problem with your query. Please try again.\n")) {
	    play_sparc(audiodir, "15" AUDIOFEXT);
	} else if (!strncmp(p,"\nHere are",strlen("\nHere are"))) { 
	    play_sparc(audiodir, "16" AUDIOFEXT);
	    play_sparc(audiodir, "17" AUDIOFEXT);
	} else if (!strncmp(p, "\nThis is the",strlen("\nThis is the"))) {
	    play_sparc(audiodir, "18" AUDIOFEXT);
	} else if (!strcmp(p,"Sorry, no restaurants match your query.\n")) {
	    play_sparc(audiodir, "19" AUDIOFEXT);
	}
    }

    //Reset();
    //ActionFlags = 0;
    RestaurantName[0] = 0;
#endif
}

char *MakeWhenQuestion(char * Date, char *Meal)
{
	static char buf[BUFSIZ];
	static char *any = "\"[BLD]\"";
	static char *anyday = "ANY DAY";
	if (!Meal[0]) 
		Meal = any;
	if (!Date[0]) 
		Date = anyday;

	if (!strcmp(Date,"ANY DAY")) {
		sprintf(buf,"(REST.mon ~ %s  or REST.tue ~ %s  or REST.wed ~ %s  or REST.thu ~ %s  or REST.fri ~ %s  or REST.sat ~ %s  or REST.sun ~ %s )",Meal,Meal,Meal,Meal,Meal,Meal,Meal);
	} else if (!strcmp(Date,"EVERY DAY")) {
	        sprintf(buf,"(REST.mon ~ %s  and REST.tue ~ %s  and REST.wed ~ %s  and REST.thu ~ %s  and REST.fri ~ %s  and REST.sat ~ %s  and REST.sun ~ %s )",Meal,Meal,Meal,Meal,Meal,Meal,Meal);
	} else if (!strcmp(Date,"WEEKENDS")) {
	        sprintf(buf,"(REST.sat ~ %s and REST.sun ~ %s )",Meal,Meal);
	} else {
		sprintf(buf,"(%s ~ %s)",Date,Meal);
	}
	
	return buf;
}

int DoOldQuery()
{
	int i;
	int flag = FALSE;
#ifdef NOQ
	return TRUE;
#else
	// If there is no old template, return false

	for (i=0;i<5;i++) {
		if (OldTemplate[i][0]) flag = TRUE;
	}
	if (!flag) return FALSE;

	// Copy the old query over the new one
	for (i=0;i<5;i++) {
		strcpy(Template[i],OldTemplate[i]);
		strcpy(PrintTemplate[i],OldPrintTemplate[i]);
	}
	PrintState();
	RestaurantName[0] = 0;
	DoQuery(TRUE);
	return TRUE;
#endif
}

void Reset()
{
	int i;
	ActionFlags = 0;
	QueryBuf[0] = 0;
	InputBuf[0] = 0;
	RestaurantName[0] = 0;
	for(i=0;i<5;i++) {
		Template[i][0] = 0;
		PrintTemplate[i][0] = 0;
	}
	TemplateFlags = 0;
}

char * spaces(int i)
{
	static char buf[30];
	int j;

	for (j=0;j<i;j++)
		buf[j]=' ';
	buf[i] = 0;	
	return buf;
}

void gui_QF(int n, char *s)
{
  char buf[4];
  sprintf(buf, "qf%d", n);
  guiproc_strs(buf, s, 0);
}

void PrintState()
{
	int i;

#ifdef ASCII
	printf("This is the current query information: \n");
	if (RestaurantName[0])
		printf("Restaurant Name: %s\n",RestaurantName);
	else for (i = 0; i < 5 ; i++) {
		printf("  %s",StateMsg[i]);
		if (PrintTemplate[i][0])
			printf("%s\n",PrintTemplate[i]);
		else
			printf("NONE\n");
	}	
	fflush(stdout);
	cout << flush;
#else
	static char buffer[BUFSIZ];
	buffer[0]=0;
	for (i = 0; i < 5 ; i++) {
		if (PrintTemplate[i][0]) {
			strcat(buffer,"@bold(");
			strcat(buffer,PrintTemplate[i]);
			strcat(buffer,")\n");
		} else
			strcat(buffer,"@italic(NONE)\n");
	}	
	Template2Output(buffer);

	for (i = 0; i < 5 ; i++) {
	  if (PrintTemplate[i][0]) {
	    gui_QF(i, PrintTemplate[i]);
	  } else
	    gui_QF(i, "NONE");
	}

	Log("[*Begin Template: %s]\n",UtteranceID);
	// Output to Logfile
	for (i = 0; i < 5 ; i++) {
		Log("%s ",StateMsg[i]);
		if (PrintTemplate[i][0])
			Log("%s,  ",PrintTemplate[i]);
		else
			Log("NONE,");
	}
	Log("\n[*End Template: %s]\n",UtteranceID);
#endif
}

void Output(char *s)
{
	strcpy(OutputBuf,s);
}

void Append(char *s)
{
#ifdef ASCII
    printf("%s", s);
    fflush(stdout);
#endif /* ASCII */
	Log("[*Begin Instruction: %s]\n%s\n[*End Instruction: %s]\n",UtteranceID,s,UtteranceID);
	strcat(OutputBuf,s);
	guiproc_strs("app", s, 0);
}

void TemplateOutput(char *s)
{
}
void Template2Output(char *s)
{
}

void SentenceOutput(char *s)
{
	guiproc_strs("rec", s, 0);
}

void *operator new(size_t sz)
{
	++News;
	return malloc(sz);
}

void operator delete(void *p)
{
	++Deletes;
	free((char *)p);
}


void Log(char *fmt ...)
{
	va_list ap;
	char c;
	va_start(ap,fmt);
	if (!LogFile[0]) {
		va_end(ap);
		return;
	}

	while (c = *fmt++) {
		if (c == '%') {
			switch(c = *fmt++)  {
				case 's':
					fprintf(logfile,"%s", va_arg(ap,char *));
			}
		} else
			fprintf(logfile,"%c",c);
	}
	va_end(ap);
	fflush(logfile);
}
