/*
 * backendInit.c
 * 
 * Provide a stand-alone _Init command
 * to allow the backend extensions to be built as a tcl dynamic library 
 * extension.
 * 
 * for the BeRP/tateyama database backend
 * 
 * 1996oct14 dpwe@icsi.berkeley.edu
 * $Header: /n/yam/da/dpwe/projects/berpdemo/berpbackend/RCS/backendInit.c,v 1.4 1998/03/14 09:51:35 dpwe Exp $
 */

#include <stdlib.h> 	/* for malloc, used in ckalloc */
#include <stdio.h>	/* popen, FILE etc */
#include <string.h>	/* for strdup? */
#include <stdarg.h>	/* for guiproc_strs(...) */

#include <tcl.h>

#ifdef DEBUG
#define DBGFPRINTF(a)	fprintf a
#else
#define DBGFPRINTF(a)	/* */
#endif /* DEBUG */

/* #include "gui.h" */
EXTERN int guiproc_strs(char *, ...);
EXTERN int guiproc(char *);
#define 	GUI_NOPROC	99

/* declare command procedures here */
EXTERN int backendloopCmd(ClientData, Tcl_Interp*, int argc, char *argv[]);
EXTERN int guiprocregCmd(ClientData, Tcl_Interp*, int argc, char *argv[]);

EXTERN int  backend_init(void);
EXTERN void backend_preloop(void);
EXTERN void backend_postloop(int argc, char *argv[]);
EXTERN void backend_end(void);

/* in dbquery.C */
EXTERN void init_dbquery(void);

/* global record of the interpreter for foul and ugly hacking */
Tcl_Interp *G_interp;

int atobool(char *s) 
{   /* convert a string to a boolean result */
    int r = 0;
    int n = strlen(s);
    if(strncasecmp(s, "YES", n) == 0) {
	r = 1;
    } else if(strncasecmp(s, "TRUE", n) == 0) {
	r = 1;
    } else if(strncasecmp(s, "1", n) == 0) {
	r = 1;
    }
    return r;
}

/* Global vars set by environment */
extern char *inputfile;
extern char *grammarfile;
extern int  do_occurs_check;
extern int  Unify;
extern int  onlineUnify;
extern char *db_prog;
extern char *database;
extern char *audiodir;
extern int  doprint;
extern int  do_cover_sem;
extern FILE *fpIn;
/* stream for parse tree window (ptw) */
extern FILE *s4ptw;
extern FILE *open_ptw(char *);

int backend_init_vars(Tcl_Interp *interp)
{   /* initialize global backendsh variables set via the command line */
    char *s;
    int  pt = 0;
    char *ptprog = NULL;

    do_occurs_check=0;
 
    Unify=1;
    onlineUnify=0;
    
    init_db_paths();
    /* icsiargs(args,&argc, (const char ***)&argv, &appname); */
    if( (s = Tcl_GetVar(interp, "input", TCL_GLOBAL_ONLY)) ) {
	inputfile = strdup(s);
    }
    if( (s = Tcl_GetVar(interp, "grammar", TCL_GLOBAL_ONLY)) ) {
	grammarfile = strdup(s);
    }
    if( (s = Tcl_GetVar(interp, "occurs", TCL_GLOBAL_ONLY)) ) {
	do_occurs_check = atobool(s);
    }
    if( (s = Tcl_GetVar(interp, "unify", TCL_GLOBAL_ONLY)) ) {
	Unify = atobool(s);
    }
    if( (s = Tcl_GetVar(interp, "online_unify", TCL_GLOBAL_ONLY)) ) {
	onlineUnify = atobool(s);
    }
    if( (s = Tcl_GetVar(interp, "parsetree", TCL_GLOBAL_ONLY)) ) {
	pt = atobool(s);
    }
    if( (s = Tcl_GetVar(interp, "ptprog", TCL_GLOBAL_ONLY)) ) {
	ptprog = strdup(s);
    }
    if( (s = Tcl_GetVar(interp, "db_prog", TCL_GLOBAL_ONLY)) ) {
	strcpy(db_prog, s);
    }
    if( (s = Tcl_GetVar(interp, "database", TCL_GLOBAL_ONLY)) ) {
	strcpy(database, s);
    }
    if( (s = Tcl_GetVar(interp, "audiodir", TCL_GLOBAL_ONLY)) ) {
	strcpy(audiodir, s);
    }
    if( (s = Tcl_GetVar(interp, "debug", TCL_GLOBAL_ONLY)) ) {
	doprint = atobool(s);
    }

    do_cover_sem=1;
 
    if (onlineUnify) Unify=1;
 
    if (inputfile == NULL) 
	fpIn = stdin;
    else
	fpIn = fopen(inputfile,"r");
    /* end of argument processing */

    if (pt) {
	s4ptw=open_ptw(ptprog);
    }
    
    DBGFPRINTF((stderr, "abt to init_dbq\n"));
    /* launch the simpdb server etc. */
    init_dbquery();

    DBGFPRINTF((stderr, "backend_init_vars: db_prog=%s\n", db_prog));
    /* fprintf(stderr, "backendInit: donefile='%s'\n", FLAGFILE); */

    return TCL_OK;
}

int Berpbackend_Init(Tcl_Interp *interp) 
{   /* Initialize the backend package */
    G_interp = interp;

    Tcl_CreateCommand(interp, "backendloop", backendloopCmd, 0, 0);
    Tcl_CreateCommand(interp, "guiprocreg", guiprocregCmd, 0, 0);

    backend_init_vars(interp);
    DBGFPRINTF((stderr, "backend_init_vars done\n"));
    if(!backend_init()) {
	return TCL_ERROR;
    }
    DBGFPRINTF((stderr, "backend_init done\n"));
    backend_preloop();
    DBGFPRINTF((stderr, "backend_preloop done\n"));

    return TCL_OK;
}

/* tateyama procs... */

/*
 * This routine will be called when there is some input from stdin.
 * Then processing these input strings, and preparing for next input.
 */
int backendloopCmd(
    ClientData cd,
    Tcl_Interp *interp,
    int argc,
    char *argv[])
{
  backend_postloop(argc, argv);
  backend_preloop();
  return TCL_OK;
}


#define mymempreserve(buf, buflen, bufsize, margin) \
	if ((buflen) < (bufsize)) { \
		ckfree(buf); \
		(buf) = ckalloc((bufsize) + (margin)); \
		(buflen) = (bufsize) + (margin); \
	}


char *guiprocstr = NULL;
int guiproc_sz = 0;

/*
 * This routine will be called to register "Tcl procedure" which is called
 * when "backendsh" wants "Tcl script" to do something, probably some commands
 * to GUI.
 */
int guiprocregCmd(
    ClientData cd,
    Tcl_Interp *interp,
    int argc,
    char *argv[])
{
  Tcl_CmdInfo tci;
  int size;

  if (argc < 2) {
    Tcl_AppendResult(interp, "wrong # args:  should be \"",
                argv[0], " proc-name\"", (char *) NULL);
    return TCL_ERROR;
  }

  /*
   * Check whether the proc named argv[1] exists or not.
   */
  if (Tcl_GetCommandInfo(interp, argv[1], &tci) == 0) {
    Tcl_AppendResult(interp, "proc \"",
                argv[1], "\" does not exist.", (char *) NULL);
    return TCL_ERROR;
  }

  size = strlen(argv[1]) + 2;
  mymempreserve(guiprocstr, guiproc_sz, size, 128);
  memcpy(guiprocstr, argv[1], size - 2);
  guiprocstr[size - 2] =' ';
  guiprocstr[size - 1] ='\0';
  return TCL_OK;
}

int guiproc_strs(char *s, ...)
{
  va_list args;
  char *bufptr, *arg;
  int retval, size = strlen(s);
  static int isize = 0;
  static char *ibuf = NULL;

  va_start(args, s);
  while (arg = va_arg(args, char *)) {
    size += strlen(arg) + 1;
  }
  va_end(args);

  mymempreserve(ibuf, isize, size + 2, 1024);

  bufptr = ibuf;

  va_start(args, s);
  *bufptr++ = '{';
  arg = s;
  /* copy the first arg */
  while (*bufptr++ = *arg++)
      ;
  *(bufptr-1) = ' ';
  /* copy the remaining args */
  while (arg = va_arg(args, char *)) {
    while (*bufptr++ = *arg++);
    *(bufptr - 1) = ' ';
  }
  *(bufptr - 1) = '}';
  *bufptr = '\0';
  va_end(args);

/* fprintf(stderr, "gpstrs='%s'\n", ibuf); */

  retval = guiproc(ibuf);
  return retval;
}

int guiproc(char *argvstr)
{
  int len, glen;
  static int ibufsize = 0;
  static char *ibuf;

  if (NULL == argvstr) {
    return TCL_ERROR;
  }
  if (NULL == guiprocstr) {
    return GUI_NOPROC;	/* proc is not defined, yet. */
  }
  if (TCL_OK != Tcl_VarEval(G_interp, guiprocstr, argvstr, NULL)) {
    char *msg;
    msg = Tcl_GetVar(G_interp, "errorInfo", TCL_GLOBAL_ONLY);
    if (msg == NULL) {
	msg = G_interp->result;
    }
    fprintf(stderr, "%s\n", msg);
    Tcl_Eval(G_interp, "exit 1");
    exit(1);
  }

  return TCL_OK;
}

/* THIS routine seems to be stupid. But I cannot do other method to avoid
 * compile error which causes q.C's free().
 */

int myfree(buf)
char *buf;
{
  free(buf);
  return 0;
}
