#include <iostream.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <malloc.h>
#include <string.h>
#include <sys/types.h> 
#include <sys/time.h>
#include <math.h>
#include <tcl.h>
#include "matrix.h"
#include "unify.h"

const int MAXSYMLEN  = 42;
const int MAXRULES = 2000;
//const int MAXNONTERMS  = 1000;
//const int MAXTERMS  = 1000;
const int MAXSYMBOLS  = 1800;
const int MAXRH = 10;
//const int TRUE = 1;
//const int FALSE = 0;
const int NORULE = -1;

const int MAXINDEX = 200;
extern int IndexCount[MAXSYMBOLS];
extern int IndexTable[MAXSYMBOLS][MAXINDEX];
extern int LeftIndexStart[MAXSYMBOLS];
extern int LeftIndexEnd[MAXSYMBOLS];

typedef unsigned int uint;

void error(const char* fmt ...);

class Grammar;
class SymbolCollection;
class FollowSet;
class NonTermIterator;
class TermIterator;
class Bigram;
class RuleCollection;
class Tree;
class Arc;
class Chart;
class ChartParser;
class Tree;
class Chart;
class Chart;
class Bigram;
class ChartParser;

class Symbol {
	int	id;
  	char 	Name[MAXSYMLEN];
	int	terminal;
public:
		Symbol(char*);
		Symbol(int);
		Symbol();
		~Symbol();
		Symbol(char*,int);
	int	IsTerminal()   {return terminal;}
	int	IsNonTerminal()  {return (!terminal);}
	int	ID()	{return id;}
	char 	*Stringify();
	char 	*NoQuotesStringify();
	void	print(ostream &os=cout);
	void	Reset();
	friend	SymbolCollection;
	friend Grammar;
	int operator !=(Symbol s2)
 		{return (terminal!=s2.terminal)||(id != s2.id);}
	int operator ==(Symbol s2)
		{return ((terminal==s2.terminal)&&((id != -1) && (id == s2.id)));}
			
};

class SymbolCollection {
	Symbol		Symbols[MAXSYMBOLS];
	int		iTerms;
	int		iNonTerms;
	int		iSymbols;
	Tcl_HashTable		HashTable;
	int	LastSym;
public:
			SymbolCollection();
	Symbol * 		AddSymbol(char *);
	int 		NSymbols();
	int		NTerms();
	int 		IsSymbol(char *);
	Symbol* 	GetSymbol(int num);
	void		SetLastSym(int);
	friend Grammar;
	friend	Symbol;
	friend	NonTermIterator;
	friend	TermIterator;
	friend	Bigram;
};

class RuleIterator;
class PostOrderTreeIterator;
class PreOrderTreeIterator;

class Rule {
	Symbol** rhs;
	double	prob;
	int	iNumChildren;
	char *  Sem;
	int	RuleNum;
	ConstraintList *Constraints;
public:
	Symbol 	*lhs;
	double	RuleProb()  {return prob;}
	int	RuleID()  {return RuleNum;}
		Rule(char *);
		Rule();
	Rule & operator=(Rule&);
		~Rule();
	void	RuleCleanup();
	void	print();
	int	IsLexical();
	friend	RuleIterator;
	friend	RuleCollection;
	friend  Tree;
	friend	Arc;
	friend	Chart;
	friend	ChartParser;
	friend	Grammar;
};

int 	GetNextToken(char *, char*&);
inline int 	isspace(char);

class RuleCollection {
	int	iRules;
	Rule	*Rules[MAXRULES];
public:
		RuleCollection();
	void	AddRule(Rule*);
	int 	NRules();
	Rule * 	GetRule(int );
	void	Reset();
	friend	RuleIterator;
};


class RuleIterator{
	int	current;
	RuleCollection *c;
public:
		RuleIterator(RuleCollection*,Symbol *);
		RuleIterator(RuleCollection*);
		RuleIterator(Grammar * g);
	void 	Reset();
	void 	Reset(Symbol *);
	Rule	*operator()();
	Rule	*operator()(Symbol*);
	Rule * NextRuleByRhs0(Symbol *);
	Rule 	*NextLexLhs(Symbol*);
};

class NonTermIterator{
	int	current;
	SymbolCollection *c;
public:
		NonTermIterator(SymbolCollection * );
	void 	NonTermIterator::Reset();
	Symbol *NonTermIterator::Next();
};

class TermIterator{
	int	current;
	SymbolCollection *c;
	int	LastSym;
public:
		TermIterator(SymbolCollection * );
	void 	TermIterator::Reset();
	Symbol *TermIterator::Next();
};

class Node {
	Symbol	*val;
	Node **  children;
	int	iNumChildren;
	double	prob;
	int	RuleNum;
public:
		Node();
		Node(Symbol*,int);
		Node(Symbol*,double,int);
	        Node(Node*);
		~Node();
	void	AddChildren(Symbol**,int);
	Node	*Copy();
	Node 	*GetLowestNode(int);
	Node	*LeftChild();
	Node	*RightChild();
	Node	*GetNonTerminal(int);
	//void 	CountRulesUsed(int *);
	Symbol	*GetVal();
	int	IsLeafNode() { return (iNumChildren == 0);}
	friend	PostOrderTreeIterator;
	friend	PreOrderTreeIterator;
	friend	Tree;
	friend	Chart;
};


class Tree {
	Node	*Root;
	double	TreeProb;
	int	SentenceLength;
      public:
	Dag     *ParentDag;
      private:
	Dag     **ChildDag;
	SubstList *SL;
	ConstraintList *Constraints;
	ConstraintIterator *ConstraintIndex;
public:
		Tree();
		Tree(Symbol*);
		Tree(Node*);
                Tree(Symbol*, Tree *, int, int, ConstraintList *);
	        ~Tree();
	void	ConjoinChild(int dot, Tree *t);
	Tree 	*Copy();
	Symbol	*GetNonTerminal(int);
	int	GetSentence(char *);
	double	GetProb();
	int	GetLen();
	Node	*GetRoot();
	int	ExpandNode(int, Rule*);
	void	print();
	void	CountRulesUsed(int *);
#ifdef TATEYAMA
	void	output2ptw(FILE *);
#endif
	friend	Chart;
	int     checkConstraints(int,int);
	int     unifyChildren(int);
};
	

typedef Node * STACKTYPE;
const int MAXSTACK = 200;

class Stack {
	STACKTYPE	TheStack[MAXSTACK];
	int		index;
public:
			Stack();
	void		Push(STACKTYPE);
	STACKTYPE	Pop();
	STACKTYPE	Peek();
};


class PostOrderTreeIterator {
	Stack s;
	Node marker;
public:
		PostOrderTreeIterator();
		PostOrderTreeIterator(Node *);
	void	operator=(Node *);
	Node	*operator()();
};
	
class PreOrderTreeIterator {
	Stack s;
	Node marker;
public:
		PreOrderTreeIterator();
		PreOrderTreeIterator(Node *);
	void	operator=(Node *);
	Node	*operator()();
	int	IsParen(Node *);
};
	
class LeafIterator {
	PostOrderTreeIterator next;
public:
		LeafIterator(Node*);
	Node *	operator()();
};



class Grammar {
	friend		Symbol;
	friend		FollowSet;
	friend		Bigram;
	RuleCollection	Productions;
	Symbol		*Start;
	SymbolCollection Symbols;
	char		LastSentence[200];
	double		LastProb;
	int		LastLength;
	int 		visitlist[MAXRULES];
	void		StoreRule(Rule r);
	Rule		*GetRandomRule(Symbol*);
	MATRIX		*LeftCorner;
	char		Header[200];

public:
		Grammar(char *, ...);
		~Grammar();
	void	print();
	void	AddRule(char *);
	int 	NRules();
	Rule * 	GetRule(int );
	int	ReadFile(char*);
	double	EmpiricalEntropy();
	double	GrammarPerplexity();
	void	GenerateRandomSentence();
	void	GetSentence(char *);
	double	GetProb();
	int	GetRulesWithRhs(Symbol*,Rule**);
	int	GetRulesWithLhs(Symbol*,Rule**);
	Symbol	*GetStart();
	void	CheckRules();
	void	ComputeTerminalExpansions(Symbol *, int *);
	void 	Reset();
	void 	ResetVisitList();
	void 	InitIndexTable();
	friend	RuleIterator;
	friend	ChartParser;
};
ostream& operator<<(ostream&s, Symbol S);
ostream& operator<<(ostream&s, Symbol *S);
