#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include "libpq.h"
#include <ctype.h>

static char OutputBuf[9000];
char * query(char *QueryBuf);


#include "gui.h"

char *
query(char *QueryBuf)
{
	PortalBuffer *p;
	int i,k,g,t,n,m;
	int nameindex, streetnumindex, streetnameindex, distindex, costindex;
	char buf[BUFSIZ];
	char tempbuf[BUFSIZ];

	buf[0] = 0;
	
	/* specify the database to access */
	PQsetdb("pnambic");

	/* start a transaction block */
	PQexec("begin");

	if(strlen(QueryBuf) == 0) {
		sprintf(OutputBuf,"Sorry, there was a problem with your query. Please try again.\n");
		return(OutputBuf);
	}
	/* fetch instances from the REST class */
	PQexec(QueryBuf);
	PQexec("fetch all in rportal");

	/* examine all the instances fetched. */
	p = PQparray("rportal");
	if (p)
		g = PQngroups(p);
	else {
		g = 0;
		sprintf(OutputBuf,"Sorry, there was a problem with your query. Please try again.\n");
		return(OutputBuf);
	}
	t = 0;

	for(k=0;k<g;k++){
		sprintf(OutputBuf,"\nHere are the restaurants matching your query:\n\n");
		strcat(OutputBuf,"@bold(Name)                      | @bold(Address)                |  @bold(Cost) | @bold(Min. from ICSI)\n");
		strcat(OutputBuf,"---------------------------------------------------------------------------\n");
		n = PQntuplesGroup(p,k);
		m = PQnfieldsGroup(p,k);

		/* get the index of the name field */
		nameindex = PQfnumberGroup(p,k,"name");
		/* get the index of the streetnumber field */
		streetnumindex = PQfnumberGroup(p,k,"street_num");
		/* get the index of the streetname field */
		streetnameindex = PQfnumberGroup(p,k,"street_name");
		/* get the index of the distance field */
		distindex = PQfnumberGroup(p,k,"minutes");
		/* get the index of the cost field */
		costindex = PQfnumberGroup(p,k,"cost");

		/* print out the instances */
		/* if no restaurants match, say so */
		guiproc("delrl");
		if(n==0){
			sprintf(OutputBuf,"Sorry, no restaurants match your query.\n");
			return(OutputBuf);
		} else {
			/* if there are restaurants that match, print them */
			for(i=0;i<n;i++){
				sprintf(tempbuf,"%s %s",
					PQgetvalue(p,t+i,streetnumindex),
					PQgetvalue(p,t+i,streetnameindex));
				sprintf(buf,"@bold(%25s) | %22s | %5s | %s\n",
					PQgetvalue(p,t+i,nameindex),
					tempbuf,
					PQgetvalue(p,t+i,costindex),
					PQgetvalue(p,t+i,distindex));
				strcat(OutputBuf,buf);
				guiproc_strs("addrl",
					"\"", PQgetvalue(p,t+i,nameindex), "\"",
					"\"", tempbuf,"\"",
				        "\"", PQgetvalue(p,t+i,costindex), "\"",
					"\"", PQgetvalue(p,t+i,distindex), "\"",
					0);
			}
			strcat(OutputBuf,"\n@italic(You may ask for additional information on any of the above restaurants).\n");
			guiproc_strs("app", "You may ask for additional information on any of the above restaurants.", 0);
		}
		t+=n;
	}

	/* close the portal. */
	PQexec("close rportal");

	/* end the transaction block */
	PQexec("end");

	/* finish execution */
	PQfinish();

	return(OutputBuf);
}

extern "C" int myfree(char *);

char * RestInfo(char *RestName)
{
  PortalBuffer *p;
  int i,k,g,t,n;
  int nameindex, streetnumindex, streetnameindex, distindex, costindex;
  int monindex,tueindex,wedindex,thuindex,friindex,satindex,sunindex;
  int commentsindex,otypeindex,phoneindex,xstreetindex;
  int typeindex;
  char buf[BUFSIZ];
  char costbuf[BUFSIZ];
  char tmpbuf[BUFSIZ];
  int value = -1;
  char *tatebuf;

  buf[0] = 0;
	
  /* specify the database to access */
  PQsetdb("pnambic");

  /* start a transaction block */
  PQexec("begin");

  /* fetch instances from the REST class */
  PQexec("retrieve portal rportal (REST.all) sort by name");
  PQexec("fetch all in rportal");

  /* examine all the instances fetched. */
  p = PQparray("rportal");
  g = PQngroups(p);
  t = 0;

  for(i = 0; i < strlen(RestName); i++)  {
	RestName[i] = toupper(RestName[i]);
	if (RestName[i] == '-')
		RestName[i] = ' ';
  }
#ifdef DEBUG
	printf("restname is %s\n",RestName);
	printf("n is %d\n",n);
#endif

  nameindex = PQfnumberGroup(p,0,"name");
  n = PQntuplesGroup(p,0);
  for(k=0;k<n;k++){
#ifdef DEBUG
	printf("comparing with %s\n",PQgetvalue(p,k,nameindex));
#endif
    if (!strcmp(PQgetvalue(p,k,nameindex),RestName)) {
	value = k;
	break;
    }
  }
  if (value == -1) {
	  sprintf(OutputBuf,"Sorry, no restaurants match your query.\n");
	  return(OutputBuf);
  }

  for(k=0;k<g;k++){


    n = PQntuplesGroup(p,k);
    /* if no restaurants match, say so */
    if(value >= n || value < 0){
      sprintf(OutputBuf,"Sorry, no restaurants match your query.\n");
      return(OutputBuf);
    }
    sprintf(OutputBuf,"\nThis is the information available on the restaurant you requested:\n\n");

    /* get the index of the name field */
    nameindex = PQfnumberGroup(p,k,"name");
    /* get the index of the type field */
    typeindex = PQfnumberGroup(p,k,"rest_type");
    /* get the index of the other_types field */
    otypeindex = PQfnumberGroup(p,k,"other_types");		
    /* get the index of the streetnumber field */
    streetnumindex = PQfnumberGroup(p,k,"street_num");
    /* get the index of the streetname field */
    streetnameindex = PQfnumberGroup(p,k,"street_name");
    /* get the index of the distance field */
    distindex = PQfnumberGroup(p,k,"minutes");
    /* get the index of the cost field */
    costindex = PQfnumberGroup(p,k,"cost");
    /* get the index of the comments field */
    commentsindex = PQfnumberGroup(p,k,"comments");
    /* get the index of the cross street field */
    xstreetindex = PQfnumberGroup(p,k,"cross_street");
    /* get the index of the phone number field */
    phoneindex = PQfnumberGroup(p,k,"phonenumber");    

    /* get the day of week indexes */
    monindex = PQfnumberGroup(p,k,"mon");
    tueindex = PQfnumberGroup(p,k,"tue");
    wedindex = PQfnumberGroup(p,k,"wed");
    thuindex = PQfnumberGroup(p,k,"thu");
    friindex = PQfnumberGroup(p,k,"fri");
    satindex = PQfnumberGroup(p,k,"sat");
    sunindex = PQfnumberGroup(p,k,"sun");

    sprintf(buf,"Name           : @bold(%s)\n",PQgetvalue(p,value,nameindex));
    strcat(OutputBuf,buf);
    sprintf(buf,"Type of Food   : @bold(%s)\n",PQgetvalue(p,value,typeindex));
    strcat(OutputBuf,buf);
    sprintf(buf,"Other types    : @bold(%s)\n",PQgetvalue(p,value,otypeindex));
    strcat(OutputBuf,buf);
    sprintf(buf,"Address        : @bold(%s %s)\n",
	    PQgetvalue(p,value,streetnumindex),
	    PQgetvalue(p,value,streetnameindex));
    strcat(OutputBuf,buf);
    sprintf(buf,"Cross Street   : @bold(%s)\n",PQgetvalue(p,value,xstreetindex));
    strcat(OutputBuf,buf);
    sprintf(buf,"Phone number   : @bold(%s)\n",PQgetvalue(p,value,phoneindex));
    strcat(OutputBuf,buf);

    sprintf(costbuf,"%s",PQgetvalue(p,value,costindex));
    if(!strcmp(costbuf,"$")) 
      sprintf(buf,"Cost           : @bold(less than $6.00 per person)\n");
    else if(!strcmp(costbuf,"$$"))
      sprintf(buf,"Cost           : @bold($6.00-$12.00 per person)\n");
    else if(!strcmp(costbuf,"$$$"))
      sprintf(buf,"Cost           : @bold($12.00-$16.00 per person)\n");
    else 
      sprintf(buf,"Cost           : @bold(more than $16.00 per person)\n");

    strcat(OutputBuf,buf);
    tatebuf = strdup(buf);
    if(!strcmp(costbuf,"$")) 
      sprintf(tatebuf,"less than $6.00 per person");
    else if(!strcmp(costbuf,"$$"))
      sprintf(tatebuf,"$6.00-$12.00 per person");
    else if(!strcmp(costbuf,"$$$"))
      sprintf(tatebuf,"$12.00-$16.00 per person");
    else 
      sprintf(tatebuf,"more than $16.00 per person");

    sprintf(buf,"Min. walk from ICSI: @bold(%s)\n",PQgetvalue(p,value,distindex));
    strcat(OutputBuf,buf);
    sprintf(buf,"Hours:(B=breakfast,L=lunch,D=dinner,N=night)\n");
    strcat(OutputBuf,buf);
    sprintf(buf,"Mon     Tue     Wed     Thu     Fri     Sat     Sun\n");
    strcat(OutputBuf,buf);
    sprintf(buf,"---     ---     ---     ---     ---     ---     ---\n");
    strcat(OutputBuf,buf);
    sprintf(buf,"@bold(@bold(%3s))     @bold(%3s)     @bold(%3s)     @bold(%3s)     @bold(%3s)     @bold(%3s)     @bold(%3s)\n",
	    PQgetvalue(p,value,monindex),
	    PQgetvalue(p,value,tueindex),
	    PQgetvalue(p,value,wedindex),
	    PQgetvalue(p,value,thuindex),
	    PQgetvalue(p,value,friindex),
	    PQgetvalue(p,value,satindex),
	    PQgetvalue(p,value,sunindex));
    strcat(OutputBuf,buf);
    for (i=0;i<BUFSIZ;i++)
	buf[i]=0;
    strcpy(tmpbuf,PQgetvalue(p,value,commentsindex));
    guiproc_strs("restinfo",
		 "\"", PQgetvalue(p,value,nameindex), "\"",
		 "\"", PQgetvalue(p,value,typeindex), "\"",
		 "\"", PQgetvalue(p,value,otypeindex), "\"",
		 "\"", PQgetvalue(p,value,streetnumindex),
		       PQgetvalue(p,value,streetnameindex), "\"",
		 "\"", PQgetvalue(p,value,xstreetindex), "\"",
		 "\"", PQgetvalue(p,value,phoneindex), "\"",
		 "\"", tatebuf, "\"",
		 "\"", PQgetvalue(p,value,distindex), "\"",
		 "\"", tmpbuf, "\"",
		 "\"", PQgetvalue(p,value,monindex), "\"",
		 "\"", PQgetvalue(p,value,tueindex), "\"",
		 "\"", PQgetvalue(p,value,wedindex), "\"",
		 "\"", PQgetvalue(p,value,thuindex), "\"",
		 "\"", PQgetvalue(p,value,friindex), "\"",
		 "\"", PQgetvalue(p,value,satindex), "\"",
		 "\"", PQgetvalue(p,value,sunindex), "\"",
		 0);
    myfree(tatebuf);

    if (strlen(tmpbuf) < 62)
	    sprintf(buf,"\n\nComments       : @bold(%s)\n",tmpbuf);
    else {
	    sprintf(buf,"\n\nComments       : ");
		strcat(buf,"@bold(");
		strncat(buf,tmpbuf,62);
		strcat(buf,")");
	    strcat(buf,"\n");
	    for (i=62;i<strlen(tmpbuf);i+=79) {
		strcat(buf,"@bold(");
		strncat(buf,tmpbuf+i,79);
		strcat(buf,")");
	    strcat(buf,"\n");
	    }
	    //strcat(buf,"\n");
    }
    strcat(OutputBuf,buf);
    t+=n;
  }

  /* close the portal. */
  PQexec("close rportal");

  /* end the transaction block */
  PQexec("end");

  /* finish execution */
  PQfinish();

  return(OutputBuf);
}

