/*******************************************************************************

	ruleit.C:
		
		Contains class:  RuleIterator

		The only non-obvious thing going on is the
		LeftIndexStart and LeftIndexEnd arrays, which
		are used (for efficiency) to mark where a collection of rules
		expanding the same nonterminal (i.e. with the same left-hand
		side) begins and ends.  For each nonterminal,
		LeftIndexStart will store the rule number of the
		first expansion MINUS 1, while LeftIndexEnd
		stores the rule number of the last expansion.
		
		The rules needn't be completely consecutive, we check each time
		anyhow, but obviously the whole thing will be more efficient
		the more consecutive they are.
		
		The tables are allocated in grammar.C, and are set 
		up in Grammar::InitIndexTable().

		Author:		Dan Jurafsky
		Date:		September, 1992
		History:	DSJ	5-27-93	Added header

******************************************************************************/
#include "grammar.h"


RuleIterator::RuleIterator(Grammar * g)
{
	c = &(g->Productions);
	current = -1;
}	

RuleIterator::RuleIterator(RuleCollection * r,Symbol *s)
{
	c = r;
	current = LeftIndexStart[s->ID()]-1;
	if (current == -2) current++;
}	

RuleIterator::RuleIterator(RuleCollection * r)
{
	c = r;
	current = -1;
}	

void RuleIterator::Reset()
{
	current = -1;
}

void RuleIterator::Reset(Symbol *s)
{
	current = LeftIndexStart[s->ID()]-1;
}

Rule * RuleIterator::operator()()
{	
	if (current == -1) 
		current = 0;
	if (current < c->iRules)
		return c->Rules[current++];
	else
		return 0;
}
	
Rule * RuleIterator::operator()(Symbol *SpecLhs)
{
	uint found = FALSE;

	/* This is only somewhat sneaky.  If the previous guy
		was current, we start this search from the one
		past current.  The sneaky part is that the init
		state sets current to -1, and -1 + 1 = 0, how convenient,
		or if the init state specified a particular symbol,
		we start from LeftIndexStart for that symbol,
		and stop at LeftIndexEnd.*/
			
	current++;


	for ( ; current <= LeftIndexEnd[SpecLhs->ID()]; current++) {
		if (c->Rules[current]->lhs == SpecLhs) {
			found = TRUE;
			break;
		}
	}
	if (found)
		return c->Rules[current];
	else 
		return 0;
}

Rule * RuleIterator::NextLexLhs(Symbol *SpecLhs)
{
	uint found = FALSE;

	/* This is only somewhat sneaky.  If the previous guy
		was current, we start this search from the one
		past current.  The sneaky part is that the init
		state sets current to -1, and -1 + 1 = 0, how convenient.*/
			
	current++;

	for ( ; current <= LeftIndexEnd[SpecLhs->ID()]; current++) {
		if (current >= c->iRules) {
			error("rule count error for symbol %s, LeftIndexEnd %d, current %d,numrules %d!!!!",SpecLhs->NoQuotesStringify(),LeftIndexEnd[SpecLhs->ID()], current,c->iRules);
		}
		if ((c->Rules[current]->lhs == SpecLhs) &&
			(!c->Rules[current]->IsLexical())){
			found = TRUE;
			break;
		}
	}
#ifdef NOTDEF
	for ( ; current < c->iRules; current++) {
		if ((c->Rules[current]->lhs == SpecLhs)  &&
			(!c->Rules[current]->IsLexical())){
			found = TRUE;
			break;
		}
	}
#endif
	if (found)
		return c->Rules[current];
	else
		return 0;
}

Rule * RuleIterator::NextRuleByRhs0(Symbol *rhs)
{
	int symno = rhs->ID();

	if (++current < IndexCount[symno]) 
		return c->Rules[IndexTable[symno][current]]; 

	return 0;
}
