#include "semant.h"
#include "unify_hash.h"
#include <ctype.h>

#define MAXBUF 1000

char *process_money(Dag *root, char *buff);
char *process_distance(Dag *root, char *buff);
char *process_cnjct(Dag *root, char *buff);
/* handle 'and' and 'or' */

char *process_NP(Dag *root, char *buff) {
  char temp[MAXBUF];
  Dag *d = NULL;
  Dag *cat = root->value_of(StringTableEntry("category"));
  if (cat && (cat->get_sym() == StringTableEntry("or") ||
      cat->get_sym() == StringTableEntry("and")))
    return process_cnjct(root, buff);
  
  Path tmpp;
  tmpp.add("category");
  Dag *category = root->get_path(&tmpp);
  tmpp.clear();
  tmpp.add("class");
  Dag *classd = root->get_path(&tmpp);
  if (category && !strcmp(category->get_sym(), "money"))
    return process_money(root, buff);
  if ((category && !strcmp(category->get_sym(), "distance")) || 
      (classd && !strcmp(classd->get_sym(), "distance")))
    return process_distance(root, buff);
  /* if this is not about money or distance, it's probably about food */

  tmpp.clear();
  tmpp.add("nationality");
  if (d = root->get_path(&tmpp)) {
    char natstr[MAXBUF];
    strcpy(natstr, d->get_sym());
    for (char *p = natstr; *p; p++)
      *p = toupper(*p);
    sprintf(temp, " ( REST.rest_type = \"%s\" or REST.other_types ~ \"%s\")",
	    natstr, natstr);
    strcat(buff, temp);
  }
  tmpp.clear();
  tmpp.add("meal-type");
  if (d = root->get_path(&tmpp)) {
    char *str = d->get_sym();
    sprintf(temp, " \"[%c]\" ", toupper(*str));
    strcat(buff, temp);
  }
  tmpp.clear();
  tmpp.add("rest-name");
  if (d = root->get_path(&tmpp)) {
    char *str = d->get_sym();
    sprintf(temp, " (REST.restname %s)", str);
    strcat(buff, temp);
  }
 return buff;
}

char *process_operator(Dag *root, char *buff) {
  if (!root || !root->is_atomic())
    return strcat(buff, " <= ");;
  char *sym = root->get_sym();
  if (!strcmp(sym, "less-than"))
    strcat(buff, " < ");
  else if (!strcmp(sym, "more-than"))
    strcat(buff, " > ");
  else
    strcat(buff, " <= ");
  return buff;
}

char *process_amount(Dag *root, char *buff) {
  char temp[MAXBUF];
  Path tmpp;
  tmpp.add("amount");
  Dag *amount = root->get_path(&tmpp);
  if (!amount)
    return NULL;
  if (amount->is_atomic()) {
    tmpp.clear();
    tmpp.add("amount-mod");
    Dag *mod = root->get_path(&tmpp);
    process_operator(mod, buff + strlen(buff));
    sprintf(temp, "%s)", amount->get_sym());
    strcat(buff, temp);
  } else {
    strcat(buff, " BETWEEN ");
    Dag *from = amount->value_of(StringTableEntry("from"));
    Dag *to = amount->value_of(StringTableEntry("to"));
    if (!from || !to) {
      cerr << "From/to features not found in DAG for amount\n";
      return NULL;
    }
    strcat(buff, from->get_sym());
    sprintf(temp, " %s)", to->get_sym());
    strcat(buff, temp);
  }
  return buff;
}

char *process_distance(Dag *root, char *buff) {
  char temp[MAXBUF];
  Dag *cat = root->value_of(StringTableEntry("class"));
  if (cat && (cat->get_sym() != StringTableEntry("distance")))
    return buff;
  
  Path tmpp;
  tmpp.add("amount");
  Dag *amount = root->get_path(&tmpp);
  tmpp.clear();
  tmpp.add("dis-units");
  Dag *units = root->get_path(&tmpp);
  if (amount) {
    sprintf(temp, "(REST.%ss", units->get_sym());
    strcat(buff, temp);
    process_amount(root, buff + strlen(buff));
  }
  return buff;
}
  
char *process_money(Dag *root, char *buff) {
  Dag *d = NULL;
  Dag *cat = root->value_of(StringTableEntry("class"));
  if (cat && (cat->get_sym() != StringTableEntry("money")))
    return buff;
  
  Path tmpp;
  tmpp.add("amount");
  Dag *amount = root->get_path(&tmpp);
  if (amount) {
    char *amountstr = amount->get_sym();
    if (!strcmp(amountstr, "much"))
      strcat(buff, "(REST.cost = \"$$$$\")");
    else {
      strcat(buff, "(REST.dollars");
      tmpp.clear();
      tmpp.add("amount-mod2");
      Dag *mod = root->get_path(&tmpp);
      if (mod && !strcmp(mod->get_sym(), "total")) {
	strcat(buff, "tot");
      }
      process_amount(root, buff + strlen(buff));
    }
  }
  return buff;
}

char *process_cnjct(Dag *root, char *buff) {
  Path catp, cp1, cp2;
  catp.add("category");
  cp1.add("cons1");
  cp2.add("cons2");
  Dag *d1 = root->get_path(&cp1);
  Dag *d2 = root->get_path(&cp2);
  char *cnjct = NULL;
  Dag *cnjctd = root->get_path(&catp);
  if (cnjctd)
    cnjct = cnjctd->get_sym();
  strcat(buff, "(");
  if (d1)
    process_NP(d1, buff + strlen(buff));
  char temp[MAXBUF];
  sprintf(temp, " %s ", cnjct);
  strcat(buff, temp);
  if (d2)
    process_NP(d2, buff + strlen(buff));
  strcat(buff, ")");
  return buff;
}

char *process_VP(Dag *root, char *buff) {
  char temp[MAXBUF];
  int not_flag = 0;        // for negative sentences

  Path tmpp;
  tmpp.add("content");
  Dag *content = root->get_path(&tmpp);

  Path catp;
  catp.add("category");
  Dag *cat = root->get_path(&catp);
  Dag *contentcat = content->get_path(&catp);
  tmpp.clear();
  tmpp.add("patient");
  Dag *patient = root->get_path(&tmpp);
  tmpp.clear();
  tmpp.add("communicated");
  Dag *communicated = root->get_path(&tmpp);
  char *catstr;
  /* day of week */
  Dag *day = root->value_of(StringTableEntry("day"));
  if (day && day->is_atomic()) {
    sprintf(buff + strlen(buff), "(REST.%.3s) ", day->get_sym());
  }
  if (cat)
    catstr = cat->get_sym();
  if (!cat || !strcmp(catstr, "food") || !strcmp(catstr, "money") ||
      !strcmp(catstr, "distance") || !strcmp(catstr, "restaurant"))
    return process_NP(root, buff);    /* must be a fragment - not a real VP at
				      /* all */
  if (!strcmp(catstr, "restart")) {
    strcat(buff, " RESTART ");
    return buff;
  }

  Dag *polarity = root->value_of(StringTableEntry("polarity"));
  if (polarity && !strcmp(polarity->get_sym(), "negative")) {
    strcat(buff, "(NOT ");
    not_flag = 1;
  }


  if (contentcat && (content->value_of(StringTableEntry("patient")) ||
		     content->value_of(StringTableEntry("content")))) {
    // content is VP, so recurse
    return process_VP(content, buff + strlen(buff));
  } else if (content) {        
    return process_NP(content, buff + strlen(buff));
  } else if (patient) {
    return process_NP(patient, buff + strlen(buff));
  }
  if (communicated) {    // "tell me about..."
    return process_NP(communicated, buff + strlen(buff));
  }

  if (not_flag)
    strcat(buff, " )");
  
  return buff;
}

char *formulate_query(Dag *root, char *buff, int bufsize) {
  Dag *cat = root->value_of(StringTableEntry("cat"));
  char *catstr = cat->get_sym();
  Path semp;
  semp.add("head");
  semp.add("sem");
  Dag *sem = root->get_path(&semp);
  if (!sem)
    return NULL;
  strcat(buff, "{ ");
  if (!strcmp(catstr, "NP") || !strcmp(catstr, "FRAGMENT"))
    process_NP(sem, buff + strlen(buff));
  else if (!strcmp(catstr, "VP") || !strcmp(catstr, "S") || !strcmp(catstr, "ss"))
    process_VP(sem, buff + strlen(buff));
  else
    return NULL;
  strcat(buff, " }");
  if (strlen(buff) > 4)
    return buff;
  else
    return NULL;
}
