#!./backendsh -f
#!/u/tateyama/bin/wish -f
#!/usr/local/bin/wish -f
#
# backend.tcl
#
# The Tcl part of running the BeRP backend
# - works with the Tcl extensions (e.g. in libbackend.so)
#
# 1997aug06 dpwe@icsi.berkeley.edu  Comments added to legacy code
# $Header: /u/drspeech/src/berpdemo/share/RCS/berpbackend.tcl,v 1.5 1999/04/07 01:38:06 dpwe Exp $

###
### Embedding
###
# If $be_root is defined on entry, make all widgets within it (else use ".").
# If $be_inpipe is defined on entry, read decoder commands from it
#    else use stdin.

if {[info vars be_root] == ""} {
    set be_root ""
}
if {[info vars be_inpipe] == ""} {
    set be_inpipe stdin
}

if {[info commands backendloop] == ""} {
    # We're not running under backendsh - load up under plain wish
    # New installed-package version of the backend library
#    lappend auto_path /u/drspeech/$env(SPEECH_ARCH)/lib
    package require BeRPbackend
}

proc myBackend {pipe} {
    # Line of input on stdin should be a recognized word sequence from noway
    global afe_audiosrc
    set uttwords [gets $pipe]
    puts "myBackend: got $uttwords"
    if {[eof $pipe]} {
	# Disable the handler to avoid loops when guiproc calls update
	fileevent $pipe readable {}
	# Indicate to the backend that it's all over
	backendloop ""
	exit
    }
    # Make sure audio input is suspended during backend, so 
    # any audio can be played back
    $afe_audiosrc close
    backendloop $uttwords
    $afe_audiosrc open
}

fconfigure $be_inpipe -buffering line
fileevent $be_inpipe readable "myBackend $be_inpipe"

# ==================== DEFINE WIDGETS ====================

# widgets for MAIN window

# The recognized words display
set recframe $be_root.recframe
frame $recframe -border 2 -relief groove
set reclabel $recframe.reclable
label $reclabel -text "Recognized Words" -anchor nw
set rectext $recframe.text
text $rectext -relief sunken -borderwidth 2 -height 1 \
	-font -Adobe-Times-Medium-I-Normal--*-240-* \
	-state disabled
$rectext tag configure bold -font -Adobe-Helvetica-Bold-R-Normal--*-240-*
pack $reclabel $rectext -side top -fill x -padx 5 -pady 5
pack $recframe -side top -fill x -expand 1 -padx 5 -pady 5

# The berp response panel
set resframe $be_root.resframe
frame $resframe -border 2 -relief groove
set reslabel $resframe.reslabel
label $reslabel -text "BeRP Response" -anchor nw
set response $resframe.response
text $response  -relief sunken -borderwidth 2 -padx 5 -pady 5 -height 4 \
	-font -Adobe-Times-Medium-R-Normal--*-240-*
$response insert end "Welcome to BeRP - How may I help you?\nREMEMBER: you must click on the RECORD button before you begin speaking.\nREADY...\n"
pack $reslabel $response -side top -fill x -padx 5 -pady 5
pack $resframe -side top -fill x -expand 1 -padx 5 -pady 5

# The query display
set qboxframe $be_root.qboxframe
frame $qboxframe -border 2 -relief groove

set qboxlabel $qboxframe.label
label $qboxlabel -text "Your Specifications" -anchor nw

set qboxqframe $qboxframe.qbox
frame $qboxqframe
set qboxframe1 $qboxqframe.frame1
frame $qboxframe1
label $qboxframe1.type -text "Type of Food" -anchor ne
label $qboxframe1.dist -text "Distance"  -anchor ne
label $qboxframe1.cost -text "Cost" -anchor ne
label $qboxframe1.day -text "Day" -anchor ne
label $qboxframe1.meal -text "Meal" -anchor ne
pack $qboxframe1.type $qboxframe1.dist $qboxframe1.cost \
			$qboxframe1.day $qboxframe1.meal -fill x

set qboxframe2 $qboxqframe.frame2
frame $qboxframe2
label $qboxframe2.type -text "NONE" -relief sunken -width 20
label $qboxframe2.dist -text "NONE"  -relief sunken
label $qboxframe2.cost -text "NONE"  -relief sunken
label $qboxframe2.day -text "NONE"  -relief sunken
label $qboxframe2.meal -text "NONE"  -relief sunken
pack  $qboxframe2.type \
	$qboxframe2.dist \
	$qboxframe2.cost \
	$qboxframe2.day \
	$qboxframe2.meal -expand 1 -fill x -padx 20

pack $qboxframe1 -side left
pack $qboxframe2 -expand 1 -fill x -side right

pack $qboxlabel  -side top -fill x -padx 5 -pady 5
pack $qboxqframe -side top -fill x -padx 20 -pady 10

pack $qboxframe -side left -fill both -expand 0 -padx 5 -pady 5

#wm maxsize . 1000 1000
#wm geometry . +0+0

#
# the restaurant list window's definitions.
#

set restlist .restlist
toplevel $restlist
set rlbaseframe $restlist.baseframe
frame $rlbaseframe
	set rlnameframe $rlbaseframe.nameframe
	frame $rlnameframe
		set rlnamelabel $rlnameframe.namelabel
		label $rlnamelabel -text "Name" -width 20
		pack $rlnamelabel -expand 1 -fill x
	set rladdrframe $rlbaseframe.addrframe
	frame $rladdrframe
		set rladdrlabel $rladdrframe.addrlabel
		label $rladdrlabel -text "Address" -width 30
		pack $rladdrlabel -expand 1 -fill x
	set rlcostframe $rlbaseframe.costframe
	frame $rlcostframe
		set rlcostlabel $rlcostframe.costlabel
		label $rlcostlabel -text "Cost" -width 5
		pack $rlcostlabel
	set rltimeframe $rlbaseframe.timeframe
	frame $rltimeframe
		set rltimelabel $rltimeframe.timelabel
		label $rltimelabel -text "Min. from ICSI" -width 15
		pack $rltimelabel
	pack $rlnameframe $rladdrframe $rlcostframe $rltimeframe \
		-expand 1 -fill x -side left

set rllabel1 $restlist.label1
label $rllabel1 -text "Here are the restaurants matching your query:"
set rllabel2 $restlist.label2
label $rllabel2 -text "You may ask for additional information on any of the above restaurants."
pack append $restlist \
	$rllabel1 {top} \
	$rlbaseframe {fillx} \
	$rllabel2 {bottom}

wm title $restlist "Restaurant List"
wm withdraw $restlist
wm geometry $restlist +0+400

#
# the restaurant information window's definitions.
#

set restinfo .restinfo
toplevel $restinfo

set riinfoframe $restinfo.iframe
frame $riinfoframe

set rilabelframe $riinfoframe.lframe
frame $rilabelframe
	set rinamelabel $rilabelframe.name
	text $rinamelabel -width 30 -height 1 -relief flat -font \
		-Adobe-Times-Medium-R-Normal--*-140-*
	$rinamelabel insert end "Name"
	$rinamelabel configure -state disabled
	set ritypelabel $rilabelframe.type
	text $ritypelabel -width 30 -height 1 -relief flat -font \
		-Adobe-Times-Medium-R-Normal--*-140-*
	$ritypelabel insert end "Type of Food"
	$ritypelabel configure -state disabled
	set riotherlabel $rilabelframe.other
	text $riotherlabel  -width 30 -height 1 -relief flat -font \
		-Adobe-Times-Medium-R-Normal--*-140-*
	$riotherlabel insert end "Other types"
	$riotherlabel configure -state disabled
	set riaddrlabel $rilabelframe.addr
	text $riaddrlabel -width 30 -height 1 -relief flat -font \
		-Adobe-Times-Medium-R-Normal--*-140-*
	$riaddrlabel insert end "Address"
	$riaddrlabel configure -state disabled
	set ricrstlabel $rilabelframe.crst
	text $ricrstlabel -width 30 -height 1 -relief flat -font \
		-Adobe-Times-Medium-R-Normal--*-140-*
	$ricrstlabel insert end "Cross Street" 
	$ricrstlabel configure -state disabled
	set riphnmlabel $rilabelframe.phnm
	text $riphnmlabel  -width 30 -height 1 -relief flat -font \
		-Adobe-Times-Medium-R-Normal--*-140-*
	$riphnmlabel insert end "Phone number"
	$riphnmlabel configure -state disabled
	set ricostlabel $rilabelframe.cost
	text $ricostlabel -width 30 -height 1 -relief flat -font \
		-Adobe-Times-Medium-R-Normal--*-140-*
	$ricostlabel insert end "Cost"
	$ricostlabel configure -state disabled
	set ritimelabel $rilabelframe.time
	text $ritimelabel  -width 30 -height 1 -relief flat -font \
		-Adobe-Times-Medium-R-Normal--*-140-*
	$ritimelabel insert end "Min. walk from ICSI"
	$ritimelabel configure -state disabled
	set ricomtlabel $rilabelframe.comt
	text $ricomtlabel -width 30 -height 1 -relief flat -font \
		-Adobe-Times-Medium-R-Normal--*-140-*
	$ricomtlabel insert end "Comments"
	$ricomtlabel configure -state disabled
	set ripady {pady 1}
	pack append $rilabelframe \
		$rinamelabel $ripady \
		$ritypelabel $ripady \
		$riotherlabel $ripady \
		$riaddrlabel $ripady \
		$ricrstlabel $ripady \
		$riphnmlabel $ripady \
		$ricostlabel $ripady \
		$ritimelabel $ripady \
		$ricomtlabel {expand filly}
#	pack configure $rilabelframe -pady 1
#		
#	pack $rinamelabel $ritypelabel $riotherlabel $riaddrlabel \
#		$ricrstlabel $riphnmlabel $ricostlabel $ritimelabel \
#		-expand 1 -pady 1
#	pack append $rilabelframe $ricomtlabel {expand filly}

set ridataframe $riinfoframe.ridataframe
frame $ridataframe
	set riname $ridataframe.name
	text $riname -relief sunken -state disabled -width 40 -height 1 \
		-font -Adobe-Helvetica-Bold-R-Normal--*-140-*
	set ritype $ridataframe.type
	text $ritype -relief sunken -state disabled -width 40 -height 1 \
		-font -Adobe-Helvetica-Bold-R-Normal--*-140-*
	set riother $ridataframe.other
	text $riother -relief sunken -state disabled -width 40 -height 1 \
		-font -Adobe-Helvetica-Bold-R-Normal--*-140-*
	set riaddr $ridataframe.addr
	text $riaddr -relief sunken -state disabled -width 40 -height 1 \
		-font -Adobe-Helvetica-Bold-R-Normal--*-140-*
	set ricrst $ridataframe.crst
	text $ricrst -relief sunken -state disabled -width 40 -height 1 \
		-font -Adobe-Helvetica-Bold-R-Normal--*-140-*
	set riphnm $ridataframe.phnm
	text $riphnm -relief sunken -state disabled -width 40 -height 1 \
		-font -Adobe-Helvetica-Bold-R-Normal--*-140-*
	set ricost $ridataframe.cost
	text $ricost -relief sunken -state disabled -width 40 -height 1 \
		-font -Adobe-Helvetica-Bold-R-Normal--*-140-*
	set ritime $ridataframe.time
	text $ritime -relief sunken -state disabled -width 40 -height 1 \
		-font -Adobe-Helvetica-Bold-R-Normal--*-140-*
	set ricomt $ridataframe.comt
	text $ricomt -relief sunken -state disabled -width 40 -height 3 \
		-font -Adobe-Helvetica-Bold-R-Normal--*-140-*
	pack $riname $ritype $riother $riaddr $ricrst $riphnm \
		$ricost $ritime $ricomt -expand 1 -fill x

pack append $riinfoframe \
	$rilabelframe {left expand filly} \
	$ridataframe {right expand fillx}

set rihoursframe $restinfo.hframe
frame $rihoursframe

	set monframe $rihoursframe.monframe
	frame $monframe
		set monlabel $monframe.label
		label $monlabel -text "Mon" -font -Adobe-Times-Medium-R-Normal--*-120-*
		set mondata $monframe.data
		label $mondata -relief sunken
		pack $monlabel $mondata -side top -expand 1 -fill x
	set tueframe $rihoursframe.tueframe
	frame $tueframe
		set tuelabel $tueframe.label
		label $tuelabel -text "Tue" -font -Adobe-Times-Medium-R-Normal--*-120-*
		set tuedata $tueframe.data
		label $tuedata -relief sunken
		pack $tuelabel $tuedata -side top -expand 1 -fill x
	set wedframe $rihoursframe.wedframe
	frame $wedframe
		set wedlabel $wedframe.label
		label $wedlabel -text "Wed" -font -Adobe-Times-Medium-R-Normal--*-120-*
		set weddata $wedframe.data
		label $weddata -relief sunken
		pack $wedlabel $weddata -side top -expand 1 -fill x
	set thuframe $rihoursframe.thuframe
	frame $thuframe
		set thulabel $thuframe.label
		label $thulabel -text "Thu" -font -Adobe-Times-Medium-R-Normal--*-120-*
		set thudata $thuframe.data
		label $thudata -relief sunken
		pack $thulabel $thudata -side top -expand 1 -fill x
	set friframe $rihoursframe.friframe
	frame $friframe
		set frilabel $friframe.label
		label $frilabel -text "Fri" -font -Adobe-Times-Medium-R-Normal--*-120-*
		set fridata $friframe.data
		label $fridata -relief sunken
		pack $frilabel $fridata -side top -expand 1 -fill x
	set satframe $rihoursframe.satframe
	frame $satframe
		set satlabel $satframe.label
		label $satlabel -text "Sat" -font -Adobe-Times-Medium-R-Normal--*-120-*
		set satdata $satframe.data
		label $satdata -relief sunken
		pack $satlabel $satdata -side top -expand 1 -fill x
	set sunframe $rihoursframe.sunframe
	frame $sunframe
		set sunlabel $sunframe.label
		label $sunlabel -text "Sun" -font -Adobe-Times-Medium-R-Normal--*-120-*
		set sundata $sunframe.data
		label $sundata -relief sunken
		pack $sunlabel $sundata -side top -expand 1 -fill x
	pack $monframe $tueframe $wedframe $thuframe $friframe \
		$satframe $sunframe -side left -expand 1 -fill x

set rilabel1 $restinfo.label1
label $rilabel1 -text "This is the information available on the restaurant you requested:" -font -Adobe-Times-Medium-R-Normal--*-140-*

set rilabel2 $restinfo.label2
label $rilabel2 -text "Hours: ('B' = breakfast, 'L' = lunch, 'D' = Dinner, N = night)" -font -Adobe-Times-Medium-R-Normal--*-120-*

pack $rilabel1 $riinfoframe $rilabel2 $rihoursframe -side top -expand 1 -fill x

wm title $restinfo "Restaurant Information"
wm withdraw $restinfo
wm geometry $restinfo +0+400


# ==================== DEFINE USEFUL PROCEDURES ====================

# This procedure display such as "hello, @bold(world)!!" string to $rectext.

proc receivetext {text} {
    global rectext
    $rectext configure -state normal
    set len [string length $text]

    $rectext delete 1.0 end
    while {1} {
	set bstart [string first "@bold(" $text]
	# set bstart [string first "(" $text]
	if {-1 == $bstart} {
	    $rectext insert end [string range $text 0 $len]
	    break
	}
	$rectext insert end [string range $text 0 [expr $bstart - 1]]
	set spos [$rectext index "end - 1 chars"]
	set bend [string first ")" $text]
	if {-1 == $bend} {
	    error "'@bold(' without ')'"
	    break;
	}
	#$rectext insert end [string range $text \
	#	[expr $bstart + 1] [expr $bend - 1]]
	$rectext insert end [string range $text \
		[expr $bstart + 6] [expr $bend - 1]]
	set epos [$rectext index "end - 1 chars"]
	$rectext tag add bold $spos $epos
	set text [string range $text [expr $bend + 1] $len]
    }
    $rectext configure -state disabled
}

set rlwidgets {}
set rlcount 0
proc addrl {name addr cost time} {
    global rlnameframe rladdrframe rlcostframe rltimeframe rlwidgets rlcount
    set tmpname $rlnameframe.name$rlcount
    label $tmpname -text $name -relief sunken
    pack append $rlnameframe $tmpname {top fillx}
    set tmpaddr $rladdrframe.addr$rlcount
    label $tmpaddr -text $addr -relief sunken
    pack append $rladdrframe $tmpaddr {top fillx}
    set tmpcost $rlcostframe.cost$rlcount
    label $tmpcost -text $cost -relief sunken
    pack append $rlcostframe $tmpcost {top fillx}
    set tmptime $rltimeframe.time$rlcount
    label $tmptime -text $time -relief sunken
    pack append $rltimeframe $tmptime {top fillx}
    lappend rlwidgets $tmpname $tmpaddr $tmpcost $tmptime
    incr rlcount
}

proc delrl {} {
    global rlwidgets rlcount
    foreach i $rlwidgets {
	destroy $i
    }
    set rlwidgets {}
    set rlcount 0
}
 
proc settext {w text} {
    $w configure -state normal
    $w delete 1.0 end
    $w insert end $text
    $w configure -state disabled
}

proc guiproc {avlist} {
    global respproc receivetext qboxframe2 response restlist restinfo \
	    riname ritype riother riaddr ricrst riphnm ricost ritime ricomt \
	    mondata tuedata weddata thudata fridata satdata sundata
    set header [lindex $avlist 0]
    set len [llength $avlist]
#    puts "guiproc: got '$avlist'"
    switch $header {
	respclr {
	    $response delete 1.0 end
	}
	forresp {
	    $response delete 1.0 end
	    $response insert end [lrange $avlist 1 $len]
	}
	addrl {
	    addrl [lindex $avlist 1] [lindex $avlist 2] [lindex $avlist 3] \
		    [lindex $avlist 4]
	}
	delrl {
	    delrl
	}
	rlwinup {
	    wm geometry $restlist +0+400
	    wm deiconify $restlist
	}
	rlwindown {
	    wm withdraw $restlist
	}
	restinfo {
	    settext $riname [lindex $avlist 1]
	    settext $ritype [lindex $avlist 2]
	    settext $riother [lindex $avlist 3]
	    settext $riaddr [lindex $avlist 4]
	    settext $ricrst [lindex $avlist 5]
	    settext $riphnm [lindex $avlist 6]
	    settext $ricost [lindex $avlist 7]
	    settext $ritime [lindex $avlist 8]
	    settext $ricomt [lindex $avlist 9]
	    $mondata configure -text [lindex $avlist 10]
	    $tuedata configure -text [lindex $avlist 11]
	    $weddata configure -text [lindex $avlist 12]
	    $thudata configure -text [lindex $avlist 13]
	    $fridata configure -text [lindex $avlist 14]
	    $satdata configure -text [lindex $avlist 15]
	    $sundata configure -text [lindex $avlist 16]
	    wm geometry $restinfo +0+400
	    wm deiconify $restinfo
	}
	rinfowdown {
	    wm withdraw $restinfo
	}
	rec {
	    receivetext [lrange $avlist 1 $len]
	}
	app {
	    $response insert end "\n"
	    $response insert end [lrange $avlist 1 $len]
	}
	qf0 {
		$qboxframe2.type configure -text [lrange $avlist 1 $len]
	}
	qf1 {
		$qboxframe2.dist configure -text [lrange $avlist 1 $len]
	}
	qf2 {
		$qboxframe2.cost configure -text [lrange $avlist 1 $len]
	}
	qf3 {
		$qboxframe2.day configure -text [lrange $avlist 1 $len]
	}
	qf4 {
		$qboxframe2.meal configure -text [lrange $avlist 1 $len]
	}
	default {
	    puts [format "Warning: I don't know such format, '%s'." \
                    $header]
	}
    }
    update
}

# ==================== OTHER WORKS ====================

# This registers "guiproc" with the C-code as the routine to which 
# items for display on the front-end are passed.
guiprocreg "guiproc"

# You can add widgets easily.
#set quit .quit
#button $quit -text quit -command {
#    destroy .
#}
#pack $quit -expand 1 -fill x
