/*
** $Header: /u/spert/src/fltvec/nfp/RCS/camfile.c,v 1.1 1997/01/25 00:13:45 davidj Exp $
**
** Cambridge compressed speech file format conversion functions.
** Version optimized for machines without floating point.
*/

#include <fltv-config.h>
#include <assert.h>
#include <stddef.h>
/* Note - using Sun SC3.0 acc, files including <assert.h> must also include
   <stdio.h> */
#include <stdio.h>
#include <math.h>
#include <fltv-defs.h>
#include <fltv-scansrc.h>


#ifdef DECL_nfp_lna8toh_vi8_vf
/*
** Convert a vector of 8 bit Cambridge Uni. lna8 value to floats.
** Version optimized for machines without floating point.
*/
void nfp_lna8toh_vi8_vf(size_t len, const fltv_int8_t* bits, float* res);
#endif

#ifdef DEFN_nfp_lna8toh_vi8_vf
void
nfp_lna8toh_vi8_vf(size_t len, const fltv_int8_t* bits, float* res)
{
    /* Pointer to the start of the lookup table. */
    static fltv_int32_t* table = NULL;
    fltv_uint8_t* from;
    fltv_int32_t* to;
    size_t i;			/* Local counter. */
    
    /* Point to the table for converting lna8 bytes to floating point. */
    if (table==NULL)
	table = (fltv_int32_t*) lna8table_vf();

    from = (fltv_uint8_t*) bits;
    to = (fltv_int32_t*) res;
    for (i=0; i<len; i++)
    {
        *to++ = table[*from++];
    }
}
#endif


#ifdef DECL_nfp_pre8toh_vi8_vf
/*
** Convert a vector of 8 bit Cambridge Uni. pre8 value to floats.
** Version optimized for machines without floating point.
*/
void nfp_pre8toh_vi8_vf(size_t len, const fltv_int8_t* bits, float* res);
#endif

#ifdef DEFN_nfp_pre8toh_vi8_vf
void
nfp_pre8toh_vi8_vf(size_t len, const fltv_int8_t* bits, float* res)
{
    /* Pointer to the start of the lookup table. */
    static fltv_int32_t* table = NULL;
    fltv_int32_t* to;
    fltv_uint8_t* from;
    size_t i;			/* Local counter. */
    
    /* Point to the table for converting pre8 bytes to floating point. */
    if (table==NULL)
	table = (fltv_int32_t*) pre8table_vf();

    from = (fltv_uint8_t*) bits;
    to = (fltv_int32_t*) res;
				       
    for (i=0; i<len; i++)
    {
        *to++ = table[*from++];
    }
}
#endif
