/*
** $Header: /u/drspeech/src/fltvec/nv/RCS/sigproc.c,v 1.6 1998/06/24 18:59:36 davidj Exp $
**
** Signal processing operations - naive implementations
*/

#include <fltv-config.h>
#include <stddef.h>
#include <math.h>
#include <fltv-defs.h>
#include <fltv-scansrc.h>


/* Convolution */

#ifdef DECL_nv_convol_vfvf_vf
/*
** Convolution core - naive version.
** "h" is reversed by the function.
** Produces a result of length x_len - h_len + 1.
*/
void nv_convol_vfvf_vf(size_t h_len, size_t x_len, const float *h,
		       const float *x, float *y);
#endif

#ifdef DECL_nv_convol_vdvd_vd
/*
** Convolution core - naive version.
** "h" is reversed by the function.
** Produces a result of length x_len - h_len + 1.
*/
void nv_convol_vdvd_vd(size_t h_len, size_t x_len, const double *h,
		       const double *x, double *y);
#endif

#ifdef DEFN_nv_convol_vfvf_vf
#define FP_TYPE float
#define NAME nv_convol_vfvf_vf
#endif


#ifdef DEFN_nv_convol_vdvd_vd
#define FP_TYPE double
#define NAME nv_convol_vdvd_vd
#endif

void
NAME(size_t h_len, size_t x_len, const FP_TYPE *h,
     const FP_TYPE *x, FP_TYPE *y)
{
  size_t i;
  const FP_TYPE *x_startp = x + h_len - 1;
  const FP_TYPE *x_endp = x - 1;

  for( i = 0; i != x_len - h_len + 1; i++ ) {
    FP_TYPE y_acc = 0.0;
    const FP_TYPE *hp = h;
    const FP_TYPE *xp;

    for( xp = x_startp; xp != x_endp; xp-- ) {
      y_acc += *xp * *hp;
      hp++;
    }
    *y++ = y_acc;
    x_startp++;
    x_endp++;
  }
}
