/*
   $Header: /u/spert/src/fltvec/testsuite/RCS/mulacc_fvfvf_mf_test.c,v 1.3 1995/07/05 23:47:03 davidj Exp $

   Test mulacc_fvfvf_mf routine.
*/

#include <stddef.h>
#include <stdlib.h>

#include "fltvec_test.h"


/* Performs one test given input values and expected output. */

void
one_test(size_t rows, size_t cols, float scale,
         const float *in_vec, const float *in_vec_t,
         const float *in_mat, const float *out_mat)

{
    /* Create padded copies of inputs. */
    float* c_in_vec = rtst_padvec_clone_vf(rows, in_vec);
    float* c_in_vec_t = rtst_padvec_clone_vf(cols, in_vec_t);
    const size_t mat_len = rows * cols;
    float* c_res_mat = rtst_padvec_clone_vf(mat_len, in_mat);

    /* Run routine. */
    mulacc_fvfvf_mf(rows, cols, scale, c_in_vec, c_in_vec_t, c_res_mat);

    /* Check result. */
    rtst_checkeq_vfvf(mat_len, c_res_mat, out_mat);

    /* Check input not corrupted. */
    rtst_checkeq_vfvf(rows, c_in_vec, in_vec);
    rtst_checkeq_vfvf(cols, c_in_vec_t, in_vec_t);

    /* Free and check padding. */
    rtst_padvec_del_vf(c_in_vec);
    rtst_padvec_del_vf(c_in_vec_t);
    rtst_padvec_del_vf(c_res_mat);
}

void
test_1x1(void)
{
    static const float in_vec[]   = {   5.0 };
    static const float in_vec_t[] = {  60.0 };
    static const float in_mat[]   = {  12.0 };
    static const float out_mat[]  = { 162.0 };

    rtst_log("Running test_1x1().\n");
    one_test(1, 1, 0.5, in_vec, in_vec_t, in_mat, out_mat);
    rtst_log("Passed test_1x1().\n");
}

void
test_3x2(void)
{
    static const float in_vec[] = { -8.0, -59.0, 36.0 };
    static const float in_vec_t[] = {  4.0, 9.0 };

    static const float in_mat[] = {
        1.0,  3.0,
        -2.0, 9.0,
        3.0, -3.0
    };

    static const float out_mat[] = {
        -63.0,  -141.0,
        -474.0, -1053.0,
        291.0,   645.0   
    };

    rtst_log("Running test_3x2().\n");
    one_test(1, 1, 2.0, in_vec, in_vec_t, in_mat, out_mat);
    rtst_log("Passed test_3x2().\n");
}

void
run_directed_tests(void)
{
    test_1x1();
    test_3x2();
}

void
run_random_tests(void)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;
    float *in_vec = malloc_vf(maxlen);
    float *in_vec_t = malloc_vf(maxlen);
    float *in_mat = malloc_vf(maxlen*maxlen);
    float *out_mat = malloc_vf(maxlen*maxlen);

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t rows = (size_t) rtst_urand_i32i32_i32(1, maxlen);
        size_t cols = (size_t) rtst_urand_i32i32_i32(1, maxlen);

        const float scale = (float) rtst_urand_i32i32_i32(-7, +7);
        rtst_urandi_i32i32_vf(rows, -7, +7, in_vec);
        rtst_urandi_i32i32_vf(cols, -7, +7, in_vec_t);
        rtst_urandi_i32i32_vf(rows*cols, -7, +7, in_mat);
        copy_vf_vf(rows*cols, in_mat, out_mat);

        nv_mulacc_fvfvf_mf(rows, cols, scale, in_vec, in_vec_t, out_mat);

        rtst_log("rows=%d cols=%d\n", rows, cols);

        one_test(rows, cols, scale, in_vec, in_vec_t, in_mat, out_mat);
    }

    free_vf(in_vec);
    free_vf(in_vec_t);
    free_vf(in_mat);
    free_vf(out_mat);
}

int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    rtst_start("mulacc_fvfvf_mf");
    run_directed_tests();
    run_random_tests();
    rtst_passed();

    rtst_exit();
    return 0;
}
