//
// Est.C
//
// Definitions of estimator objects
// Get fed a series of values and estimate their mean and sd
// or lowpass smooth them 
// or something.
//
// dpwe@media.mit.edu 1996feb14
//

#include "Est.H"
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


Estimator::Estimator(void)
{
    Reset();
}

Estimator::~Estimator(void)
{
}

void Estimator::Reset(double _mean /* = 0 */, double _sd /* = 0 */, 
		      int _n /* = 0 */)
{   /* Return to default startup condition */
    mean = _mean;
    meansq = mean*mean + _sd*_sd;
    n = _n;
}    

double Estimator::NewVal(double x)
{   /* Add a new value; return the smoothed vsn */
    ++n;
    mean = ((n-1)*mean + x)/n;
    meansq = ((n-1)*meansq + x*x)/n;
    return mean;
}

double Estimator::Mean(void) const
{
    return mean;
}

double Estimator::SD(void) const
{
    double sd = sqrt(meansq - mean*mean);
    return sd;
}

void Estimator::Print(ostream& os /* = stdout */, char *tag /* =0 */) const
{
    os << "Estimator" << (void*)this;
    if(tag && strlen(tag)) os << " '" << tag <<"'";
    os << ":" << endl;
    os << "  n=" << n << " mean=" << mean << " meansq=" << meansq << " mu=" 
	<< Mean() << " sd=" << SD() << " N=" << N() << endl;
}

#ifdef MAIN

#define ___quote(ARG)      #ARG
#define ___QUOTE(ARG)	___quote(ARG)
#define TEST(num,clause,elseclause)					\
          if(!(clause))	{ cerr << "FAIL Test " << num << ": "	        \
			       << ___QUOTE(clause) << endl;  elseclause ; }

int fEq(float f1, float f2)
{   // Approximately equal for the purposes of the test
    if(fabs(f1-f2) < 0.0001)
	return 1;
    else
	return 0;
}

void testEstimator(void)
{
    Estimator est;

    TEST(1.0, est.Mean()==0 && est.SD()==0 && est.N()==0, cout << est);
    est.NewVal(1);
    est.NewVal(2);
    est.NewVal(3);
    est.NewVal(4);
    TEST(1.1, est.N()==4 && fEq(est.Mean(),2.5) && fEq(est.SD(),sqrt(1.25)), 
	 cout << est);

    RecursiveEst re(4);
    re.NewVal(1);
    re.NewVal(1);
    re.NewVal(1);
    re.NewVal(1);
    TEST(1.2, re.N()==4 && fEq(re.Mean(),1.0-pow(.75,4)), cout << re);

    BecomesRecEst bre(4);
    bre.NewVal(1);
    bre.NewVal(0);
    bre.NewVal(0);
    bre.NewVal(0);
    TEST(1.3, bre.N()==4 && fEq(bre.Mean(),0.25), cout << bre);
    bre.NewVal(1);
    bre.NewVal(1);
    TEST(1.4, bre.N()==6 && fEq(bre.Mean(),1.0-pow(0.75,3)), cout << bre << " mean " << bre.Mean() << " ?= " << 1.0-pow(0.75,3) << endl);

    RapidRecEst rre(40);
    rre.NewVal(1);    rre.NewVal(1);    rre.NewVal(1);    rre.NewVal(1);
    double v = exp(-4./80)*(1-pow(.75,4))+(1-exp(-4./80))*(1-pow(1-1./40,4));
    TEST(1.5, rre.N()==4 && fEq(rre.Mean(), v), cout << rre << rre.fastRE << "v=" << v << endl);
    int i;
    for(i=0; i<100; ++i)	rre.NewVal(1);
    v = exp(-104./80)*(1-pow(.75,104))+(1-exp(-104./80))*(1-pow(1-1./40,104));
    TEST(1.6, rre.N()==104 && fEq(rre.Mean(), v), cout << rre << rre.fastRE << "v=" << v << endl);

}

int frEq(floatRef& fr1, floatRef& fr2)
{
    int rc = 0;
    int i;
    if(fr1.Size()==fr2.Size()) {
	for(i = 0; i<fr1.Size(); ++i) {
	    if(!fEq(fr1[i], fr2[i]))
		return 0;
	}
	rc = 1;
    }
    return rc;
}

#include "FloatVecOps.H"	// for op==

void testFvEstimator(void)
{
    fvEstimator est(2);
    floatVec fv1(2);
    fv1[0] = 1;
    fv1[1] = 0;
    floatVec un(2);
    un[0] = 1;
    un[1] = 1;

    if(frEq(fv1,un)) { cout << "hay" << endl; }

    TEST(2.0, frEq(est.Mean(),0*un) && frEq(est.SD(),0*un) && est.N()==0, cout << est);
    est.NewVal(1*fv1);
    est.NewVal(2*fv1);
    est.NewVal(3*fv1);
    est.NewVal(4*fv1);
    TEST(2.1, est.N()==4 && frEq(est.Mean(),2.5*fv1) && frEq(est.SD(),fv1*sqrt(1.25)), cout << est);

    fvRecursiveEst re(2,4);
    re.NewVal(1*fv1);
    re.NewVal(1*fv1);
    re.NewVal(1*fv1);
    re.NewVal(1*fv1);
    TEST(2.2, re.N()==4 && frEq(re.Mean(),(1.0-pow(.75,4))*fv1), cout << re);

    fvBecomesRecEst bre;
    bre.Resize(2);
    bre.tc = 4;
    bre.NewVal(1*fv1);
    bre.NewVal(0*fv1);
    bre.NewVal(0*fv1);
    bre.NewVal(0*fv1);
    TEST(2.3, bre.N()==4 && frEq(bre.Mean(),0.25*fv1), cout << bre);
    bre.NewVal(1*fv1);
    bre.NewVal(1*fv1);
    TEST(2.4, bre.N()==6 && frEq(bre.Mean(),(1.0-pow(0.75,3))*fv1), cout << bre << " mean " << bre.Mean() << " ?= " << 1.0-pow(0.75,3) << endl);

    fvRapidRecEst rre(2,40);
    rre.NewVal(fv1);    rre.NewVal(fv1);    rre.NewVal(fv1);    rre.NewVal(fv1);
    double v = exp(-4./80)*(1-pow(.75,4))+(1-exp(-4./80))*(1-pow(1-1./40,4));
    TEST(2.5, rre.N()==4 && frEq(rre.Mean(), v*fv1), cout << rre << rre.fastRE << "v=" << v << endl);
    int i;
    for(i=0; i<100; ++i)	rre.NewVal(fv1);
    v = exp(-104./80)*(1-pow(.75,104))+(1-exp(-104./80))*(1-pow(1-1./40,104));
    TEST(2.6, rre.N()==104 && frEq(rre.Mean(), v*fv1), cout << rre << rre.fastRE << "v=" << v << endl);

}

main(void)
{
    testEstimator();
    cout << "testEstimator done" << endl;
    testFvEstimator();
    cout << "testFvEstimator done" << endl;

    return 0;
}

#endif /* MAIN */

