//
// Patch.C
//
// Definition of Patch object - the ubiquitous piece-of-energy-surface
// for surfex system (rewrite)
// 
// dpwe@media.mit.edu 1995nov12
//

#include <assert.h>
#include <iostream.h>
#include "Patch.H"

int Patch::abortOnRange = 0;	// should we abort if accessed out of range?

probRef Patch::ColRef(int pc) const 
{   // Translate ColRef request relative to tBase
    //assert(pc>=timeBase && pc < (timeBase+cols));
    probRef _pr;
    if(pc>=timeBase && pc < (timeBase+cols)) {
	_pr.CloneRef(probArray::ColRef(pc - timeBase));
    } else { 
	cerr << "empty col " << pc << " when cols=" << timeBase << ".." 
	     << timeBase+cols << endl;
	_pr.CloneRef(emptyCol);
	if(abortOnRange)
	    abort();
    } 
    return _pr; 
}

Patch& Patch::operator += (const Patch& p)	// add cols of time supt overlap only
{ 
    int cmin = max(timeBase, p.timeBase);
    int cmax = min(timeBase+cols, p.timeBase+p.cols);
    for(int c = cmin; c<cmax; ++c)  {
	this->ColRef(c) += p.ColRef(c);
    } 
//    double oldbottom = (this->ColRef(cmin))[0];
//    cout << "Pch::Op+=:cols="<<cmin<<".."<<cmax<<endl;
//    cout << "i.e. " << (this->ColRef(cmin))[0] << "=" << oldbottom << "+" << (p.ColRef(cmin))[0] << endl;
    // If we didn't take all the cols of what we were passed, maybe a bug
    if(0 && (cmin > p.timeBase || cmax < p.timeBase+p.cols)) {
	cerr << "Patch::+=: time=" << timeBase << " only took cols " 
	    << cmin << " to " << cmax-1 << " of " << p.timeBase 
	    << " to " << p.timeBase+p.cols-1 << endl;
    }

    return *this;
}

void Patch::SquaredAccum(const Patch& p)
{   /* Add the data in the specified patch, but only in their overlap. 
       Where they do overlap, data is combined in the *squared* domain, 
       i.e. output = sqrt(orig*orig + new*new).  */
    int cmin = max(timeBase, p.timeBase);
    int cmax = min(timeBase+cols, p.timeBase+p.cols);
    for(int c = cmin; c<cmax; ++c)  {
	(this->ColRef(c)).Hypot(p.ColRef(c));
    }
    // If we didn't take all the cols of what we were passed, maybe a bug
    if(0 && (cmin > p.timeBase || cmax < p.timeBase+p.cols)) {
	cerr << "Patch::SqAc: time=" << timeBase << " only took cols " 
	    << cmin << " to " << cmax-1 << " of " << p.timeBase 
	    << " to " << p.timeBase+p.cols-1 << endl;
    }
}

int Patch::SaveExp(char *filename)
{   /* Write out the patch's exp to a DATfile, properly padded for timeBase */
    // Build the array
    floatArray outputArray(rows, maxColExtent);
    outputArray.arrayRef::CopyAt(giveExp(), 0, timeBase);

    int rc = 0;
    rc = (outputArray.Save(filename)>0);

    return rc;
}

// fPatch - a patch of a floatArray (not 3fold probArray)

int fPatch::abortOnRange = 0;	// should we abort if accessed out of range?

floatRef fPatch::ColRef(int pc) const 
{   // Translate ColRef request relative to tBase
    //assert(pc>=timeBase && pc < (timeBase+cols));
    floatRef _fr;
    if(pc>=timeBase && pc < (timeBase+cols)) {
	_fr.CloneRef(floatArray::ColRef(pc - timeBase));
    } else { 
	cerr << "fP empty col " << pc << " when cols=" << timeBase << ".." 
	     << timeBase+cols << endl;
	_fr.CloneRef(emptyCol);
	if(abortOnRange)
	    abort();
    } 
    return _fr; 
}
