//
// Patch.H
//
// Definition of Patch object - the ubiquitous piece-of-energy-surface
// for surfex system (rewrite)
// 
// dpwe@media.mit.edu 1995nov12
//

#ifndef _PATCH_H
#define _PATCH_H

#include <assert.h>

#include "ProbVec2.H"

//  class pSurface : public probArray {
//  protected:
//      int       base;
//  public:
//      pSurface(int st = NUM_TIME_STEPS, int chs = NUM_FRQ_CHANS, int bs = 0) 
//  	: base(bs), probArray(chs, st)
//  	        { }
//      pSurface(char *filename, int dataset=0)	// read from a DAT file
//  	: base(0), probArray(0, 0)
//                  { Load(filename, dataset); }
//      ~pSurface(void)
//                  { }
//  
//      int&      TimeBase(void) 	{ return base; }
//      int       TimeSteps(void)	{ return cols; }
//      int       FrqChans(void) 	{ return rows; }
//  };

class Patch : public probArray {
protected:
    int timeBase;
public:
    Patch(int st = 0, int chs = 0, int bs = 0) 
	: timeBase(bs), probArray(chs, st)
	{ emptyCol.Resize(rows); emptyCol.Clear(); 
	  maxColExtent = timeBase+cols; }
    Patch(char *filename, int dataset=0)	// read from a DAT file
 	: timeBase(0), probArray(0, 0)
	{ Load(filename, dataset); 
	  emptyCol.Resize(rows); emptyCol.Clear(); 
	  maxColExtent = timeBase+cols; }
    Patch(const Patch& p) : probArray(p)
        { timeBase = p.timeBase; 
	  emptyCol.Resize(rows); emptyCol.Clear(); 
	  maxColExtent = timeBase+cols; }
    ~Patch(void)
        { }
    Patch& operator = (const Patch& p)
        { probArray::operator = (p); timeBase = p.timeBase; 
	  emptyCol.Resize(rows); emptyCol.Clear(); 
	  maxColExtent = timeBase+cols;
          return *this; }
    Patch& operator = (const probArray& p)
        { probArray::operator = (p); 	
	  emptyCol.Resize(rows); emptyCol.Clear();
	  maxColExtent = timeBase+cols;
          return *this; }

    // Need to trap Resize to adjust emptyCol
    void Resize(int nr, int nc) { emptyCol.Resize(nr); emptyCol.Clear();
                                  probArray::Resize(nr, nc); 
                                  maxColExtent = max(maxColExtent, 
						     timeBase+cols);}

    int       TimeBase(void) 	const  	{ return timeBase; }
    int       TimeSteps(void)	const  	{ return cols; }
    int	      TimeMax(void)	const  	{ return maxColExtent; }
    int       FrqChans(void) 	const  	{ return rows; }

    void      SetTimeBase(int t)	{ timeBase = t; 
                                          maxColExtent = max(maxColExtent, 
							     timeBase+cols); }

    probRef ColRef(int pc) const; // Translate ColRef request relative to tBase

    Patch& operator += (const Patch& p);	// add cols of time supt overlap only
    void SquaredAccum(const Patch& p);	// like += but in squared domain

    int SaveExp(char *filename);	// Write the exp part of the patch 
                                        // to a dat file with proper padding

    int maxColExtent;		// record the largest we ever were

    static int abortOnRange;	// should we abort if accessed out of range?

 private:
    probVec emptyCol;
};

class fPatch : public floatArray {
protected:
    int timeBase;
public:
    fPatch(int st = 0, int chs = 0, int bs = 0) 
	: timeBase(bs), floatArray(chs, st)
	{ emptyCol.Resize(rows); emptyCol.Clear(); 
	  maxColExtent = timeBase+cols; }
    fPatch(const fPatch& p) : floatArray(p)
        { timeBase = p.timeBase; 
	  emptyCol.Resize(rows); emptyCol.Clear(); 
	  maxColExtent = timeBase+cols; }
    ~fPatch(void)
        { }
    fPatch& operator = (const fPatch& p)
        { floatArray::operator = (p); timeBase = p.timeBase; 
	  emptyCol.Resize(rows); emptyCol.Clear(); 
	  maxColExtent = timeBase+cols;
          return *this; }
    fPatch& operator = (const floatArray& p)
        { floatArray::operator = (p); 	
	  emptyCol.Resize(rows); emptyCol.Clear();
	  maxColExtent = timeBase+cols;
          return *this; }

    // Need to trap Resize to adjust emptyCol
    void Resize(int nr, int nc) { emptyCol.Resize(nr); emptyCol.Clear();
                                  floatArray::Resize(nr, nc); 
                                  maxColExtent = max(maxColExtent, 
						     timeBase+cols);}

    int       TimeBase(void) 	const  	{ return timeBase; }
    int       TimeSteps(void)	const  	{ return cols; }
    int	      TimeMax(void)	const  	{ return maxColExtent; }
    int       FrqChans(void) 	const  	{ return rows; }

    void      SetTimeBase(int t)	{ timeBase = t; 
                                          maxColExtent = max(maxColExtent, 
							     timeBase+cols); }

    floatRef ColRef(int pc) const; // Translate ColRef request rel to tBase

    int maxColExtent;		// record the largest we ever were

    static int abortOnRange;	// should we abort if accessed out of range?

 private:
    floatVec emptyCol;
};

#endif /* _PATCH_H */




