//
// nSurface.H
//
// minimal non-template redefinition of floatArrays &c
// to replace Ref.H Vec.H floatRef.H Array.H floatArray.h which crash DCC
//
// dpwe@media.mit.edu 1995aug13
// $Header: /u/drspeech/src/guitools/farray_otcl/RCS/nSurface.C,v 1.5 1998/08/20 01:52:54 dpwe Exp $


#include "nSurface.H"

#ifdef HAVE_LIBDAT
#include "myDAT.h"
#endif /* HAVE_LIBDAT */

#include <assert.h>

// Special returnable value for out-of-range accesses
float floatRef::dummy = 0.0;
float arrayRef::dummy = 0.0;

void arrayRef::Copy(const arrayRef& fa)
{   /* Copy the overlap of this and passed array (don't resize) */
    int i,j; 
    for(i=0; i<min(rows, fa.rows); ++i) { 
	for(j=0; j<min(cols, fa.cols); ++j) { 
	    El(i,j) = fa.Val(i,j); 
	} 
    } 
}

void arrayRef::CopyAt(const arrayRef& fa, int tr, int lc)
{   // Copy an array into this one, but apply a particular offset
    // so that this[tr+i][lc+j] = fa[i][j] for all reasonabe ixs
    // tr and lc can be -ve to discard top and left of fa
    int rn = max(0, -tr); 
    int rx = min(rows-tr, fa.rows);
    int cn = max(0, -lc); 
    int cx = min(cols-lc, fa.cols);
//    cout << "CopyAt("<<tr<<","<<lc<<"): "<<fa.rows<<"x"<<fa.cols<<" into "<<rows<<"x"<<cols<<endl;
    for(int i=rn; i<rx; ++i) { 
	for(int j=cn; j<cx; ++j) { 
	    El(tr+i,lc+j) = fa.Val(i,j); 
	}
    }
}

void arrayRef::AddAt(const arrayRef& fa, int tr, int lc)
{   // Add a floatArray to this one, offset according to tr and lc
    // tr and lc can be -ve to discard top and left of fa
    int rn = max(0, -tr); 
    int rx = min(rows-tr, fa.rows);
    int cn = max(0, -lc); 
    int cx = min(cols-lc, fa.cols);
    for(int i=rn; i<rx; ++i) { 
	for(int j=cn; j<cx; ++j) { 
	    El(tr+i,lc+j) += fa.Val(i,j); 
	}
    }
}

floatArray arrayRef::Mult(const arrayRef& a) const
{   /* Do matrix multiplication, this x a, return a new floatArray */
    if(this->cols != a.rows) {
	cerr << "arrayRef::Mult: cannot do " << this->rows << "x" << this->cols << " * " << a.rows << "x" << a.cols << endl;
	floatArray rslt(0,0);
	return rslt;
    }
    floatArray rslt(this->rows, a.cols);
    int r,c;
    for(r = 0; r<rslt.Rows(); ++r) {
	for(c = 0; c<rslt.Cols(); ++c) {
	    rslt.El(r,c) = dot(this->RowRef(r), a.ColRef(c));
	}
    }
    return rslt;
}

int  floatArray::Load(const char *filename, int dataset/*=0*/)
{   /* Load the array from a given dataset of a DAT file.
       Return the number of elements loaded, or -1 if bad read. */
    int ds, ch, rw, cl, rc = -1;
    double sr; 
    char fm, c = 'f';

#ifdef HAVE_LIBDAT    
    if(DATInfo((char *)filename, &ds, &ch, &rw, &cl, &sr, &fm)==0)  {
	return -1;  }
    if(dataset >= ds)  {  return -1; }
    /* cols are the fastest rotating dimension in a DAT file, 
       so make an implicit transposition on reading into the ucharArray */
    Resize(rw, cl);
    rc = DATRead((char *)filename, dataset, 0, -1, -1, 
		 (void **)&data, c, &rw, &cl, &sr); 
//cerr << "fa::Load: DATR returned " << rc << endl;
    Transpose();
    adata = data;
#endif /* HAVE_LIBDAT */
    return rc;
}

int  floatArray::Save(const char *filename)
{   /* Save the array to a DAT file.
       Return the number of elements saved, or -1 if an error. */
    int dataset = 0;	// always write just one dataset for now
    int ds, ch, rw, cl; double sr; char fm; char c = 'f', rc = -1;
    int srows, scols;
    int overwrite = 1;
    char srcfmt = 'f';
    char outfmt = 'f';
    double sampRate = /* guess */ 22050.0/100.0;

#ifdef HAVE_LIBDAT
//    if(DATInfo((char *)filename, &ds, &ch, &rw, &cl, &sr, &fm)==0)  {
//	return -1;  }
//    if(dataset >= ds)  {  return -1; }
    /* cols are the fastest rotating dimension in a DAT file, 
       so make an implicit transposition on reading into the ucharArray */
    Transpose();
    srows = Rows();	scols = Cols();	
    rc = DATWrite((char *)filename, dataset, 0, -1, -1, 
	     (void *)data, srcfmt, srows, scols, sampRate, outfmt, overwrite);
    Transpose();	// get it back to normal
#endif /* HAVE_LIBDAT */
    return rc;
}

void floatArray::Transpose(void)
{ 
    int nr = cols; int nc = rows;
    float *ndata = new float[nr*nc];
    int i, j;
    for(i=0;i<nr;++i)  {  
	for(j=0;j<nc;++j)  {
	    *(ndata+i*nc+j) = *(data+j*cols+i); 
	} 
    }
    delete [] data; 
    adata = data = ndata; 
    rows = nr; 
    cols = nc; 
    len = nr*nc; 
}

void floatArray::OuterProd(const floatRef& rowMarg, const floatRef& colMarg)
{   /* Recreate this array to be the outer product of rowMarg and colMarg.
       Created array has rowMarg.Size() rows and colMarg.Size() cols, 
       and El(r, c) = rowMarg.El(r)*colMarg.El(c) .  */
    rows = rowMarg.Size();
    cols = colMarg.Size();
    delete [] data; 
    data = new float[rows*cols];
    adata = data;
    len = rows*cols;
    int i, j;
    for(i=0;i<rows;++i)  {  
	for(j=0;j<cols;++j)  {
	    El(i,j) = rowMarg.Val(i)*colMarg.Val(j);
	} 
    }
}

void floatArray::FlipUD(void)
{ 
    int i, j;
    double d;
    for(i=0;i<rows/2;++i)  {  
	for(j=0;j<cols;++j)  {
	    d = *(data+(rows-1-i)*cols+j); 
	    *(data+(rows-1-i)*cols+j) = *(data+i*cols+j);
 	    *(data+i*cols+j) = d;
	} 
    }
}

void floatArray::FlipLR(void)
{ 
    int i, j;
    double d;
    for(j=0;j<cols/2;++j)  {
	for(i=0;i<rows;++i)  {  
	    d = *(data+i*cols+(cols-1-j)); 
	    *(data+i*cols+(cols-1-j)) = *(data+i*cols+j); 
	    *(data+i*cols+j) = d;
	} 
    }
}

void floatArray::Resample(int nr, int nc)
{ 
    float *ndata = new float[nr*nc];
    int i, j;
    for(i=0;i<nr;++i)  {  
	for(j=0;j<nc;++j)  {
//	    *(ndata+i*nc+j) = *(data
//				+((int)(i*(rows-1)/(nr-1)))*cols
//				+(j*(cols-1)/(nc-1))); 
	    *(ndata+i*nc+j) = *(data
				+((int)((i*rows)/nr))*cols
				+((j*cols)/nc));
	} 
    }
    delete [] data; 
    adata = data = ndata; 
    rows = nr; 
    cols = nc; 
    len = nr*nc; 
}

void floatArray::Resize(int nr, int nc)
{ 
    float *ndata = new float[nr*nc];
    float *a, *b;
    int i, j, dr = min(nr, rows), dc = min(nc, cols);
    for(i=0;i<dr;++i)  {  
	for(j=0;j<dc;++j)  {
	    a = ndata+i*nc+j;
	    b = data+i*cols+j;
	    // *(ndata+i*nc+j) = *(data+i*cols+j);
	    *a = *b;
	}
	for(j=dc; j<nc; ++j) {
	    *(ndata+i*nc+j) = 0;
	}
    }
    for(i=dr; i<nr; ++i)  {
	for(j=0; j<nc; ++j) {
	    *(ndata+i*nc+j) = 0;
	}
    }
    delete [] data;
    adata = data = ndata;
    rows = nr;
    cols = nc;
    len = nr*nc;
}

#ifdef FAMAIN
#define ___quote(ARG)      #ARG
#define ___QUOTE(ARG)	___quote(ARG)
#define TEST(num,clause)					\
        if(!(clause))	{ cerr << "FAIL Test " << num << ": "	\
			      << ___QUOTE(clause) << endl;  }

#include <stdio.h>

int AEq(float a, float b)
{   // loose version of operator==(float,float) - allow up to 0.001 diff
    if(fabs(a-b)<0.001)	return 1;
    else		return 0;
}

void testFloatVec(void)
{   //  test the floatVec new functions
    // Default constructor
    floatVec fv(3);
    int i,j;

    for(i = 0; i<3; ++i)  {
	fv.El(i) = 10+100*i;
    }
    // Min
    TEST("1.1", fv.Min()==10.0);	
    // Max
    TEST("1.2", fv.Max()==210.0);
    // Scale
    fv.Scale(0.1);
    TEST("1.3", fv.El(0)==1.0);
    TEST("1.4", fv.Max()==21.0);
    // Offset
    fv.Offset(-5.0);
    TEST("1.5", fv.El(0)==-4.0);
    TEST("1.6", fv.Max()==16.0);
    // Log
    fv.Offset(5.0);
    fv.El(0) = 1.0;
    fv.El(1) = 2.7182818284590452354;
    fv.Log();
    TEST("1.7", fv.El(0)==0 && AEq(fv.El(1),1.0));
    // Exp
    fv.Exp();
    TEST("1.7b", AEq(fv.El(0),1.0) && AEq(fv.El(2),21.0));

    // Resample
    fv.Resize(0);
    fv.Resize(3);
    fv.El(1)=1;
    fv.Resample(2);	// 2 is just edges, so all zero
    TEST("1.8", fv.Max()==0);
    fv.Resize(3);
    fv.El(1)=1;
    fv.Resample(4);
    TEST("1.9", AEq(fv.El(1),fv.El(2)));
    // that val should be 2/3 between 0 and 1
    // i.e. 0.666667
    TEST("1.10", AEq(fv.El(1), 0.6666667));
    // round it out
    fv.Resample(3);
    fv.Scale(3.0);
    fv.Resample(5);
    TEST("1.11", AEq(fv.El(1), 1.0));

    // Test the new ref operator
    fv.Resize(10);
    floatVec fv2(4);
    fv.Enumerate(1);
    fv2.Enumerate(101);

    fv.Ref(4) = fv2;
    TEST("1.12", fv.Len() == 10 && fv.El(3)==4 && fv.El(4)==101 && fv.El(7)==104 && fv.El(8)==9);

    // can't do fv=fv2.Ref(0,1) here because assignment to float*Vec* 
    // clones, whereas assignment to float*Ref* copies valid overlap.
    fv.Copy(fv2.Ref(0,1));
    TEST("1.13", fv.Len() == 10 && fv.El(0)==101 &fv.El(1)==2);
    fv.Print();

    fv.Enumerate();
    fv.Ref(2,2) = fv2.Ref(2,2);
    TEST("1.14", fv.Len() == 10 && fv.El(1)==1 && fv.El(2)==103 && fv.El(3)==104 && fv.El(4)==4);

    // Check that Ref works even when step is not 1
    floatArray fa(3,3);
    (fa.ColRef(1)).Ref(1) = fv2;
    TEST("1.15", fa.El(1,0)==0 && fa.El(0,1) == 0 && fa.El(1,2) == 0 && fa.El(1,1) == 101 && fa.El(2,1) == 102);

    // Print()
    fv.Print(cerr, "Test 1 done.");
}

void testArray(void)
{   //  test the floatArray class
    // Default constructor
    floatArray a1;
    TEST("2.1", a1.Rows()==0 && a1.Cols()==0);
    // Regular constructor
    floatArray a2(2,3);
    TEST("2.2", a2.Rows()==2 && a2.Cols()==3);
    // El() and Val()
    a2.El(0,0) = 0; a2.El(1,0)=1; a2.El(0,1)=10; a2.El(1,1)=11;
    a2.El(0,2) = 20; a2.El(1,2)=21;
    TEST("2.3", a2.Val(0,0)==0 && a2.Val(1,0)==1 && a2.Val(0,1)==10 \
	 && a2.Val(1,1)==11 && a2.Val(0,2)==20 && a2.Val(1,2)==21);
    // El for access too
    TEST("2.4", a2.El(0,0)==0 && a2.El(1,0)==1 && a2.El(0,1)==10 \
	 && a2.El(1,1)==11 && a2.El(0,2)==20 && a2.El(1,2)==21);
    // Index accessing - indices same order - [row][col]
    TEST("2.5", a2[0][0]==0 && a2[1][0]==1 && a2[0][1]==10 \
	 && a2[1][1]==11 && a2[0][2]==20 && a2[1][2]==21);
    // Copy constructor
    floatArray a3(a2);
    TEST("2.6", a3.Rows()==a2.Rows() && a3.Cols()==a2.Cols() \
	 && a3[0][0]==0 && a3[1][0]==1 && a3[0][1]==10 \
	 && a3[1][1]==11 && a3[0][2]==20 && a3[1][2]==21);
    // Comparison - same
    TEST("2.7", a3==a2);
    // different
    a3[0][2]=1;
    TEST("2.8", !(a3==a2));
    a3[0][2]=20;
    // different sizes
    TEST("2.9", !(a3==a1));
    // Copy assignment
    a1.Resize(a2.Rows(), a2.Cols());
//    a1 = a2; 
    a1.Copy(a2); 
    TEST("2.10", (a2==a1));
    a1[0][2]=200;
    TEST("2.11", !(a2==a1));
    // Assign() alias for copy assignment
    a1.Resize(0,0);
    TEST("2.11b", !(a1==a2));
    a1.CloneArray(a2); 
    TEST("2.11c", (a2==a1));
    a1[0][2]=200;
    // Get rows
    floatRef r1(5, new float[5]);
    r1 = a1.RowRef(1);
    // doesn't modify the length of the dataref
    TEST("2.13", r1.Len() == 5);
    TEST("2.14", r1[0]==1 && r1[1]==11 & r1[2]==21);
    // undersize Ref to GetRow
    r1[2]=255; 
    r1.SetLen(2);
    r1 = a1.RowRef(1);
    TEST("2.16", r1[0]==1 && r1._El(2)==255);
    // Ref._El does no bounds checking
    // undersize Ref to SetRow
    a1.RowRef(0) = r1;
    TEST("2.17", a1[0][0]==1 && a1[0][1]==11 
	 && a1[0][2]==200);
    // oversize ref to SetRow
    r1.SetLen(5);
    a1.RowRef(0) = r1;
    TEST("2.18", a1[0][2]==255);
    // Transpose
    a1.CloneArray(a2);
    a1.Transpose();
    TEST("2.20", a1.Rows()==3 && a1.Cols()==2 && a1[0][0]==0 && a1[1][0]==10 
	 && a1[2][0]==20 && a1[0][1]==1 && a1[1][1]==11 && a1[2][1]==21);
    // Get cols
    r1 = a1.ColRef(1);
    // doesn't modify the length of the dataref
    TEST("2.22", r1.Len() == 5);
    TEST("2.23", r1[0]==1 && r1[1]==11 & r1[2]==21);
    // undersize Ref to GetCol
    r1[2]=255;
    r1.SetLen(2);
    r1 = a1.ColRef(1);
    TEST("2.25", r1[0]==1 && r1._El(2)==255);
    // undersize Ref to SetCol
    a1.ColRef(0) = r1;
    TEST("2.26", a1[0][0]==1 && a1[1][0]==11 
	 && a1[2][0]==20);
    // oversize ref to SetCol
    r1.SetLen(5);
    a1.ColRef(0) = r1;
    TEST("2.27", a1[2][0]==255);
    a1[2][0]=20;
    a1.Transpose();	// back to 3 cols, 2 rows
    a1[0][0]=0;
    a1[0][1]=10;
    a1[0][2]=20;
    // Sum
    a1.Resize(3,2);
    a1[0][0] = 48;	a1[0][1] = 51;
    a1[1][0] = 49;	a1[1][1] = 52;
    a1[2][0] = 50;	a1[2][1] = 53;    
    TEST("2.31a",  (a1.ColRef(0)).Sum()==147);
    TEST("2.31a",  (a1.ColRef(1)).Sum()==156);
    // Clear
    a1.Clear();
    TEST("2.32", a1.Sum()==0);
    // Resize
    a1.CloneArray(a3);
    a1.Resize(3,2);
    TEST("2.33", a1.Rows()==3 && a1.Cols()==2 && a1[0][1]==10 && a1[1][1]==11
	 && a1[2][1]==0);
    a1.CloneArray(a3);
    a1.Resize(1,4);
    TEST("2.34", a1.Rows()==1 && a1.Cols()==4 && a1[0][1]==10 && a1[0][2]==20
	 && a1[0][3]==0);
    // DAT file access
    char tmpnam[] = "/usr/tmp/tmp.dat";
//    a3.saveDAT(tmpnam);
//    a1.loadDAT(tmpnam);
//    TEST("2.35", a1==a3);
//    cerr << "DAT tests a bit thin at the moment." << endl;
    // test array set
    a1.Resize(2,4);
    a1.Fill(99);
    TEST("2.36", a1.Rows()==2 && a1.Cols()==4 && a1[0][0]==99 && a1[1][0]==99 && a1[0][1]==99 && a1[1][1]==99);
    // Direct row-to-row copying via refs
    a1[0][0] = 101; a1[0][1] = 102; 
    a1[1][0] = 103; a1[1][1] = 104;
    //    a1[1] = a1[0];
    // You can't do it like that because a1 only has ONE ref, which is re-used.
    floatRef re(a1[0]);
    a1[1] = re;
    TEST("2.37", a1[1][0]==101 && a1[1][1]==102);
    // Basic sanity check on ref : it's a row, so it has 4 columns
    // and the colums are spaced a row's length apart
    //    TEST("2.37b", re.Len()==4 && re.Step()==2);
    // No, columns are fast rotating, so step ==1
    TEST("2.37b", re.Len()==4 && (re.Step()==1 || re.Step()==2));
    if(re.Step()==1)	cout << "Columns are fastest rotating (a(0,0),a(0,1)..)" << endl;
    else		cout << "Rows are fastest rotating (a(0,0),a(1,0)..)" << endl;
    // Col-to-col copying via refs
    a2.Resize(4,2);
    a2[0][0] = 101; a2[0][1] = 102; 
    a2[1][0] = 103; a2[1][1] = 104;
    re.CloneRef(a2.ColRef(0));
    // Basic sanity check on ref : it's a col of a 4x2, so it has 4 rows
    // cols have their row elements close-packed
    //    TEST("2.38a", re.Len()==4 && re.Step()==1);
    // nope, row elements are spaced out
    TEST("2.38a", re.Len()==4 && (re.Step()==2 || re.Step()==1));
    if(re.Step()==2)	cout << "Columns are fastest rotating (a(0,0),a(0,1)..)" << endl;
    else		cout << "Rows are fastest rotating (a(0,0),a(1,0)..)" << endl;
    a2.ColRef(1) = re;
    TEST("2.38", a2[0][1]==101 && a2[1][1]==103);
    // Col-to-row copying
    a1.Fill(0);
    a1[1] = a2.ColRef(0);	// Can do this without copying to a Ref
                                // because a1 and a2 are diff arrays, hence
                                // different internal Refs used
    TEST("2.39", a1[1][0]==101 && a1[1][1]==103);
    // Row-to-col copying
    a2.Fill(11);
    a2.ColRef(1) = a1[1];
    TEST("2.40", a2[0][1]==101 && a1[1][1]==103);

    // RowRef too
    re.CloneRef(a2[1]);
    floatRef r2(a2.RowRef(1));
    TEST("2.41", re.Data()==r2.Data() && re.Len()==r2.Len() && re.Step()==r2.Step());

    // RowRef, ColRef out of bounds
    floatRef *rup;
    a2.Resize(2,2);
    // Array now just aborts for reference requests out of bounds - don't test
#ifndef ABORT_OOR
    fprintf(stderr, "(am testing null returns for out-of-range)\n");
    rup = &a2.ColRef(0);
    TEST("2.42b", rup!=0);
    rup = &a2.ColRef(1);
    TEST("2.42c", rup!=0);
    rup = &a2.RowRef(0);
    TEST("2.43b", rup!=0);
    rup = &a2.RowRef(1);
    TEST("2.43c", rup!=0);
    // ColRef and RowRef now return *new* floatRefs by *value* - can't 
    // take their address any more (1996feb01)

    rup = &a2.RowRef(1);
    TEST("2.43c", rup!=0);
    rup = &a2.RowRef(1);
    TEST("2.43c", rup!=0);
#endif /* ABORT_OOR */

    // test the dot function (doesn't belong to this test, but here we are)
    floatVec dotfv(2);
    dotfv[0] = 3; dotfv[1] = 4;
    TEST("2.44", dot(dotfv, dotfv)==5*5);
    TEST("2.45", dotfv.Norm()==5);
    TEST("2.46", Norm(dotfv)==5);
    dotfv.Resize(3);
    dotfv[2] = 5;
    floatVec dfv2(3);
    dfv2.Fill(2);
    TEST("2.47", AEq(dot(dotfv, dfv2), 2*3+2*4+2*5));

    // test floor and ceil, well, gotta be somewhere
    dotfv.Resize(4);
    dotfv[0] = 1; dotfv[1] = 2; dotfv[2] = 3; dotfv[3] = 4;
    dotfv.Floor(2);
    TEST("2.48", dotfv[0]==2 && dotfv[1]==2 && dotfv[2]==3 && dotfv[3]==4);
    dotfv.Ceil(3);
    TEST("2.49", dotfv[0]==2 && dotfv[1]==2 && dotfv[2]==3 && dotfv[3]==3);
    dotfv.Ceil(30);
    TEST("2.50", dotfv[0]==2 && dotfv[1]==2 && dotfv[2]==3 && dotfv[3]==3);
    dotfv.Ceil(-1);
    // (also test operator == (double) while we're here)
    //TEST("2.51", dotfv==-1);  
    // no, if anything, that returns a vector now.

    // Print()
    a1.Print(cerr, "Test 2 done.");
}

void testArrayMult(void)
{   // check out the array multiply routine
    floatArray a1(2, 3);
    floatArray a2(3, 4);
    a1.El(0,0) = 1; a1.El(0,1) = 2; a1.El(0,2) = 3;
    a1.El(1,0) = 4; a1.El(1,1) = 5; a1.El(1,2) = 6;
    a2.El(0,0) = 10; a2.El(0,1) = 11; a2.El(0,2) = 12; a2.El(0,3) = 13;
    a2.El(1,0) = 20; a2.El(1,1) = 21; a2.El(1,2) = 22; a2.El(1,3) = 23;
    a2.El(2,0) = 30; a2.El(2,1) = 31; a2.El(2,2) = 32; a2.El(2,3) = 33;
    floatArray a3 = a1.Mult(a2);
    TEST("3.1", a3.Rows() == 2 && a3.Cols() == 4);
    TEST("3.2", a3.Val(0,0)==140 && a3.Val(0,1)==146 && a3.Val(1,1)==335);
    a3.Print(cerr, "Test 3 done");
}

int main(int argc, char **argv)
{
    cout << "Testing floatVecs.." << endl;
    testFloatVec();
    cout << "Testing floatArrays.." << endl;
    testArray();
    cout << "Testing floatArray Mult.." << endl;
    testArrayMult();

    return 0;
}

#endif /* FAMAIN */
