/*
 *  DATUtils.h - A set of functions to provide encapsulated access to DAT files
 *  after DATmex/ files
 *  dpwe@media.mit.edu 1995may10
 *
 */

int Fmt2ByteCount(char fmt);
    /* return the number of bytes per data point for the format 
       indicated by the char (buUcdDfFzZxX).  Zero if unrecognized */

int DATRead(char *fileName, int dset, int chan, int row, int col, 
	    void **dataPtrPtr, char format, 
	    int *rowsPtr, int *colsPtr, double *srPtr);
    /* Read a 2D matrix of data from the DAT file specified by <fileName>.  
       The first two of the four dimensions (<dset>, <chan>, <row>, <col>)
       that are not (-1) are held constant, and the result scans across the 
       remaining two.  If fewer than two dimensions are not -1, then they 
       are arbitrarily taken to be zero starting with <dset>.
       <format> specifies the type of data to write into <**dataPtrPtr> -- 
       'f' for float, 'F' for doubles, 'c' for signed char, 'b' for 
       unsigned byte etc (these are the libdat format chars).  
       <*srPtr> receives the sample rate of the DAT file if non-null.  
       If <*dataPtrPtr> is NULL, it is allocated.  The return code is the 
       number of *data points* written to <**dataPtrPtr>, or -1 on 
       failure. <*rowsPtr> receives the number of rows read, and 
       <*colsPtr> the number of columns.  Data is stored with columns as 
       the fastest rotating dimension, image-style. */

int DATWrite(char *fileName, int dset, int chan, int row, int col, 
	     void *dataPtr, char format, int nrows, int ncols, 
	     double sampleRate, char outFormat, int trunc);   
    /* Write a 2D matrix of data out to a DAT file.  The data is read from 
       <*dataPtr>, which is a pointer to <nrows>x<ncols> of the data point 
       indicated by the format character <format>.  (columns are the 
       fastest rotating dimension, image-style).  The DAT file is given 
       a samplingRate tag if <sr> is greater than zero.  If the DAT file 
       exists, attempt to replace or append the slice indicated by 
       <dset>, <chan>, <row>, <col> where a value of (-1) indicates that 
       the dimension is to be written over, not held constant.  
       <outFormat> is the output format for the file, or ' ' to copy an 
       existing file's format.  Return samples written, -1 on error. 
       If <trunc> is nonzero, any existing file is destroyed (truncated) 
       first. */

int DATInfo(char *fileName, int *dsetsPtr, int *chansPtr, int *rowsPtr, 
	    int *colsPtr, double *srPtr, char *fmtPtr);
   /* Obtain information for the DATfile specified by <filename>.
       Return the number of dataset, channels, rows and columns 
       in the appropriate pointers.  Return the sampling rate (if any) 
       and format character too.  Returns 0 if file could not be opened, 
       else 1. */

int DATTagRead(char *fileName, char *tagname, char *tagval, int tagvallen);
    /* Read a descriptor field for the DATfile specified by <filename>.
       Opens the named DAT file, attempts to retrieve a field specified 
       by *tagname, and returns as much of the associated data in 
       tagval, which is assumed preallocated to tagvallen.  *tagval 
       is null-terminated. 
       Return 1 if *fileName successfully opened as a DATfile (even 
       if the tagname not found) else 0.    */

int DATTagWrite(char *fileName, char *tagname, char *tagval);
    /* Write a descriptor field for the DATfile specified by <filename>.
       Opens the named DAT file (which must exist), write an entry 
       in the descriptor for *tagname as the null-terminated string 
       *tagval.
       Return 1 if *fileName successfully opened as a DATfile else 0. */
