#ifndef CDL_SLOT_H
#define CDL_SLOT_H

// CdlSlot.H
//
// Header file for the classes associated with the 'slot' operator 
// in the class definition language (.cdl) files of ObjectTCL (otcl)
//
// .. an extension to Dean Sheenan's ObjectTcl 1.1
//
// dpwe@media.mit.edu 1995nov11

// System Includes
#include <fstream.h>

// one method takes the tclInterp for error reporting
#include <tcl.h>

// Local Includes
#include "CdlMethod.H"

class CdlSlot : public CdlMethod {
// subclass it off CdlMethod to allow easy reuse of CdlFile code
// Use the {get|set}Rtn methods of CdlMethod to manipulate the type.
public:
    CdlSlot(char *name);
    virtual ~CdlSlot();

    void executeBinding(char *className, ofstream &of);
    void setReturn(ofstream& of, CdlIndent& tab);
    void setValFromStr(char *strName, ofstream& of, CdlIndent& tab);

    void setSetScript(char *script);
    void setGetScript(char *script);

public:
    char *setScript;
    char *getScript;
};

class CdlFakeSlot : public CdlSlot {
// An instance slot that is 'fake' i.e. not part of the C++ target, but 
// part of the TclCpp interface to it
public:
    CdlFakeSlot(char *name);
    virtual ~CdlFakeSlot();

    void setReturn(ofstream& of, CdlIndent& tab);
    void setValFromStr(char *strName, ofstream& of, CdlIndent& tab);

    void genDec (ofstream &);
    void genDefaultInit (ofstream &);
    void genDestruct (ofstream &);

    void setString(char *strName, ofstream& of, CdlIndent& tab);
    void initString(char *strName, ofstream& of, CdlIndent& tab);
    void cleanString(char *strName, ofstream& of, CdlIndent& tab);

    // check setup of fake slot
    int isOK(Tcl_Interp* i);
};

class CdlClassSlot : public CdlSlot {
// Class slot is a kind of slot.  Should really have a slot baseclass then 
// a ClassSlot and an InstanceSlot subclass, but this is smaller
public:
    CdlClassSlot(char *name);
    virtual ~CdlClassSlot();

    void setReturn(char *className, ofstream& of, CdlIndent& tab);
    void setValFromStr(char *className, char *strName, ofstream& of, CdlIndent& tab);

};

#endif /* CDL_SLOT_H */
