/*
 * pfif.h
 *
 * C language interface to QN pfile fns for Tcl
 * 
 * 1997apr02 dpwe@icsi.berkeley.edu
 * $Header: /u/drspeech/src/guitools/pfif_otcl/RCS/pfif.H,v 1.5 1999/05/22 00:42:35 dpwe Exp $
 */

#ifndef _PFIF_H_
#define _PFIF_H_

#include <floatArray/floatArray.H>
#include <QN_PFile.h>

class pfileif {
private:
    FILE *file;
//    QN_OutFtrLabStream_PFile* outpfile;
//    QN_InFtrLabStream_PFile* inpfile;
    QN_OutFtrLabStream* outpfile;
    QN_InFtrLabStream* inpfile;
    // whether this pfile is closed, or open for read or write
    int mode;
    // debug level (passed to QN functions)
    int debug;
    // flag for random access or not (passed to QN functions)
    int indexed;
    // details of the stream
    int nfeatures;
    int nlabels;
    int nutts;
    int totalframes;
    // keeping track of writes
    int current_utt;
    int current_frame;
    // options : normalized features
    int normutts;
    QN_InFtrStream *normipf;

    int openRead(char *filename);
    int openWrite(char *filename);

    int readFramesSub(int nframes, float *buf);
    int writeFramesSub(int nframes, const float *buf);

public:
    pfileif(int a_normutts = 0);
    ~pfileif(void);

    int open(char *filename, char *mode);
    void close(void);

    int getNumUtts(void) { return nutts; }
    int getNumFeatures(void) { return nfeatures; }
    int getNumFrames(void) { return totalframes; }
    /* getNumLabels() is deprecated, since NumLabels is assumed exactly 1 ?? */
    int getNumLabels(void) { return nlabels; }

    int setNumFeatures(int nfeats);

    int numFramesForUtt(int utt);

    int seek(int utt, int frame);
    int tell(int* putt, int *pframe);

    floatVec readFrame(void);
    floatArray readFrames(int nframes);

    /* always return 1D vector?? */
    int readLabels(int nlabels, int* labels);

    int writeFrame(floatRef& frame);
    int writeFrames(arrayRef& frames);

    int writeLabels(int nlabels, int* labels);

    int writeUttEnd(void);

    int isOpen(void);
};

#endif /* _PFIF_H_ */

