/*
 *  pfinfo.C - A Matlab C Extension (MEX) to get info on ICSI "pfile" files
 *  after DATRead.c
 *  Matlab 5.0 version
 *  1997jun11 dpwe@icsi.berkeley.edu
 *  $Header: /u/drspeech/src/SPRACHcore/guitools/pfif_otcl/RCS/pfinfo.C,v 1.2 1998/11/17 01:17:32 dpwe Exp $
 */

#include    <stdio.h>
#include    <math.h>
#include    <ctype.h>
#include    "mex.h"

#include    "pfif.H"

#define MAX_FNAME 300

extern "C"
void
mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    char    filename[MAX_FNAME];
    int     i, j;
    int     utnum = -1;		/* meaning: return info for whole file */

    /* have to instance this template to make link work OK */
    floatArray 	dummyFA(1,1);
    float   	*dummypf = &(dummyFA.El(0,0));
    
    if (nrhs < 1){
	mexPrintf("pfinfo    [R, U, F, L] = pfinfo(FILE [, UTNUM])  Get info on a pfile\n");
	mexPrintf("          specified by filename FILE, returning:\n");
	mexPrintf("          R as the total number of frames\n");
	mexPrintf("          U as the total number of utterances\n");
	mexPrintf("          F as the number of features per frame\n");
	mexPrintf("          L as the number of labels per frame (always 1)\n");
	mexPrintf("          if UTNUM is specified, R is the frames for that utterance alone.\n");
	mexPrintf("1997jun11 dpwe@icsi.berkeley.edu\n");
	return;
    }
    if (!mxIsChar(prhs[0])){	/* FILENAME */
	mexErrMsgTxt("pfinfo: first argument (FILENAME) must be a string");
	return;
    } else {
	/* Convert to String */
	mxGetString(prhs[0], filename, MAX_FNAME);
    }
    if(nrhs>1)  {		/* UTNUM */
	utnum = mxGetScalar(prhs[1]);
    }
    /* arguments all set up */
    /* open the file and find out what it has */
    pfileif pfif;
    if (!pfif.open(filename, "r")) {
    	mexErrMsgTxt("pfinfo: unable to open pfile");
    	return;
    }
    int nutts     = pfif.getNumUtts();
    int nframes   = pfif.getNumFrames();
    int nfeats    = pfif.getNumFeatures();
    int nlabs     = pfif.getNumLabels();
    if(utnum >=0) {
	/* we are asking about a particular utterance */
	if (utnum >= nutts) {
	    mexErrMsgTxt("pfinfo: requested UTNUM not in file");
	    return;
	}
	/* get size of requested utt */
	nframes = pfif.numFramesForUtt(utnum);
    }
    /* Fill in the returns */
    int nd = 2;
    int dims[2];
    dims[0] = 1;
    dims[1] = 1;
    /* first return is frame count */
    if(nlhs > 0) {
	mxArray *frms = mxCreateNumericArray(nd, dims, mxINT32_CLASS, mxREAL);
	*(int *)mxGetPr(frms) = nframes;
	plhs[0] = frms;
    }
    /* second return is utterance count */
    if(nlhs > 1) {
	mxArray *utts = mxCreateNumericArray(nd, dims, mxINT32_CLASS, mxREAL);
	*(int *)mxGetPr(utts) = nutts;
	plhs[1] = utts;
    }
    /* third return is feature width */
    if(nlhs > 2) {
	mxArray *feas = mxCreateNumericArray(nd, dims, mxINT32_CLASS, mxREAL);
	*(int *)mxGetPr(feas) = nfeats;
	plhs[2] = feas;
    }
    /* fourth return is label width */
    if(nlhs > 3) {
	mxArray *labs = mxCreateNumericArray(nd, dims, mxINT32_CLASS, mxREAL);
	*(int *)mxGetPr(labs) = nlabs;
	plhs[3] = labs;
    }
}
