/* 
 * stftSound.cc
 * 
 * Calculate the short-time fourier transform magnitude of a chunk of sound
 * 1996nov27 dpwe@icsi.berkeley.edu  after tohex.c
 * $Header: $
 */

/* std libs */
#include <stdlib.h>
#include <string.h>
/* personal libs */
#include <math.h>

extern "C" {
#include <dpwelib/dpwelib.h>
#include <dpwelib/genutils.h>
#include <dpwelib/vecops.h>

#ifdef SGI_FFT
#include "sgi_fft.h"
#else
#include <dpwelib/fft.h>
#endif /* SGI_FFT */
}

#include <dpwelib/Sound.H>
#include <floatArray/floatArray.H>

#define DFLT_FFTO	1024
#define DFLT_FFTH	256

int CalcSTFT(Sound* insnd, int firstFrame, int nFrames,
	     floatArray* outfa, int fftOrder, int hopSize, int winSize)
{  /* Calculate STFTM for a range of sound */
    long    len, remain, got, thistime;
    float   *sndbuf, *fftbuf, *window;
    int     chans, i;
    int     winskip, dataskip, dowinlen;
    int     opbins = fftOrder/2+1;
    float   val;
    int     v2;
    int     frames = 0, targFrames;
#ifdef SGI_FFT
    float   *workspace = NULL;
#endif /* SGI_FFT */

    if(fftOrder > winSize)  {
	/* window shorter than fft */
	dataskip = (fftOrder - winSize)/2;
	winskip = 0;
	dowinlen = winSize;
    } else {
	/* window as big as or bigger than fft */
	dataskip = 0;
	winskip  = (winSize - fftOrder)/2;
	dowinlen = fftOrder;
    }

    insnd->setChannels(1);
    sndbuf = new float[fftOrder];
    VZero(sndbuf, 1, fftOrder);
    fftbuf = new float[2*fftOrder];
    window = new float[winSize];
    VSetHanning(window, 1, 1.0, winSize);

#ifdef SGI_FFT
    workspace = sfft1dui(fftOrder, workspace);
#endif /* SGI_FFT */

/*    len = insnd->getFrames();
    remain = len; */
    remain = nFrames;

    targFrames = (int)(1+remain/hopSize);
/*    fprintf(stderr, "Expecting %d frames of %d cols\n", 
	    targFrames, opbins); */

    /* Setup float array */
    outfa->Resize(opbins, targFrames);

    /* preload just under half the buffer */
    /* remain -= insnd->readFrames(sndbuf+fftOrder/2, fftOrder/2-hopSize); */

    int startframe = firstFrame - fftOrder/2;
    if(startframe < 0) {
	insnd->frameSeek(0);
	insnd->readFrames(sndbuf - startframe + hopSize, 
			  fftOrder+startframe-hopSize);
    } else {
	insnd->frameSeek(startframe);
	insnd->readFrames(sndbuf + hopSize, fftOrder-hopSize);
    }

    /* while(remain)  { */
    while(frames < targFrames)  {
	thistime = MIN(remain, hopSize);
	/* shift remainder back in sndbuf */
	VCopy(sndbuf+hopSize, 1, sndbuf, 1, fftOrder-hopSize);
	/* read new frames */
	got = insnd->readFrames(sndbuf+fftOrder-hopSize, thistime);
	if(got < hopSize) {
	    VZero(sndbuf+fftOrder-hopSize+got, 1, hopSize-got);
	}
	if(dataskip) {   /* need to clear residual of previous FFT */
	    VZero(fftbuf, 1, 2*fftOrder);
	}
	/* VCopyZero(sndbuf, 1, fftOrder-hopSize+got, fftbuf, 1, fftOrder); */
	VCopyZero(sndbuf+dataskip, 1, fftOrder-hopSize+got-dataskip, 
		  fftbuf+dataskip, 1, dowinlen);
	VVMul(window+winskip, 1, fftbuf+dataskip, 1, dowinlen);
#ifdef SGI_FFT
	sfft1du(-1, fftOrder, fftbuf, 1, workspace);
#else /* SGI_FFT */
	fft_packed(fftbuf, fftOrder);
#endif /* SGI_FFT */
	RCVmag((cpxrect *)fftbuf, 1, (cpxpolar *)fftbuf, 1, opbins);
	for(i = 0; i<opbins; ++i)  {
	    val = 4.343*log(fftbuf[2*i]);
	    outfa->El(i, frames) = val;
	}
	++frames;
	if(got < thistime )
	    remain = 0;		/* hit end of something */
	else
	    remain -= thistime;
    }
/*    fprintf(stderr, "%d frames processed, written to %s\n", frames, outpath); */

    /* release buffers */
    delete [] window;
    delete [] fftbuf;
    delete [] sndbuf;

    return frames;
}

