/*
 * stftSoundInit.C
 * 
 * Init file for stftSound, tcl interface to FFT of sound frames
 *
 * 1997aug11 dpwe@icsi.berkeley.edu after SoundInit.cc
 * $Header: /n/yam/da/dpwe/src/tcl/sound_otcl/RCS/stftSoundInit.C,v 1.1 1997/08/13 01:42:28 dpwe Exp dpwe $
 */

#include <tcl.h>
#include <dpwelib/Sound.H>
#include <SoundTcl.H>
#include <floatArray/floatArray.H>
#include <floatArray/floatArrayTcl.H>

/****************** The stftSound command **************/

#define DFLT_FFTORDER	1024
#define DFLT_HOPFACT	4
/* default hopsize = fftorder/HOPFACT */

extern int CalcSTFT(Sound* insnd, int firstFrame, int nFrames,
		    floatArray* outfa, int fftOrder, int fftHop, int winSize);

int stftSoundCommand(ClientData /* clientData */, Tcl_Interp *interp, 
		 int argc, char **argv)
{	/* tcl command to instance a new sound object */
    if(argc < 3 || argc > 8) {
	Tcl_AppendResult(interp, "wrong $ args: should be \"", 
			 argv[0], " sound farray ?firstframe? ?nframes? ?fftorder? ?ffthop? ?winsize?", NULL);
	return TCL_ERROR;
    }

    int firstframe, nframes, fftorder, ffthop, winsize;
    floatArray *faPtr;
    Sound *snd;

    Tcl_HashEntry *entryPtr;

    /* Get the sound */
    if( (snd = SoundFromName(argv[1])) == NULL) {
	// no sound registered to that name
	Tcl_AppendResult(interp, "no such sound: ", argv[1], NULL);
	return TCL_ERROR;
    }

    /* Get the floatArray */
    if( (faPtr = FAfromName(argv[2])) == NULL) {
	Tcl_AppendResult(interp, "no such floatArray: ", argv[2], NULL);
	return TCL_ERROR;
    }
    
    /* Other args, or defaults */
    firstframe = 0;
    nframes = 0;
    if(argc > 3) {
	if (Tcl_GetInt(interp, argv[3], &firstframe) != TCL_OK) {
	    return TCL_ERROR;
	}
    }
    if(argc > 4) {
	if (Tcl_GetInt(interp, argv[4], &nframes) != TCL_OK) {
	    return TCL_ERROR;
	}
    }
    if(nframes <= 0) {		// nframes == 0 means whole (rest of) sound
	nframes = snd->getFrames() - firstframe;
    }
    if(argc > 5) {
	if (Tcl_GetInt(interp, argv[5], &fftorder) != TCL_OK) {
	    return TCL_ERROR;
	}
    } else {
	fftorder = DFLT_FFTORDER;
    }
    if(argc > 6) {
	if (Tcl_GetInt(interp, argv[6], &ffthop) != TCL_OK) {
	    return TCL_ERROR;
	}
    } else {
	ffthop = fftorder/DFLT_HOPFACT;
    }
    if(argc > 7) {
	if (Tcl_GetInt(interp, argv[7], &winsize) != TCL_OK) {
	    return TCL_ERROR;
	}
    } else {
	winsize = fftorder;
    }
    /* OK, do it */
    int cols = CalcSTFT(snd, firstframe, nframes, faPtr, fftorder, ffthop, winsize);
    sprintf(interp->result, "%d", cols);
    return TCL_OK;
}

EXTERN int SoundStft_Init(Tcl_Interp *interp);

int SoundStft_Init(Tcl_Interp *interp)
{   /* add the tcl interface to the stftSound command */
    Tcl_CreateCommand(interp, "stftsound", stftSoundCommand, (ClientData) NULL,
	    (void (*)(ClientData)) NULL);
    return TCL_OK;
}

/* Initialization commands */
extern "C" int Stftsound_Init(Tcl_Interp *interp)
{   /* Package opening call */
    int rc = TCL_OK;
    
    // for Tcl7.5+, signal the package mechanism
    Tcl_PkgProvide(interp, "StftSound", SOUND_VERSION);
    
    if(SoundStft_Init(interp)==TCL_ERROR) {return TCL_ERROR;}

    return rc;
}

extern "C" int Stftsound_SafeInit(Tcl_Interp *interp)
{   /* Package opening call */
    return Stftsound_Init(interp);
}

