/*
   $Header: /u/spert/src/intvec/testsuite/RCS/vecele_vi16_vi16_test.c,v 1.2 1996/01/05 00:18:57 davidj Exp $

   Test element wise vector routines of the form op_vi16_vi16.
*/

#include <stddef.h>
#include <stdlib.h>

#include "intvec_test.h"


typedef void (*rtn_t)(size_t len, const intv_int16_t* in,
		      intv_int16_t* res);

/* Performs one test given input values and expected output. */

void
onetest_normal(rtn_t rtn, size_t len,
	const intv_int16_t* in, const intv_int16_t* correct)
{
    /* Create padded copies of inputs. */
    intv_int16_t* c_in = rtst_padvec_clone_vi16(len, in);
    intv_int16_t* c_res = rtst_padvec_new_vi16(len);

    /* Run routine. */
    rtn(len, c_in, c_res);

    /* Check result. */
    rtst_checkeq_vi16vi16(len, c_res, correct);

    /* Check input not corrupted. */
    rtst_checkeq_vi16vi16(len, in, c_in);

    /* Free and check padding. */
    rtst_padvec_del_vi16(c_in);
    rtst_padvec_del_vi16(c_res);
}

void
copy_6(rtn_t rtn)
{
    static const intv_int16_t in[] = { 0, -1, 1, -12345, 0x7fff, 0x8000};
    static const intv_int16_t correct[] = { 0, -1, 1, -12345, 0x7fff, 0x8000};
    
    onetest_normal(rtn, 6, in, correct);
}

void
swapb_explicit(rtn_t rtn)
{
    static const intv_int16_t in[] = { 0, 0x1234, 0xfedc, 0xffff };
    static const intv_int16_t correct[] = { 0, 0x3412, 0xdcfe, 0xffff };
    
    onetest_normal(rtn, sizeof(in)/sizeof(in[0]), in, correct);
}

/* Compare the results of two vector routines */

void
random_test(rtn_t nv_rtn, rtn_t test_rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;
    intv_int16_t *in = rtst_padvec_new_vi16(maxlen);
    intv_int16_t *res = rtst_padvec_new_vi16(maxlen);

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len = (size_t) rtst_urand_i32i32_i32(1, maxlen);

        rtst_badrand_vi16(len, in);
        nv_rtn(len, in, res);

        rtst_log("len=%d\n", len);

        onetest_normal(test_rtn, len, in, res);
    }

    rtst_padvec_del_vi16(in);
    rtst_padvec_del_vi16(res);
}

void
test_copy(void)
{
    rtst_start("copy_vi16_vi16");
    rtst_log("testing nv.\n");
    copy_6(nv_copy_vi16_vi16);
#ifdef CHECK_TOR
    rtst_log("testing tor.\n");
    copy_6(tor_copy_vi16_vi16);
    random_test(nv_copy_vi16_vi16, tor_copy_vi16_vi16);
#endif
    rtst_passed();
}

void
test_swapb(void)
{
    rtst_start("swapb_vi16_vi16");
    rtst_log("testing nv.\n");
    swapb_explicit(nv_swapb_vi16_vi16);
/*
#ifdef CHECK_TOR
    rtst_log("testing tor.\n");
    swapb_explicit(tor_swapb_vi16_vi16);
    random_test(nv_swapb_vi16_vi16, tor_swapb_vi16_vi16);
#endif
*/
    rtst_passed();
}

int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    test_copy();
    test_swapb();

    rtst_exit();

    return 0;
}
