/*
   $Header: /u/spert/src/intvec/testsuite/RCS/vecele_vi8_vi8_test.c,v 1.2 1996/01/03 20:56:58 davidj Exp $

   Test element wise vector routines of the form op_vi8_vi8.
*/

#include <stddef.h>
#include <stdlib.h>

#include "intvec_test.h"


typedef void (*rtn_t)(size_t len, const intv_int8_t* in,
		      intv_int8_t* res);

/* Performs one test given input values and expected output. */

void
onetest_normal(rtn_t rtn, size_t len,
	const intv_int8_t* in, const intv_int8_t* correct)
{
    /* Create padded copies of inputs. */
    intv_int8_t* c_in = rtst_padvec_clone_vi8(len, in);
    intv_int8_t* c_res = rtst_padvec_new_vi8(len);

    /* Run routine. */
    rtn(len, c_in, c_res);

    /* Check result. */
    rtst_checkeq_vi8vi8(len, c_res, correct);

    /* Check input not corrupted. */
    rtst_checkeq_vi8vi8(len, in, c_in);

    /* Free and check padding. */
    rtst_padvec_del_vi8(c_in);
    rtst_padvec_del_vi8(c_res);
}

void
copy_6(rtn_t rtn)
{
    const intv_int8_t in[] = { 0, -1, 1, -12, 0x7f, 0x80};
    const intv_int8_t correct[] = { 0, -1, 1, -12, 0x7f, 0x80};
    
    onetest_normal(rtn, 6, in, correct);
}

/* Compare the results of two vector routines */

void
random_test(rtn_t nv_rtn, rtn_t test_rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;
    intv_int8_t *in = rtst_padvec_new_vi8(maxlen);
    intv_int8_t *res = rtst_padvec_new_vi8(maxlen);

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len = (size_t) rtst_urand_i32i32_i32(1, maxlen);

        rtst_urand_i8i8_vi8(len, INTV_INT8_MIN, INTV_INT8_MAX, in);
        nv_rtn(len, in, res);

        rtst_log("len=%d\n", len);

        onetest_normal(test_rtn, len, in, res);
    }

    rtst_padvec_del_vi8(in);
    rtst_padvec_del_vi8(res);
}

void
test_copy(void)
{
    rtst_start("copy_vi8_vi8");
    rtst_log("testing nv.\n");
    copy_6(nv_copy_vi8_vi8);
#ifdef CHECK_TOR
    rtst_log("testing tor.\n");
    copy_6(tor_copy_vi8_vi8);
    random_test(nv_copy_vi8_vi8, tor_copy_vi8_vi8);
#endif
     rtst_passed();
}

int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    test_copy();

    rtst_exit();

    return 0;
}
