/*
** $Id: mul_vi32i32_vi32.S,v 1.3 1995/08/31 00:35:45 davidj Exp $
**
** davidj - Thu Aug 25 18:28:38 1994
*/

#include <tor/tordef.h>

#ifdef DECL_tor_mul_vi32i32_vi32
void tor_mul_vi32i32_vi32(size_t len, const intv_int32_t* a,
			  intv_int32_t b, intv_int32_t* res);
#endif

#ifdef DEFN_tor_mul_vi32i32_vi32

	.globl	tor_mul_vi32i32_vi32

#define	a_v		vv1
#define ah_v		vv2
#define albl_v		vv3
#define ahbl_v		vv4
#define albh_v		vv5
#define res_v		vv6
#define temp_v		vv7

#define len_r		a0
#define aptr_r		a1
#define b_r		a2
#define resptr_r	a3

#define bh_r		t0
#define shift_r		t1
#define fourvlr_r	t2
#define conf_muluu_r	t3
#define conf_mulsu_r	t4
#define conf_mulus_r	t5
#define conf_shladd_r	t6
#define done_r		t7


	.ent	tor_mul_vi32i32_vi32
tor_mul_vi32i32_vi32:
 # Check the length - jump forward if > 32

        .set    noreorder
        bgtu    len_r, MAXVLEN, morethan32
        ctvu    len_r, vlr              # Assume len <= 32
        .set    reorder

 # The short case (len<=32) follows directly for better cache performance

        lw.v	a_v, aptr_r
	li	conf_muluu_r, (FX_AU | FX_BU | FX_CLIP_PASS)
	li	conf_shladd_r, ((16*FX_SHLAM) | FX_SEPSHAM \
			       | FX_NOROUND | FX_CLIP_PASS)
	li	shift_r, 16
	fxmul.vs res_v, a_v, b_r, conf_muluu_r
	srav.vs	ah_v, a_v, shift_r
	sra	bh_r, b_r, 16
	li	conf_mulus_r, (FX_AU | FX_CLIP_PASS)
	fxmul.vs albh_v, a_v, bh_r, conf_mulus_r
	li	conf_mulsu_r, (FX_BU | FX_CLIP_PASS)
	fxadd.vv res_v, albh_v, res_v, conf_shladd_r
	fxmul.vs ahbl_v, ah_v, b_r, conf_mulsu_r
	fxadd.vv res_v, ahbl_v, res_v, conf_shladd_r
	sw.v	res_v, resptr_r
	j	ra	

 # The len>32 case

morethan32:
	li	v0, MAXVLEN
	ctvu	v0, vlr
	lw.v	a_v, aptr_r
	add	aptr_r, MAXVLEN*4
	li	fourvlr_r, MAXVLEN*4
	li	conf_muluu_r, (FX_AU | FX_BU | FX_CLIP_PASS)
	li	shift_r, 16
	fxmul.vs albl_v, a_v, b_r, conf_muluu_r
	srav.vs	ah_v, a_v, shift_r
	li	conf_mulus_r, (FX_AU | FX_CLIP_PASS)
	sra	bh_r, b_r, 16
	fxmul.vs albh_v, a_v, bh_r, conf_mulus_r
	li	conf_shladd_r, ((16*FX_SHLAM) | FX_SEPSHAM \
			       | FX_NOROUND | FX_CLIP_PASS)
	fxadd.vv temp_v, albh_v, albl_v, conf_shladd_r
	li	conf_mulsu_r, (FX_BU | FX_CLIP_PASS)
	fxmul.vs ahbl_v, ah_v, b_r, conf_mulsu_r


loop:
	lwai.v	a_v, aptr_r, fourvlr_r
	subu	len_r, MAXVLEN
 	sltu	done_r, len_r, MAXVLEN+1
	fxadd.vv res_v, ahbl_v, temp_v, conf_shladd_r
	fxmul.vs albl_v, a_v, b_r, conf_muluu_r
	srav.vs	ah_v, a_v, shift_r
	swai.v	res_v, resptr_r, fourvlr_r
	fxmul.vs albh_v, a_v, bh_r, conf_mulus_r
	fxadd.vv temp_v, albh_v, albl_v, conf_shladd_r
	fxmul.vs ahbl_v, ah_v, b_r, conf_mulsu_r
	beqz	done_r, loop

/* Wind up after the loop */

	ctvu	len_r, vlr
	fxadd.vv res_v, ahbl_v, temp_v, conf_shladd_r
	sw.v	res_v, resptr_r

	j	ra

	.end	tor_mul_vi32i32_vi32


#endif
