. $Header: /n/abbott/da/drspeech/src/quicknet/RCS/MLPW.man,v 1.1 2001/03/13 19:56:41 dpwe Exp $
.TH MLPW 5 "$Date: 2001/03/13 19:56:41 $" ICSI "ICSI SPEECH SOFTWARE"
.SH NAME
mlpw \- Family of binary-encoded neural-net weights file formats used by QuickNet
.SH DESCRIPTION
.LP
The \fImlpw\fP
file format is used to store neural net weights in a more compact and 
more quickly-accessed format than the traditional ASCII RAP3
.BR weights (5)
format.  The same information is stored in the same order, but 
the values are coded, typically as 32 bit floats or 16 bit fixed-point 
ints, less often as 8 or 32 bit ints, or 64 bit doubles.
Each section (e.g. weights or biases of a particular layer) may be 
coded in a different format.
.LP
\fImlpw\fP files are usually created with 
.BR qnstrn (1)
(or will be when it is modified to support them)
and converted to and from other formats with 
.BR qncopywts (1).  
They will be read directly by future versions of
.BR qnsfwd (1) 
and
.BR ffwd (1).
.SS The header
The header as currently defined consists of 5 4-byte integers 
in big-endian order:
.P
.ta .5i +1.5i
.nf
	\f4magic\f1	magic number = 0x4D4C5057 ("MLPW")
	\f4version\f1	version code = 20010313 (today)
	\f4nettype\f1	nettype/version (e.g. softmax)
	\f4nlayers\f1	count of unit layers (3 for MLP3)
	\f4nsections\f1	count of sections (4 for MLP3)
.fi
.P
Then follow \f4nlayers\f1 4-byte ints specifying the number of units in 
each layer (starting at the input), followed by the sections.
.P
Each section also has a small header, consisting of 3 4-byte integers:
.P
.ta .5i +1.5i
.nf
	\f4sectiontype\f1	QN_SectionSelector tag for this section
	\f4numvalues\f1	how many weights in this section
	\f4datatype\f1	data type flag (bytes/wt + 32 for float)
.fi
.P
For fixed-point data formats (only), this is followed by 
a 4-byte int giving the fixed-point 'exponent' 
for this section.  
After this come the actual coded weight values.
.P
In an MLP3, there are 4 sections:  (0) 
input-to-hidden weights, (1) hidden-to-output weights, (2) hidden layer 
bias weights, and (3) output layer bias weights.
Since bias values occupy a slightly different range (they are typically 
distributed around -log(n_units)), they are often stored with a larger 
exponent and/or more bits per weight.  The MLPW file format 
supports this without difficulty.
.SH NOTES/BUGS
Short-format (MLPWS) files are typically 1/4 the size of ASCII RAP3 
files, or 1/2 the size of gzipped ASCII files, and load 5-10x faster. 
Since the weights are calculated on the SPERT boards using 16 bit 
fixed-point arithmetic, there is usually no accuracy loss in storing 
them this way. 
.P
You shouldn't ever have to access these files directly. 
Instead, use the QuickNet class 
.BR QN_MLPWeightFile_MLPW (3).
.P
Little tested at present.
.SH AUTHOR
Dan Ellis <dpwe@ee.columbia.edu>
.SH SEE ALSO
.BR qncopywts (1).
