// -*- C++ -*-
// $Header: /n/abbott/da/drspeech/src/SPRACHcore/quicknet/RCS/QN_MLPWeightFile_MLPW.cc,v 1.3 2001/04/16 14:07:31 dpwe Exp $
// 
// QN_MLPWeightFle_MLPW.cc
//
// Define a new *binary* MLP Weight file format
// 1997jun19 dpwe@icsi.berkeley.edu
//

#ifndef NO_RCSID
const char* QN_MLPWeightFile_MLPW_rcsid =
    "$Header: /n/abbott/da/drspeech/src/SPRACHcore/quicknet/RCS/QN_MLPWeightFile_MLPW.cc,v 1.3 2001/04/16 14:07:31 dpwe Exp $";
#endif

/* Must include the config.h file first */
#include <QN_config.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include <string.h>
#include <math.h>
#include "fltvec.h"

#ifdef HAVE_LIBFXLIB
extern "C" {
#include "fxlib.h"
};
#endif

#include "QN_types.h"
#include "QN_Logger.h"
#include "QN_utils.h"
#include "QN_libc.h"
#include "QN_MLPWeightFile_MLPW.h"

// Missing intvec fns
#ifndef QNInt16
typedef short QNInt16;
#endif
#ifndef QNInt8
typedef signed char QNInt8;
#endif

void
conv_vi32_vi16(size_t len, QNInt32* from, QNInt16* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
        *to++ = (*from++)>>16;
    }
}

/* in fxlib
void
conv_vi16_vi32(size_t len, QNInt16* from, QNInt32* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
        *to++ = ((QNInt32)*from++)<<16;
    }
}
*/

void
conv_vi32_vi8(size_t len, QNInt32* from, QNInt8* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
        *to++ = (*from++)>>24;
    }
}

/* in QN_libc 
void
conv_vi8_vi32(size_t len, QNInt8* from, QNInt32* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
        *to++ = ((QNInt32)*from++)<<24;
    }
}
*/

void
htob_vd_vd(size_t len, double *from, double *to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
#ifdef WORDS_BIGENDIAN
	*to++ = *from++;
#else 
	QNInt32 *p1 = (QNInt32 *)to;
	QNInt32 *p3 = (QNInt32 *)from;
	QNInt32 *p2 = p1 + 1;
	QNInt32 *p4 = p3 + 1;
	*p1 = btoh_i32_i32(*p4);
	*p2 = btoh_i32_i32(*p3);
#endif
    }
}

void
btoh_vd_vd(size_t len, double *from, double *to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
#ifdef WORDS_BIGENDIAN
	*to++ = *from++;
#else 
	QNInt32 *p1 = (QNInt32 *)to;
	QNInt32 *p3 = (QNInt32 *)from;
	QNInt32 *p2 = p1 + 1;
	QNInt32 *p4 = p3 + 1;
	*p1 = btoh_i32_i32(*p4);
	*p2 = btoh_i32_i32(*p3);
#endif
    }
}

#ifndef min
#define min(a,b) ((a)<(b)?(a):(b))
#endif

#ifndef max
#define max(a,b) ((a)>(b)?(a):(b))
#endif

#ifndef HAVE_LIBFXLIB
// fxlib missing, must provide here
void conv_vx16_vf(size_t len, int res_exp, const QNInt16* in, float* res) {
    double k = pow(2.0, res_exp - 15);
    while(len--) {
	*res++ = (float)k*(*in++);
    }
}
void conv_vx32_vf(size_t len, int res_exp, const QNInt32* in, float* res) {
    double k = pow(2.0, res_exp - 31);
    while(len--) {
	*res++ = (float)k*(*in++);
    }
}
void conv_vf_vx16s(size_t len, int res_exp, const float* in,
		   QNInt16* res) {
    double k = pow(2.0, 15 - res_exp);    
    while(len--) {
	*res++ = (QNInt16)rint( (*in++) * k );
    }
}
void conv_vf_vx32s(size_t len, int res_exp, const float* in,
		   QNInt32* res) {
    double k = pow(2.0, 31 - res_exp);
    while(len--) {
	*res++ = (QNInt32)rint( (*in++) * k );
    }
}
#endif


// Name of this class
const char* QN_MLPWeightFile_MLPW::CLASSNAME = "QN_MLPWeightFile_MLPW";

int QN_MLPWeightFile_MLPW::read_header(void) {
    // Read the header of an MLPW file
    char buf[QN_MLPW_SIZEOFMAGIC];
    int nr;
    QNInt32 tmp;
    int version;

    if(nr = fread(buf, QN_MLPW_SIZEOFMAGIC, 1, stream)) {
	if (strncmp(buf, QN_MLPW_MAGIC, QN_MLPW_SIZEOFMAGIC) == 0) {
	    // Magic number looks good - read other fields
	    nr += fread(&tmp, sizeof(QNInt32), 1, stream);
	    version = btoh_i32_i32(tmp);
	    if (version == QN_MLPW_VERSION) {
		nr += fread(&tmp, sizeof(QNInt32), 1, stream);
		netType = btoh_i32_i32(tmp);
		nr += fread(&tmp, sizeof(QNInt32), 1, stream);
		nLayers = btoh_i32_i32(tmp);
		nr += fread(&tmp, sizeof(QNInt32), 1, stream);
		nSections = btoh_i32_i32(tmp);
		int i;
		// Read layer sizes
		if (nLayers < QN_MLPW_MAX_NLAYERS) {
		    for (i = 0; i < nLayers; ++i) {
			nr += fread(&tmp, sizeof(QNInt32), 1, stream);
			layerSize[i] = btoh_i32_i32(tmp);
		    }
		}
		// Check that we read enough
		if (nr == 5+nLayers) {
		    // success
		    return 1;
		}
	    }
	}
    }
    // fail
    return 0;    
}

int QN_MLPWeightFile_MLPW::write_header(void) {
    // Write the header of an MLPW file
    int nw;
    QNInt32 tmp;

    nw = fwrite(QN_MLPW_MAGIC, QN_MLPW_SIZEOFMAGIC, 1, stream);
    tmp = htob_i32_i32(QN_MLPW_VERSION);
    nw += fwrite(&tmp, sizeof(QNInt32), 1, stream);
    tmp = htob_i32_i32(netType);
    nw += fwrite(&tmp, sizeof(QNInt32), 1, stream);
    tmp = htob_i32_i32(nLayers);
    nw += fwrite(&tmp, sizeof(QNInt32), 1, stream);
    tmp = htob_i32_i32(nSections);
    nw += fwrite(&tmp, sizeof(QNInt32), 1, stream);
    int i;
    for (i = 0; i < nLayers; ++i) {
	tmp = htob_i32_i32(layerSize[i]);
	nw += fwrite(&tmp, sizeof(QNInt32), 1, stream);
    }
    // Check that we wrote enough
    if (nw == 5+nLayers) {
	// success
	return 1;
    }
    // fail
    return 0;    
}

int QN_MLPWeightFile_MLPW::read_sect_header(int sect, 
					    int numvals, 
					    QN_MLPW_dtype* datatype, 
					    int* exponent) {
    int nr;
    QNUInt32 tmp;

    if(nr = fread(&tmp, sizeof(QNUInt32), 1, stream)) {
	tmp = btoh_i32_i32(tmp);
	if ((QN_SectionSelector)tmp == sect) {
	    // It looks like the right section
	    tmp = 0;
	    nr += fread(&tmp, sizeof(QNUInt32), 1, stream);
	    tmp = btoh_i32_i32(tmp);
	    if (numvals != tmp) {
		QN_ERROR(CLASSNAME, "error reading MLPW sect header from \'%s\': expecting %d vals, got %d", 
			 filename, numvals, tmp);
	    }
	    // read datatype
	    nr += fread(&tmp, sizeof(QNUInt32), 1, stream);
	    *datatype = (QN_MLPW_dtype)btoh_i32_i32(tmp);
	    // maybe read exponent
	    if (*datatype == QN_MLPW_dt_int32 \
		|| *datatype == QN_MLPW_dt_int16 \
		|| *datatype == QN_MLPW_dt_int8) {
		// Read wt & bias exponents for fixed-point types
		fread(&tmp, sizeof(QNInt32), 1, stream);
		*exponent = btoh_i32_i32(tmp);
	    }
	    // Check that we read enough
	    if (nr == 3) {
		// success
		return 1;
	    }
	}
    }
    // fail
    return 0;    
}

int QN_MLPWeightFile_MLPW::write_sect_header(int sect, 
					    int numvals, 
					    QN_MLPW_dtype datatype, 
					    int exponent /* = 0 */) {
    int nw;
    QNUInt32 tmp;

    // write section ID
    tmp = htob_i32_i32((QNUInt32)sect);
    nw = fwrite(&tmp, sizeof(QNUInt32), 1, stream);
    // write vector size
    tmp = htob_i32_i32((QNUInt32)numvals);
    nw += fwrite(&tmp, sizeof(QNUInt32), 1, stream);
    // write datatype
    tmp = htob_i32_i32((QNUInt32)datatype);
    nw += fwrite(&tmp, sizeof(QNUInt32), 1, stream);
    // maybe write exponent
    if (datatype == QN_MLPW_dt_int32 \
	|| datatype == QN_MLPW_dt_int16 \
	|| datatype == QN_MLPW_dt_int8) {
	tmp = htob_i32_i32((QNUInt32)exponent);
	fwrite(&tmp, sizeof(QNUInt32), 1, stream);
    }
    // Check that we wrote enough
    if (nw == 3) {
	// success
	return 1;
    }
    // fail
    return 0;    
}

// Constructor for an input weight file

QN_MLPWeightFile_MLPW::QN_MLPWeightFile_MLPW(int dbg, FILE* a_stream, 
					     QN_FileMode a_mode,
					     const char* a_name,
					     int n_layers, 
					     int layer_sizes[], 
					     QN_MLPW_ntype nt /* = none */)
  : debug(dbg),
    mode(a_mode),
    stream(a_stream),
    filename(a_name)
{

    int i;
    for (i = 0; i < QN_MLPW_MAX_NSECTS; ++i) {
	// uninitialized values to say we haven't read them yet
	exponents[i] = -1;
	datatype[i] = QN_MLPW_dt_none;
    }

    if (a_mode==QN_READ)
    {
	if (debug>=2)
	{
	    QN_LOG(CLASSNAME, "Accessing MLPW file %s for reading.",
		   filename);
	}
	if (read_header() == 0) {
	    QN_ERROR(CLASSNAME, "error reading MLPW header from \'%s\'.", 
		     filename);
	}
	// Reading header sets up local fields
	if (n_layers > 0 && nLayers != n_layers) {
		QN_ERROR(CLASSNAME, "number of layers in MLPW file \'%s\' is"
			 " %lu - it should be %lu.", filename, nLayers, 
			 n_layers);
	}
	int i; 
	for (i = 0; i < n_layers; ++i) {
	    if (layerSize[i] != layer_sizes[i]) {
		QN_ERROR(CLASSNAME, "number of units in layer %i of MLPW "
			 "file \'%s\' is %lu - it should be %lu.", 
			 i, filename, layerSize[i], layer_sizes[i]);
	    }
	}

	// Read past first vector header, so we are ready to read data
	io_count = layerSize[0] * layerSize[1];
	io_state = QN_MLP3_INPUT2HIDDEN;
	read_sect_header(io_state, io_count, &datatype[0], &exponents[0]);
	exp = exponents[0];
	dtype = datatype[0];
    }
    else if (a_mode == QN_WRITE)
    {
	if (debug>=2)
	{
	    QN_LOG(CLASSNAME, "Accessing MLPW file %s for writing.",
		   filename);
	}
	// default data types - integer?
	datatype[0] = QN_MLPW_dt_int16;	// i2h
	datatype[1] = QN_MLPW_dt_int16;	// h2o
	datatype[2] = QN_MLPW_dt_int32;	// hbias
	datatype[3] = QN_MLPW_dt_int32;	// obias
	// default fxlib exponent for wts and biases
	exponents[0] = 2;	// i2h wts
	exponents[1] = 2;	// h2o wts
	exponents[2] = 4;	// h bias
	exponents[3] = 4;	// o bias

	nLayers = n_layers;
	nSections = (nLayers-1)*2;

	int i;
	for (i = 0; i < nLayers; ++i) {
	    layerSize[i] = layer_sizes[i];
	}

	// io_state holds the QN_SectionSelector we're about to write,
	// Use a dummy value to say we haven't written the header yet
#define QN_MLP3_HEADER QN_WEIGHTS_UNKNOWN
	
	io_state = QN_MLP3_HEADER;

    }
    else
	assert(0);

    // We support only 3 layer MLPs as it stands
    assert(nLayers == 3);

}

enum QN_SectionSelector
QN_MLPWeightFile_MLPW::get_weighttype(int section)
{
    switch(section)
    {
    case 0:
	return QN_MLP3_INPUT2HIDDEN;
	break;
    case 1:
	return QN_MLP3_HIDDEN2OUTPUT;
	break;
    case 2:
	return QN_MLP3_HIDDENBIAS;
	break;
    case 3:
	return QN_MLP3_OUTPUTBIAS;
	break;
    default:
	return QN_WEIGHTS_UNKNOWN;
	break;
    }
}

// Set the fixed-point exponents & datatypes
void
QN_MLPWeightFile_MLPW::set_datatype(QN_SectionSelector sect, 
				    QN_MLPW_dtype dt, int a_exp /* = 0 */) {
    switch(sect) {
    case QN_MLP3_INPUT2HIDDEN:
	datatype[0] = dt;
	exponents[0] = a_exp;
	break;
    case QN_MLP3_HIDDEN2OUTPUT:
	datatype[1] = dt;
	exponents[1] = a_exp;
	break;
    case QN_MLP3_HIDDENBIAS:
	datatype[2] = dt;
	exponents[2] = a_exp;
	break;
    case QN_MLP3_OUTPUTBIAS:
	datatype[3] = dt;
	exponents[3] = a_exp;
	break;
    default:
	QN_ERROR(CLASSNAME, "attempt to set type for unknown section %d.", 
		 sect);
	break;
    }
    // maybe update cached exp value
    if (sect == io_state) {
	exp = a_exp;
	dtype = dt;
    }
}
    
// Read the fixed-point exponents & datatypes
void
QN_MLPWeightFile_MLPW::get_datatype(QN_SectionSelector sect, 
				    QN_MLPW_dtype *dt, int *a_exp /* = NULL */) {
    switch(sect) {
    case QN_MLP3_INPUT2HIDDEN:
	*dt = datatype[0];
	if(a_exp) *a_exp = exponents[0];
	break;
    case QN_MLP3_HIDDEN2OUTPUT:
	*dt = datatype[1];
	if(a_exp) *a_exp = exponents[1];
	break;
    case QN_MLP3_HIDDENBIAS:
	*dt = datatype[2];
	if(a_exp) *a_exp = exponents[2];
	break;
    case QN_MLP3_OUTPUTBIAS:
	*dt = datatype[3];
	if(a_exp) *a_exp = exponents[3];
	break;
    default:
	QN_ERROR(CLASSNAME, "attempt to get type for unknown section %d.", 
		 sect);
	break;
    }
}
    
// Read in a buffer full of weights
size_t QN_MLPWeightFile_MLPW::read(float* buf, size_t count) {
#define QN_MLPW_BUF_SIZE 1024
    float *mybuf = new float[QN_MLPW_BUF_SIZE];

    assert(mode == QN_READ);

    // This is a hack so we can keep attempting to write past the end of file
    // without crashing
    if (io_count==0 || count==0)
	return(0);

    int thiscount, thiscountrem, thisblock;
    int bytespersamp = QN_MLPW_dt_sizeof(dtype);
    int wtsperbuf = QN_MLPW_BUF_SIZE*(sizeof(float)/max(sizeof(float),bytespersamp));
    int countrem = count;

    while (countrem > 0 & io_count > 0) {
	thiscount = min(countrem, io_count);
	thiscountrem = thiscount;
	while (thiscountrem > 0) {
	    thisblock = min(wtsperbuf, thiscountrem); 
	    // Do the read
	    if (fread(mybuf, bytespersamp, thisblock, stream) != thisblock) {
		QN_ERROR(CLASSNAME, "error reading weight file \'%s\' - %s",
			 filename, strerror(errno));
	    }
	    
	    // Convert data
	    switch (dtype) {
	    case QN_MLPW_dt_float:
		btoh_vf_vf(thisblock, mybuf, buf);
		break;
	    case QN_MLPW_dt_double:
		btoh_vd_vd(thisblock, (double *)mybuf, (double *)mybuf);
		conv_vd_vf(thisblock, (double *)mybuf, buf);
		break;
	    case QN_MLPW_dt_int32:
		btoh_vi32_vi32(thisblock, (QNInt32 *)mybuf,(QNInt32 *)mybuf);
		conv_vx32_vf(thisblock, exp, (QNInt32 *)mybuf, buf);
		break;
	    case QN_MLPW_dt_int16:
		btoh_vi16_vi16(thisblock, (QNInt16 *)mybuf,(QNInt16 *)mybuf);
		conv_vx16_vf(thisblock, exp, (QNInt16 *)mybuf, buf);
		break;
	    case QN_MLPW_dt_int8:
		conv_vi8_vi32(thisblock, (QNInt8 *)mybuf,(QNInt32 *)mybuf);
		conv_vx32_vf(thisblock, exp, (QNInt32 *)mybuf, buf);
		break;
	    default:
		QN_ERROR(CLASSNAME, "unknown MLPW data type %d", dtype);
	    }
	    buf += thisblock;

	    thiscountrem -= thisblock;
	}
	countrem -= thiscount;
	io_count -= thiscount;

	// Maybe update state machine
	if (io_count==0) {
	    int ix = 0;
	    switch(io_state) {
	    case QN_MLP3_INPUT2HIDDEN:
		io_state = QN_MLP3_HIDDEN2OUTPUT;
		io_count = layerSize[1] * layerSize[2];
		ix = 1;
		break;
	    case QN_MLP3_HIDDEN2OUTPUT:
		io_state = QN_MLP3_HIDDENBIAS;
		io_count = layerSize[1];
		ix = 2;
		break;
	    case QN_MLP3_HIDDENBIAS:
		io_state = QN_MLP3_OUTPUTBIAS;
		io_count = layerSize[2];
		ix = 3;
		break;
	    case QN_MLP3_OUTPUTBIAS:
		if (countrem > 0) {
		    QN_ERROR(CLASSNAME, "%d too many weights writing MLPW", 
			     countrem);
		}
		break;
	    default:
		assert(0);
	    }
	    read_sect_header(io_state, io_count, 
				&datatype[ix], &exponents[ix]);
	    exp = exponents[ix];
	    dtype = datatype[ix];
	}
    }
    // We get here if we return as many elements as requested
    delete [] mybuf;

    return count;
}

// Write out a buffer full of weights
size_t QN_MLPWeightFile_MLPW::write(const float* buf, size_t count) {
    float *mybuf = new float[QN_MLPW_BUF_SIZE];

    assert(mode == QN_WRITE);

    // If we haven't already, write the header
    // deferred to here to give caller a chance to set up exponents
    // before they are written out.
    if (io_state == QN_MLP3_HEADER) {
	int ec;

	ec = write_header();
	if (ec<0) {
	    QN_ERROR(CLASSNAME, "error writing weight file \'%s\' - %s.",
		     filename, strerror(errno));
	}
	io_count = layerSize[0] * layerSize[1];
	io_state = QN_MLP3_INPUT2HIDDEN;
	exp = exponents[0];
	dtype = datatype[0];
	write_sect_header(io_state, io_count, dtype, exp);
    }

    // This is a hack so we can keep attempting to write past the end of file
    // without crashing
    if (io_count==0 || count==0)
	return(0);

    int thiscount, thiscountrem, thisblock;
    int bytespersamp = QN_MLPW_dt_sizeof(dtype);
    int wtsperbuf = QN_MLPW_BUF_SIZE*(sizeof(float)/max(sizeof(float),bytespersamp));
    int countrem = count;

    while (countrem > 0 & io_count > 0) {
	thiscount = min(countrem, io_count);
	thiscountrem = thiscount;
	while (thiscountrem > 0) {
	    thisblock = min(wtsperbuf, thiscountrem); 
	    //mul_vff_vf(thisblock, buf, invWeightScale, mybuf);
	    switch (dtype) {
	    case QN_MLPW_dt_float:
		htob_vf_vf(thisblock, buf, mybuf);
		break;
	    case QN_MLPW_dt_double:
		conv_vf_vd(thisblock, buf, (double *)mybuf);
		htob_vd_vd(thisblock, (double *)mybuf, (double *)mybuf);
		break;
	    case QN_MLPW_dt_int32:
		conv_vf_vx32s(thisblock, exp, buf, (QNInt32 *)mybuf);
		htob_vi32_vi32(thisblock, (QNInt32 *)mybuf,(QNInt32 *)mybuf);
		break;
	    case QN_MLPW_dt_int16:
		conv_vf_vx16s(thisblock, exp, buf, (QNInt16 *)mybuf);
		htob_vi16_vi16(thisblock, (QNInt16 *)mybuf,(QNInt16 *)mybuf);
		break;
	    case QN_MLPW_dt_int8:
		conv_vf_vx32s(thisblock, exp, buf, (QNInt32 *)mybuf);
		conv_vi32_vi8(thisblock, (QNInt32 *)mybuf,(QNInt8 *)mybuf);
		break;
	    default:
		QN_ERROR(CLASSNAME, "unknown MLPW data type %d", dtype);
	    }

	    if (fwrite(mybuf, bytespersamp, thisblock, stream) != thisblock) {
		QN_ERROR(CLASSNAME, "error writing weight file \'%s\' - %s",
			 filename, strerror(errno));
	    }
	    buf += thisblock;
	    thiscountrem -= thisblock;
	}
	countrem -= thiscount;
	io_count -= thiscount;

	// Maybe update state machine
	if (io_count==0) {
	    int ix = 0;
	    switch(io_state) {
	    case QN_MLP3_INPUT2HIDDEN:
		io_state = QN_MLP3_HIDDEN2OUTPUT;
		ix = 1;
		io_count = layerSize[1] * layerSize[2];
		break;
	    case QN_MLP3_HIDDEN2OUTPUT:
		io_state = QN_MLP3_HIDDENBIAS;
		ix = 2;
		io_count = layerSize[1];
		break;
	    case QN_MLP3_HIDDENBIAS:
		io_state = QN_MLP3_OUTPUTBIAS;
		ix = 3;
		io_count = layerSize[2];
		break;
	    case QN_MLP3_OUTPUTBIAS:
		if (countrem > 0) {
		    QN_ERROR(CLASSNAME, "%d too many weights writing MLPW", 
			     countrem);
		}
		break;
	    default:
		assert(0);
	    }
	    exp = exponents[ix];
	    dtype = datatype[ix];
	    write_sect_header(io_state, io_count, dtype, exp);
	}
    }

    delete [] mybuf;

    return count;
}

enum QN_WeightMaj
QN_MLPWeightFile_MLPW::get_weightmaj(QN_SectionSelector)
{
    return QN_OUTPUTMAJOR;
}

enum QN_FileMode
QN_MLPWeightFile_MLPW::get_filemode()
{
    return mode;
}

