const char* QN_MLP_MultiSPERTFx3_rcsid =
    "$Header: /u/drspeech/src/quicknet/RCS/QN_MLP_MultiSPERTFx3.cc,v 1.6 1999/04/02 01:01:01 dpwe Exp $";

/* Must include the config.h file first */
#include <QN_config.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "fltvec.h"
#include "QN_types.h"
#include "QN_Logger.h"
#include "QN_MLP_MultiSPERTFx3.h"

#if defined(QN_HAVE_LIBSPCMASTER)

#include "QN_SPC_MultiSPERTFx3_1.h"
// #include "QN_MLP_MultiSPERTFx3_slave.h"
#include "spcmaster.h"

// A simple minimum routine

static inline unsigned int
min(unsigned int a, unsigned int b)
{
    if (a<b)
	return a;
    else
	return b;
}

QN_MLP_MultiSPERTFx3::QN_MLP_MultiSPERTFx3(
    int a_debug, const char* a_dbgname,
    size_t a_input, size_t a_hidden,
    size_t a_output,
    enum QN_OutputLayerType a_outtype,
    size_t a_slaves,
    int a_in2hid_exp,
    int a_hid2out_exp,
    size_t a_size_bunch)
  : clog(a_debug, "QN_MLP_MultiSPERTFx3", a_dbgname),
    n_input(a_input),
    n_hidden(a_hidden),
    n_output(a_output),
    out_layer_type(a_outtype),
      in2hid_exp(a_in2hid_exp),
      hid2out_exp(a_hid2out_exp),
      n_slaves(a_slaves),
      size_bunch(a_size_bunch)
{
    const float init = 0.0f;	// Initialize weights and layers to this

    // Maybe we do not support all output layer types
    switch(out_layer_type)
    {
    case QN_OUTPUT_SIGMOID:
    case QN_OUTPUT_SIGMOID_XENTROPY:
    case QN_OUTPUT_SOFTMAX:
    case QN_OUTPUT_LINEAR:
	break;
    default:
	assert(0);		// Only the above output layer types are
				// supported.
    }

    in_y = malloc_vi16(n_input*size_bunch);
    out_x = malloc_vf(n_output*size_bunch*n_slaves);

    out_dedx = malloc_vf( n_output * size_bunch);
    copy_f_vf(n_output * size_bunch, init, out_dedx);

    // These intermediate values are only needed for sigmoid output layers
    // ...and only then when we are NOT doing cross entropy
    if (out_layer_type==QN_OUTPUT_SIGMOID)
    {
	out_dedy = malloc_vf(n_output*size_bunch);
	copy_f_vf(n_output*size_bunch, init, out_dedy);
	out_dydx = malloc_vf(n_output*size_bunch);
	copy_f_vf(n_output*size_bunch, init, out_dydx);
    }

    slaves = new spc_server*[n_slaves];
    slaves_hidden = new size_t[n_slaves];

    spc_args args;
    size_t hidden_remaining = n_hidden;
    for(size_t p=0;p<n_slaves;p++)
    {
	size_t hidden_this_p = hidden_remaining/(n_slaves-p);
	slaves_hidden[p] = hidden_this_p;

	slaves[p] = new spc_server("qnspc_multispertfx3_1");
	if(slaves[p]==0) {
	    clog.error("Cannot open partial MLP %ul",(unsigned long) p);
	}

	args.clear();
	args.put_i32( SPC_IN, 0); // Old debug argument.
	args.put_u32( SPC_IN, n_input);
	args.put_u32( SPC_IN, hidden_this_p);
	args.put_u32( SPC_IN, n_output);
	// Only need to add bias on partial net 0.
	args.put_u32( SPC_IN, (p==0)? 1 : 0);
	args.put_i32( SPC_IN, in2hid_exp);
	args.put_i32( SPC_IN, hid2out_exp);
	args.put_u32( SPC_IN, size_bunch);
	
	slaves[p]->call( QN_SPC_MultiSPERTFx3::CONSTRUCT, &args );
	slaves[p]->wait();

	hidden_remaining -= hidden_this_p;
    }
}

QN_MLP_MultiSPERTFx3::~QN_MLP_MultiSPERTFx3()
{
    for(size_t p=0;p<n_slaves;p++)
      {
        slaves[p]->call( QN_SPC_MultiSPERTFx3::DESTRUCT );
	/* exits spert program --> won't synchronize */
      }
    delete[] slaves_hidden;
    delete[] slaves;
    if (out_layer_type==QN_OUTPUT_SIGMOID)
    {
	free_vf(out_dydx);
	free_vf(out_dedy);
    }
    free_vf(out_dedx);
    free_vf(out_x);
    free_vi16(in_y);
    free(slaves);
}

size_t
QN_MLP_MultiSPERTFx3::size_layer(QN_LayerSelector layer) const
{
    size_t r;

    switch(layer)
    {
    case QN_MLP3_INPUT:
	r = n_input;
	break;
    case QN_MLP3_HIDDEN:
	r = n_hidden;
	break;
    case QN_MLP3_OUTPUT:
	r = n_output;
	break;
    default:
	r = 0;
	break;
    }
    return r;
}

void
QN_MLP_MultiSPERTFx3::size_section(QN_SectionSelector section,
				   size_t* output_p, size_t* input_p) const
{
    switch(section)
    {
    case QN_MLP3_INPUT2HIDDEN:
	*output_p = n_hidden;
	*input_p = n_input;
	break;
    case QN_MLP3_HIDDENBIAS:
	*output_p = n_hidden;
	*input_p = 1;
	break;
    case QN_MLP3_HIDDEN2OUTPUT:
	*output_p = n_output;
	*input_p = n_hidden;
	break;
    case QN_MLP3_OUTPUTBIAS:
	*output_p = n_output;
	*input_p = 1;
	break;
    default:
	assert(0);
    }
}

size_t
QN_MLP_MultiSPERTFx3::num_connections() const
{
    // Note how we take account of the biases separately
    return (n_input+n_output) * n_hidden + n_hidden + n_output;
}

void
QN_MLP_MultiSPERTFx3::forward(size_t n_frames, const float* in, float* out)
{
    int i, frames_this_bunch;

    for (i=n_frames; i > 0; i-= SIZE_METABUNCH)
    {
      frames_this_bunch = min(SIZE_METABUNCH, i);
      forward_bunchP( frames_this_bunch, in, out);
      in += n_input * frames_this_bunch;
      out += n_output * frames_this_bunch;
    }
}

void
QN_MLP_MultiSPERTFx3::train(size_t n_frames, const float* in,
			    const float* target, float* out)
{
    int i, frames_this_bunch;
    
    for ( i=n_frames; i > 0; i -= SIZE_METABUNCH)
    {
      frames_this_bunch = min(SIZE_METABUNCH, i);
      train_bunchP( frames_this_bunch, in, target, out);
      in += n_input * frames_this_bunch;
      out += n_output * frames_this_bunch;
      target += n_output * frames_this_bunch;
    }
}

void
QN_MLP_MultiSPERTFx3::forward_bunchP(size_t n_frames, const float* in, float* out)
{
  //  Pipelined forward processing:
  //
  //  convert in1
  //  call P1->fwd(in1)
  //  call P2->fwd(in1)
  // -----------loop---------
  //  convert in2
  //  read P1.out1         call P1->fwd(in2)
  //  read P2.out1         call P2->fwd(in2)
  //  add P1.out1+P2.out1
  // ------------------------
  //  read P1.outN
  //  read P2.outN
  //  add P1.outN+P2.outN
  //  
   const size_t size_input = n_input*size_bunch;
   const size_t size_output = n_output*size_bunch;
   size_t frames_to_fwd=n_frames;
   size_t frames_to_read=n_frames;
   size_t fwd_batch=size_bunch;
   size_t read_batch=size_bunch;
   spc_args args1, args2;
   spc_args *tmpargs, *fwdargs1, *fwdargs2;
   float *t_out;
   fxbool sat=FXFALSE;

   fwdargs1 = &args1;
   fwdargs2 = &args2;

// pipeline loop
    size_t first_iter = 1;
    while( frames_to_read )
    {
       if(frames_to_fwd)
       {
	  fwd_batch = min(size_bunch, frames_to_fwd);
	  sat=conv_vf_vx16s( fwd_batch*n_input, in_y_exp, in, in_y);
	  if (sat)
	  {
	   fprintf(stderr,"In forward_bunchP: features saturated on input.\n");
	  }
	  in += size_input;
       }

       read_batch = min(size_bunch, frames_to_read);
       t_out = out_x;
       tmpargs=fwdargs2; fwdargs2=fwdargs1; fwdargs1=tmpargs;
       for( size_t p=0;p<n_slaves;p++ )
       {
	  if(first_iter==0)
	  {
	     slaves[p]->wait();
	     slaves[p]->get_vf(t_out);
	     t_out+=size_output;
	  }
	  if(frames_to_fwd)
	  {
	    if(p==0)
	    {
	       fwdargs1->clear();
	       fwdargs1->put_u32(SPC_IN, fwd_batch);
	       fwdargs1->put_vi16(SPC_IN, in_y, fwd_batch*n_input);
	       fwdargs1->put_vf(SPC_OUT, 0, fwd_batch*n_output);
	    }
	   slaves[p]->call( QN_SPC_MultiSPERTFx3::FORWARD, fwdargs1 );
	  }   // end FORsize_t p=0;p<n_slaves;p++
       }
       frames_to_fwd -= fwd_batch;
       if(first_iter==0) {
	 frames_to_read -= read_batch;

	 t_out = out_x;
	 for( size_t p=1; p < n_slaves; p++) // add partial sums -> output
	 {
	   t_out += size_output;
	   acc_vf_vf(read_batch*n_output,t_out, out_x);
	 }

	 switch(out_layer_type)     // comp output nonlinearity
	 {
	   case QN_OUTPUT_SIGMOID:
	   case QN_OUTPUT_SIGMOID_XENTROPY:
	     sigmoid_vf_vf(read_batch*n_output, out_x, out);
	     break;
	   case QN_OUTPUT_SOFTMAX:
	   {
	     float* cur_out_x = out_x;
	     float* cur_out = out;

	     for (size_t i=0; i<read_batch; i++)
	     {
	       softmax_vf_vf(n_output, cur_out_x, cur_out);
	       cur_out_x += n_output;
	       cur_out += n_output;
	     }
	     break;
	   }
	   case QN_OUTPUT_LINEAR:
	     copy_vf_vf(read_batch*n_output, out_x, out);
	     break;
	   default:
	     assert(0);
	 }
	 out += size_output;
       }
       first_iter = 0;
    }  //end while(frames_to_read>0)

}

void
QN_MLP_MultiSPERTFx3::train_bunchP(size_t n_frames, const float *in,
				   const float* target, float* out)
{
//  Pipelined backprop training
//
//  convert in1
//  call P1->fwd(in1)  call P2->fwd(in1)
//  read P1.out1
//  read P2.out1
//  comp  err1(P1.out1,P2.out1)
//  ------------------------
//  convert in2
//  call P1->fwd(in2)  call P2->fwd(in2)
//  read P1.out2       call P1->bck(err1)
//  read P2.out2       call P2->bck(err1)
//  comp  err1(P1.out2,P2.out2)
//  ------------------------
//  call P1->bck(errN) call P2->bck(errN)
//  

   const size_t size_input = n_input*size_bunch;
   const size_t size_output = n_output*size_bunch;
   const int in_y_exp = 3;
   fxbool sat=FXFALSE;
   float *t_out;
   float *t_target=(float*)target;
   size_t sz_output;
   size_t fwd_batch=0;
   size_t last_batch=0;
   size_t frames_to_fwd = n_frames;
   int first_iter = 1;
   int bck_called = 0;
   spc_args fwdargs;
   spc_args bckargs;

// Pipelined training
   while(frames_to_fwd)
   {
      last_batch = fwd_batch;
      fwd_batch = min(size_bunch, frames_to_fwd);
      sat=conv_vf_vx16s( fwd_batch*n_input, in_y_exp, in, in_y);
      if (sat)
      {
	 fprintf(stderr,"In forward_bunchP: features saturated on input.\n");
      }
      in += size_input;

      for( size_t p=0;p<n_slaves;p++ )
      {
	 if(p==0)
	 {
	    fwdargs.clear();
	    fwdargs.put_i32( SPC_IN, fwd_batch);
	    fwdargs.put_vi16( SPC_IN, in_y, fwd_batch*n_input);
	    fwdargs.put_vf( SPC_OUT, 0, fwd_batch*n_output);
	 }
	 if(bck_called)
	 {
	     slaves[p]->wait();   // synch with backprop call
	 }

	 slaves[p]->call( QN_SPC_MultiSPERTFx3::FORWARD, &fwdargs );
       }
      t_out = out_x;
      for( size_t p=0;p<n_slaves;p++ )
      {
	  slaves[p]->wait();
	  slaves[p]->get_vf(t_out);
	  t_out+=size_output;
 
	  if(!first_iter)
	  {
	     if(p==0)
	     {
		bckargs.clear();
		bckargs.put_u32(SPC_IN, last_batch);
		bckargs.put_vf(SPC_IN, out_dedx, last_batch*n_output);
	     }
	     slaves[p]->call( QN_SPC_MultiSPERTFx3::BACKWARD, &bckargs );
	     bck_called=1;
	  }
      }   // end FORsize_t p=0;p<n_slaves;p++

       frames_to_fwd -= fwd_batch;
       sz_output=fwd_batch*n_output;
       t_out = out_x;
       for(size_t p=1;p<n_slaves;p++)
       {
	  t_out += size_output;
	  acc_vf_vf(sz_output,t_out, out_x);
       }

       switch(out_layer_type)
       {
	  case QN_OUTPUT_SIGMOID:
	    sigmoid_vf_vf(sz_output, out_x, out);
	    sub_vfvf_vf(sz_output, out, t_target, out_dedy);
	    dsigmoid_vf_vf(sz_output, out, out_dydx);
	    mul_vfvf_vf(sz_output, out_dydx, out_dedy, out_dedx);
	    break;
	  case QN_OUTPUT_SIGMOID_XENTROPY:
	    sigmoid_vf_vf(sz_output, out_x, out);
	    sub_vfvf_vf(sz_output, out, t_target, out_dedx);
	    break;
	  case QN_OUTPUT_SOFTMAX:
	  {
	     size_t i;
	     float* cur_out_x = out_x;
	     float* cur_out = out;

	     for (i=0; i<fwd_batch; i++)
	     {
		softmax_vf_vf(n_output, cur_out_x, cur_out);
		cur_out_x += n_output;
		cur_out += n_output;
	     }
	     sub_vfvf_vf(sz_output, out, t_target, out_dedx);
	     break;
	  }
	  case QN_OUTPUT_LINEAR:
	    copy_vf_vf(sz_output, out_x, out);
	    sub_vfvf_vf(sz_output, out, t_target, out_dedx);
	    break;
	  default:
	    assert(0);
       }
       first_iter=0;
       out+=size_output;
       t_target+=size_output;
   }
   for(size_t p=0;p<n_slaves;p++)
   {
      if(bck_called)
      {
	 slaves[p]->wait();   // synch with backprop call
      }
      if(p==0)
      {
	 bckargs.clear();
	 bckargs.put_u32(SPC_IN, fwd_batch+size_bunch); /* use old values */
	 bckargs.put_vf(SPC_IN, out_dedx , fwd_batch*n_output);
      }
      slaves[p]->call( QN_SPC_MultiSPERTFx3::BACKWARD, &bckargs );
   }
   for(size_t p=0;p<n_slaves;p++)
   {
      slaves[p]->wait();   // synch with backprop call
   }
}



void
QN_MLP_MultiSPERTFx3::set_weights(enum QN_SectionSelector which,
			     size_t row, size_t col,
			     size_t n_rows, size_t n_cols,
			     const float* from)
{
    assert((col==0));
    float from_buf[n_output*(n_hidden/n_slaves+1)];
    spc_args args;

    switch(which)
    {
        case QN_MLP3_INPUT2HIDDEN: {
	  assert((n_cols==n_input));
	  size_t p=0;
	  size_t rows=n_rows;
	  size_t skip=row;
	  while( skip >= slaves_hidden[p])     // skip to proc 'row'
	  {
	    skip-=slaves_hidden[p];
	    p++;
	  }
	  while( rows )                 // transfer 'rows' rows
	  {
	    size_t r = min( rows, slaves_hidden[p]-skip);
	    int size_from = r*n_input;
	    args.clear();
	    args.put_u32( SPC_IN, which);
	    args.put_u32( SPC_IN, skip);
	    args.put_u32( SPC_IN, 0);
	    args.put_u32( SPC_IN, r);
	    args.put_u32( SPC_IN, n_input);
	    args.put_vf( SPC_IN, (float*)from, size_from );
	    slaves[p]->call( QN_SPC_MultiSPERTFx3::SETWEIGHTS, &args);
	    slaves[p]->wait();
	    from+=size_from;
	    skip=0;
	    rows-=r;
	    p++;
	  }
	  break; }
	case QN_MLP3_HIDDENBIAS:
	  assert((n_rows==n_hidden)&&(n_cols==1));
	  for(size_t p=0;p<n_slaves;p++)
	  {
	    int size_from = slaves_hidden[p];
	    args.clear();
	    args.put_u32( SPC_IN,which);
	    args.put_u32( SPC_IN,0);
	    args.put_u32( SPC_IN,0);
	    args.put_u32( SPC_IN,slaves_hidden[p]);
	    args.put_u32( SPC_IN,1);
	    args.put_vf( SPC_IN, (float*)from, size_from );
	    slaves[p]->call( QN_SPC_MultiSPERTFx3::SETWEIGHTS, &args);
	    slaves[p]->wait();
	    from+=size_from;
	  }
	  break;
        case QN_MLP3_HIDDEN2OUTPUT:
	  assert((n_cols==n_hidden));
	  for(size_t p=0;p<n_slaves;p++)
	  {
	    int width = slaves_hidden[p];
	    int size_from = width*n_rows;
	    for(size_t r=0; r<n_rows; r++)   // copy sub-matrix[p] to buf
	    {
	      copy_vf_vf(width, from+n_hidden*r, from_buf+width*r);
	    }
	    args.clear();
	    args.put_u32( SPC_IN,which);
	    args.put_u32( SPC_IN,row);
	    args.put_u32( SPC_IN,0);
	    args.put_u32( SPC_IN,n_rows);
	    args.put_u32( SPC_IN,slaves_hidden[p]);
	    args.put_vf( SPC_IN, from_buf, size_from );
	    slaves[p]->call( QN_SPC_MultiSPERTFx3::SETWEIGHTS, &args);
	    slaves[p]->wait();
	    from+=width;
	  }
	  break;
	case QN_MLP3_OUTPUTBIAS:
	  assert((n_rows==n_output)&&(n_cols==1));
	  args.clear();
	  args.put_u32( SPC_IN,which);
	  args.put_u32( SPC_IN,0);
	  args.put_u32( SPC_IN,0);
	  args.put_u32( SPC_IN,n_output);
	  args.put_u32( SPC_IN,1);
	  args.put_vf( SPC_IN, (float*)from, n_output );
	  slaves[0]->call( QN_SPC_MultiSPERTFx3::SETWEIGHTS, &args);
	  slaves[0]->wait();
	  break;
	default:
	  assert(0);
    }
}

void
QN_MLP_MultiSPERTFx3::get_weights(enum QN_SectionSelector which,
			      size_t row, size_t col,
			      size_t n_rows, size_t n_cols,
			      float* to) const
{
    assert((col==0));
    float to_buf[(n_hidden/n_slaves+1)*n_output];
    spc_args args;

    clog.log(QN_LOG_PER_RUN, "get_weights which=%lu row=%lu col=%lu nr=%lu nc=%lu", (unsigned long)which, (unsigned long)row, (unsigned long)col, (unsigned long)n_rows, (unsigned long)n_cols);

    switch(which)
    {
        case QN_MLP3_INPUT2HIDDEN: {
	  assert((n_cols==n_input));
	  size_t p=0;
	  size_t rows=n_rows;
	  size_t skip=row;
	  while( skip >= slaves_hidden[p])     // skip to proc 'row'
	  {
	    skip-=slaves_hidden[p];
	    p++;
	  }

	  while( rows )                 // transfer 'rows' rows
	  {
	    size_t r = min( rows, slaves_hidden[p]-skip);
	    int size_to = r*n_input;
	    args.clear();
	    args.put_u32( SPC_IN,which);
	    args.put_u32( SPC_IN,skip);
	    args.put_u32( SPC_IN,0);
	    args.put_u32( SPC_IN,r);
	    args.put_u32( SPC_IN,n_input);
	    args.put_vf( SPC_OUT,0, size_to );
    clog.log(QN_LOG_PER_RUN, "i2h: getting %lu,%lu+%lu,%lu=%lu from SPC %lu", 
	     (unsigned long)skip, (unsigned long)0, (unsigned long)r, (unsigned long)n_input, (unsigned long)size_to, (unsigned long)p);
	    slaves[p]->call( QN_SPC_MultiSPERTFx3::GETWEIGHTS, &args);
	    slaves[p]->wait();
	    slaves[p]->get_vf( to);
	    to+=size_to;
	    skip=0;
	    rows-=r;
	    p++;
	  }
	  break;}
	case QN_MLP3_HIDDENBIAS:
	  assert((n_rows==n_hidden)&&(n_cols==1));
	  for(size_t p=0;p<n_slaves;p++)
	  {
	    args.clear();
	    args.put_u32( SPC_IN,which);
	    args.put_u32( SPC_IN,0);
	    args.put_u32( SPC_IN,0);
	    args.put_u32( SPC_IN,slaves_hidden[p]);
	    args.put_u32( SPC_IN,1);
	    args.put_vf( SPC_OUT,0, slaves_hidden[p]);
    clog.log(QN_LOG_PER_RUN, "hb: getting %lu,%lu+%lu,%lu=%lu from SPC %lu", 
	     (unsigned long)0, (unsigned long)0, (unsigned long)slaves_hidden[p], (unsigned long)1, (unsigned long)slaves_hidden[p], (unsigned long)p);
	    slaves[p]->call( QN_SPC_MultiSPERTFx3::GETWEIGHTS, &args);
	    slaves[p]->wait();
	    slaves[p]->get_vf(to);
	    to+=slaves_hidden[p];
	  }
	  break;
        case QN_MLP3_HIDDEN2OUTPUT:
	  assert((n_cols==n_hidden));
	  for(size_t p=0;p<n_slaves;p++) 
	  {
	    int size_to = slaves_hidden[p]*n_rows;
	    args.clear();
	    args.put_u32( SPC_IN,which);
	    args.put_u32( SPC_IN,row);
	    args.put_u32( SPC_IN,0);
	    args.put_u32( SPC_IN,n_rows);
	    args.put_u32( SPC_IN,slaves_hidden[p]);
	    args.put_vf( SPC_OUT,0, size_to );
    clog.log(QN_LOG_PER_RUN, "h2o: getting %lu,%lu+%lu,%lu=%lu from SPC %lu", 
	     (unsigned long)row, (unsigned long)0, (unsigned long)n_rows, (unsigned long)slaves_hidden[p], (unsigned long)size_to, (unsigned long)p);
	    slaves[p]->call( QN_SPC_MultiSPERTFx3::GETWEIGHTS, &args);
	  }
	  for(size_t p=0;p<n_slaves;p++)
	  {
	    size_t width=slaves_hidden[p];
 
	    slaves[p]->wait();
	    slaves[p]->get_vf( to_buf);
	    for(size_t r=0;r<n_rows;r++)
	    {
	      copy_vf_vf(width, to_buf+r*width, to+r*n_hidden);
	    }
	    to += width;
	  }
	  break;
	case QN_MLP3_OUTPUTBIAS: 
	  assert((n_rows==n_output)&&(n_cols==1));
	  {
	    args.clear();
	    args.put_u32( SPC_IN,which);
	    args.put_u32( SPC_IN,0);
	    args.put_u32( SPC_IN,0);
	    args.put_u32( SPC_IN,n_output);
	    args.put_u32( SPC_IN,1);
	    args.put_vf( SPC_OUT,0, n_output );
    clog.log(QN_LOG_PER_RUN, "ob: getting %lu,%lu+%lu,%lu=%lu from SPC %lu", 
	     (unsigned long)0, (unsigned long)0, (unsigned long)n_output, (unsigned long)1, (unsigned long)n_output, (unsigned long)0);

	    slaves[0]->call( QN_SPC_MultiSPERTFx3::GETWEIGHTS, &args);
	    slaves[0]->wait();
	    slaves[0]->get_vf(to);
 	  }
	  break;
	default:
	  assert(0);
    }

}

void
QN_MLP_MultiSPERTFx3::set_learnrate(enum QN_SectionSelector which, float learnrate)
{
    spc_args args;
    args.put_u32( SPC_IN, which);
    args.put_f( SPC_IN, learnrate);
    for(size_t p=0;p<n_slaves;p++)
    {
	    slaves[p]->call( QN_SPC_MultiSPERTFx3::SETLEARN, &args );
	    slaves[p]->wait();
    }

    switch(which)
    {
    case QN_MLP3_INPUT2HIDDEN:
	neg_in2hid_learnrate = -learnrate;
	break;
    case QN_MLP3_HIDDEN2OUTPUT:
	neg_hid2out_learnrate = -learnrate;
	break;
    case QN_MLP3_HIDDENBIAS:
	neg_hid_learnrate = -learnrate;
	break;
    case QN_MLP3_OUTPUTBIAS:
	neg_out_learnrate = -learnrate;
	break;
    default:
	assert(0);
	break;
    }
}

float 
QN_MLP_MultiSPERTFx3::get_learnrate(enum QN_SectionSelector which) const
{
    float res;			// Returned learning rate
    spc_args args;
    args.put_u32( SPC_IN, which);
    args.put_f( SPC_OUT, 0);

    slaves[0]->call( QN_SPC_MultiSPERTFx3::GETLEARN, &args);
    slaves[0]->wait();
    slaves[0]->get_f( &res);

    return res;
}

#else
static int dummy;
#endif /* #if defined(QN_HAVE_LIBSPCMASTER) && defined(QN_HAVE_LIBFXLIB) */

