const char* QN_SentDB_InFtrLabStream_rcsid =
    "$Header: /n/lasagna/db/davidj/spert/src/quicknet/RCS/QN_SentDB_InFtrLabStream.cc,v 1.3 1996/05/25 02:27:56 davidj Exp $";

// QN_SentDB_InFtrLabStream is a hack, presenting the new new input stream
// interfaces as old QN_SentDB objects.


#include <QN_config.h>
#include <assert.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "fltvec.h"
#include "QN_types.h"
#include "QN_SentDB_InFtrLabStream.h"
#include "QN_libc.h"
#include "QN_Logger.h"


// Constructor.  This opens and analyses the pfile, and builds up some
// information that will be useful for accessing the pfile later.

QN_SentDB_InFtrLabStream::QN_SentDB_InFtrLabStream(int a_dbg,
						   const char* a_dbgname,
						   QN_InFtrLabStream& str)
  : log(a_dbg, "QN_SentDB_InFtrLabStream", a_dbgname),
    instr(str),
    n_ftrs(instr.num_ftrs()),
    n_labs(instr.num_labs())
{
    n_sents = instr.num_segs();
    assert(n_sents!=QN_SIZET_BAD && n_sents>0);
    n_frames = instr.num_frames();
    assert(n_frames!=QN_SIZET_BAD && n_frames>0);
}

QN_SentDB_InFtrLabStream::~QN_SentDB_InFtrLabStream()
{
}

// Get the number of frames in a given sentence.
size_t
QN_SentDB_InFtrLabStream::num_frames_in_sents(size_t first_sent,
					      size_t num_sents)
{
    size_t sent;		// Current sentence
    size_t frames = 0;		// Accumulator for number of frames
    
    assert((first_sent+num_sents)<=n_sents);

    for (sent=first_sent; sent<first_sent+num_sents; sent++)
    {
	size_t frames_this_sent; // Frames in current sentence
	
	frames_this_sent = instr.num_frames(sent);
	if (frames_this_sent==QN_SIZET_BAD || frames_this_sent==0)
	{
	    assert(0);
	}
	frames += frames_this_sent;
    }
    return frames;
}

size_t
QN_SentDB_InFtrLabStream::get_sentences(size_t first_sent, size_t num_sents,
					QNUInt32* sentnos, QNUInt32* framenos,
					float* ftrs, QNUInt32* labs,
					float* tgts)
{
    size_t sent;		// Current sentence
    size_t frame;		// Current frame
    size_t total_frames = 0;	// Accumulator for number of frames
    QN_SegID segid;		// The segmend ID of current sentence

    assert((first_sent+num_sents)<=n_sents);
    assert(tgts==NULL);

    segid = instr.set_pos(first_sent,0);
    assert(segid!=QN_SEGID_BAD);
    for (sent=first_sent; sent<first_sent+num_sents; sent++)
    {
	size_t num_frames;

	num_frames = instr.num_frames(sent);
	assert(num_frames!=QN_SIZET_BAD);

	for (frame=0; frame<num_frames; frame++)
	{
	    if (sentnos!=NULL)
		*sentnos++ = sent;
	    if (framenos!=NULL)
		*framenos++ = frame;
	    instr.read_ftrslabs(1, ftrs, labs);
	    if (ftrs!=NULL)
		ftrs += n_ftrs;
	    if (labs!=NULL)
		labs += n_labs;
	}
	total_frames += num_frames;
	segid = instr.nextseg();
    }
    return total_frames;
}

////////////////////////////////////////////////////////////////


QN_SentDB_InFtrStream_InLabStream::
QN_SentDB_InFtrStream_InLabStream(int a_dbg,
				  const char* a_dbgname,
				  QN_InFtrStream& a_ftr_str,
				  QN_InLabStream& a_lab_str,
				  size_t a_bunch_size)
								     
  : log(a_dbg, "QN_SentDB_InFtrStream_InLabStream", a_dbgname),
    ftr_str(a_ftr_str),
    lab_str(a_lab_str),
    bunch_size(a_bunch_size),
    n_ftrs(ftr_str.num_ftrs()),
    n_labs(lab_str.num_labs())
{
    n_sents = ftr_str.num_segs();
    if (lab_str.num_segs()!=n_sents)
    {
	log.error("Cannot combine feature and label streams containing "
		  "different number of sentences.");
    }
    n_frames = ftr_str.num_frames();
    if (lab_str.num_frames()!=n_frames)
    {
	log.error("Cannot combine feature and label streams containing "
		  "different number of frames.");
    }
    if (n_sents==QN_SIZET_BAD || n_frames==QN_SIZET_BAD)
    {
	log.error("Cannot find size of label and feature streams - "
		  "label and feature streams must be indexed.");
    }
}

QN_SentDB_InFtrStream_InLabStream::~QN_SentDB_InFtrStream_InLabStream()
{
}

// Get the number of frames in a given sentence.
size_t
QN_SentDB_InFtrStream_InLabStream::num_frames_in_sents(size_t first_sent,
					      size_t num_sents)
{
    size_t sent;		// Current sentence
    size_t frames = 0;		// Accumulator for number of frames
    
    assert((first_sent+num_sents)<=n_sents);

    for (sent=first_sent; sent<first_sent+num_sents; sent++)
    {
	size_t frames_this_sent; // Frames in current sentence
	
	frames_this_sent = ftr_str.num_frames(sent);
	if (frames_this_sent!=lab_str.num_frames(sent))
	{
	    log.error("Sentence %lu has different length in feature "
		      "and label streams.", (unsigned long) sent);
	}
	if (frames_this_sent==QN_SIZET_BAD || frames_this_sent==0)
	{
	    assert(0);
	}
	frames += frames_this_sent;
    }
    return frames;
}

size_t
QN_SentDB_InFtrStream_InLabStream::
get_sentences(size_t first_sent, size_t num_sents,
	      QNUInt32* sentnos, QNUInt32* framenos,
	      float* ftrs, QNUInt32* labs, float* tgts)
{
    size_t sent;		// Current sentence
    size_t frame;		// Current frame
    size_t total_frames = 0;	// Accumulator for number of frames
    QN_SegID ftr_segid, lab_segid; // The segmend ID of current sentence

    assert((first_sent+num_sents)<=n_sents);
    assert(tgts==NULL);

    ftr_segid = ftr_str.set_pos(first_sent,0);
    assert(ftr_segid!=QN_SEGID_BAD);
    lab_segid = lab_str.set_pos(first_sent, 0);
    assert(lab_segid!=QN_SEGID_BAD);
    assert(lab_segid==ftr_segid);
    for (sent=first_sent; sent<first_sent+num_sents; sent++)
    {
	size_t ftr_frames, lab_frames;

	ftr_frames = ftr_str.num_frames(sent);
	assert(ftr_frames!=QN_SIZET_BAD);
	lab_frames = lab_str.num_frames(sent);
	assert(lab_frames!=QN_SIZET_BAD);
	assert(lab_frames==ftr_frames);

	for (frame=0; frame<ftr_frames; frame+=bunch_size)
	{
	    size_t this_bunch_size; // The frames we are reading in one go.
	    size_t frame2;	// A counter used to iterate over all frames
				// in the bunch.

	    this_bunch_size = bunch_size;
	    if ((ftr_frames - frame) < bunch_size)
		this_bunch_size = ftr_frames - frame;
	    for (frame2 = frame; frame2<(frame+this_bunch_size); frame2++)
	    {
		if (sentnos!=NULL)
		    *sentnos++ = sent;
		if (framenos!=NULL)
		    *framenos++ = frame2;
	    }
	    ftr_str.read_ftrs(this_bunch_size, ftrs);
	    lab_str.read_labs(this_bunch_size, labs);
	    if (ftrs!=NULL)
		ftrs += (n_ftrs * this_bunch_size);
	    if (labs!=NULL)
		labs += (n_labs * this_bunch_size);
	}
	total_frames += ftr_frames;
	ftr_segid = ftr_str.nextseg();
	lab_segid = lab_str.nextseg();
	assert(ftr_segid==lab_segid);
    }
    return total_frames;
}

