// $Header: /n/lasagna/db/davidj/spert/src/quicknet/RCS/QN_SentDB_InFtrLabStream.h,v 1.3 1996/05/25 02:33:43 davidj Exp $

#ifndef QN_SentDB_InFtrLabStream_h_INCLUDED
#define QN_SentDB_InFtrLabStream_h_INCLUDED

// QN_SentDB_InFtrLabStream is a hack, presenting the new new input stream
// interfaces as old QN_SentDB objects.

/* Must include the config.h file first */
#include <QN_config.h>
#include <assert.h>
#include <stdio.h>
#include "QN_types.h"
#include "QN_SentDB.h"
#include "QN_streams.h"
#include "QN_Logger.h"

class QN_SentDB_InFtrLabStream : public QN_SentDB
{
public:
    QN_SentDB_InFtrLabStream(int a_dbg, const char* a_dbgname,
			     QN_InFtrLabStream& a_instr);
    ~QN_SentDB_InFtrLabStream();

    // Return the number of frames in the database.
    size_t num_frames() { return n_frames; };
    // Return the number of sentences in the database.
    size_t num_sentences() { return n_sents; };
    // Return the number of frames in a range of sentences.
    size_t num_frames_in_sents(size_t first_sent, size_t num_sents);
    // Return the number of features in a frame.
    size_t num_features_in_frame() { return n_ftrs; };
    // Return the number of labels in a frame.
    size_t num_labels_in_frame() { return n_labs; };
    // Return the number of targets in a frame.
    size_t num_targets_in_frame() { return 0; };
    
    // Gets the labels and features for a range of sentences.
    // If sentnos is NULL, do not get sentence numbers.
    // If framenos is NULL, do not get frame numbers.
    // If ftrs is NULL, do not get features.
    // If labs is NULL, do not get labels.
    // If tgts is NULL, do not get targets.
    // Use num_frames_in_sents() first to check space
    size_t get_sentences(size_t first_sent, size_t num_sent,
			 QNUInt32* sentnos, QNUInt32* framenos,
			 float* ftrs, QNUInt32* labs, float* tgts);

    // Some details about the InFtrLabStream itself - not in abstract class

private:
    QN_ClassLogger log;		// Used for logging.

    QN_InFtrLabStream& instr;	// The InFtrLabStream reader object

    size_t n_sents;		// Number of sentences in InFtrLabStream
    size_t n_frames;		// Total number of frames in InFtrLabStream
    const size_t n_ftrs;	// Number of featuers in InFtrLabStream
    const size_t n_labs;	// Number of labels in InFtrLabStream
};

////////////////////////////////////////////////////////////////
// Another hack to convert separate label and feature streams to
// form an old-style SentDB.

class QN_SentDB_InFtrStream_InLabStream : public QN_SentDB
{
public:
    QN_SentDB_InFtrStream_InLabStream(int a_dbg, const char* a_dbgname,
				      QN_InFtrStream& a_ftr_str,
				      QN_InLabStream& a_lab_str,
				      size_t a_bunch_size = 100);
    ~QN_SentDB_InFtrStream_InLabStream();

    // Return the number of frames in the database.
    size_t num_frames() { return n_frames; };
    // Return the number of sentences in the database.
    size_t num_sentences() { return n_sents; };
    // Return the number of frames in a range of sentences.
    size_t num_frames_in_sents(size_t first_sent, size_t num_sents);
    // Return the number of features in a frame.
    size_t num_features_in_frame() { return n_ftrs; };
    // Return the number of labels in a frame.
    size_t num_labels_in_frame() { return n_labs; };
    // Return the number of targets in a frame.
    size_t num_targets_in_frame() { return 0; };
    
    // Gets the labels and features for a range of sentences.
    // If sentnos is NULL, do not get sentence numbers.
    // If framenos is NULL, do not get frame numbers.
    // If ftrs is NULL, do not get features.
    // If labs is NULL, do not get labels.
    // If tgts is NULL, do not get targets.
    // Use num_frames_in_sents() first to check space
    size_t get_sentences(size_t first_sent, size_t num_sent,
			 QNUInt32* sentnos, QNUInt32* framenos,
			 float* ftrs, QNUInt32* labs, float* tgts);

    // Some details about the InFtrLabStream itself - not in abstract class

private:
    QN_ClassLogger log;		// Used for logging.

    QN_InFtrStream& ftr_str;	// The InFtrStream reader object.
    QN_InLabStream& lab_str;	// The InLabStream reader object.

    size_t bunch_size;		// The number of frames to read from input
				// stream at one time.

    size_t n_sents;		// Number of sentences in the streams.
    size_t n_frames;		// Total number of frames in the streams.
    const size_t n_ftrs;	// Number of features in InFtrStream.
    const size_t n_labs;	// Number of labels in InLabStream.

};


#endif // #ifdef QN_SentDB_InFtrLabStream_h_INCLUDED
