/*
 *      $Header: /u/drspeech/src/wordscore/RCS/sentence.h,v 1.6 1999/03/01 04:42:13 dpwe Exp $
 *
 *	Sentence class for handling strings, etc
 */

#ifndef __SENTENCE_H__
#define __SENTENCE_H__

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "defines.h"


class sentence {

private:

  FILE *fp ;			// input file pointer
  char **utt;			// utterance array
  int max_num_words;
  char *buffer;			// complete utterance
  int buffer_size;
  int numWords ;		// number of words in the utterance
  int lineNo ;			// current line number
  char *fileName;		// source file name
  char *BSIL ;			// begin silence string
  char *ESIL ;			// end silence string
  char *MSIL ;			// mid-silence string
  int bsillen;
  int esillen;
  int msillen;

public:

  // constructor --------------------------------------------------------------
  sentence( char *filename, char *bsil, char *esil, char *msil ) {

    // open the file containing the data 
    // "-" is taken to mean standard input
    if (strcmp(filename, "-") == 0) {
	fp = stdin;
	fileName = "stdin";
    } else {
        fp = fopen( filename, "r" ) ;
	fileName = strdup(filename);
    }
    if( fp == NULL ) {
      fprintf( stderr, "sentence::sentence : " ) ;
      perror( filename ) ;
      exit( 1 ) ;
    }

    // initialize the number of words
    numWords = 0 ;

    // initialize the line number
    lineNo = 0;

    buffer_size = BUFFER_SIZE;
    buffer = new char[buffer_size];

    max_num_words = MAX_NUM_WORDS;
    utt = new char *[max_num_words];

    // set the begin, end and middle silence
    BSIL = strdup( bsil ) ;  bsillen = strlen(BSIL);
    ESIL = strdup( esil ) ;  esillen = strlen(ESIL);
    MSIL = strdup( msil ) ;  msillen = strlen(MSIL);
  }

  ~sentence() {
    delete[] buffer;
    delete[] utt;
  }
    
  // get the next sentence from the file --------------------------------------
  int getNext() {

    // initialize number of words
    numWords = 0 ;

    // read the sentence into the main buffer
    lineNo++;
    char *fgets_result = fgets( buffer, buffer_size, fp );
    if (fgets_result &&
	lineNo == 1 && !strncmp(buffer, "noway version", 13) ) {
      lineNo++;
      fgets_result = fgets( buffer, buffer_size, fp );
    }
    while (fgets_result && buffer[strlen(buffer)-1] != '\n') {
      char *tmp_buf=new char[2 * buffer_size];
      memcpy(tmp_buf, buffer, buffer_size);

      delete[] buffer;

      buffer = tmp_buf;
      char *buf_ptr = buffer + strlen(buffer);
      fgets_result = fgets(buf_ptr, buffer_size, fp);
      buffer_size *= 2;
    }

    if (!fgets_result) {
       if (fp != stdin) {
           fclose(fp);
       }
       return 0 ;
    }

    // get the first word
    utt[numWords] = strtok( buffer, " \t\n" ) ;
    if( utt[numWords] == NULL ) {
      // empty sentence, it's okay.
      return 1;
    }	
    
    // ignore starting numbers and parens
    if( isdigit( utt[numWords][0] ) || utt[numWords][0] == '(' ) {
      utt[numWords] = strtok( NULL, " \t\n" ) ;
      if( utt[numWords] == NULL ) {
        // empty sentence, it's okay.
        return 1;
      }
    }
           
    // ignore starting silence
    if( bsillen && !strncmp( utt[numWords], BSIL, bsillen ) ) {
      utt[numWords] = strtok( NULL, " \t\n" ) ;
      if( utt[numWords] == NULL ) {
        // empty sentence, it's okay.
	return 1;
      }
    }
           
    // get the remaining words
    numWords++ ;
    while( ( utt[numWords] = strtok( NULL, " \t\n" ) ) != NULL ) {
      if( !msillen || strncmp( utt[numWords], MSIL, msillen ) ) 
	numWords++ ;
      if( numWords >= max_num_words ) {
	char **tmp_utt = new char *[max_num_words * 2];
	memcpy(tmp_utt, utt, max_num_words * sizeof(char*));
	delete[] utt;
	utt = tmp_utt;
	max_num_words *= 2;
      }
    }

    // check for trailing silence
    if( esillen && !strncmp( utt[numWords-1], ESIL, esillen ) ) numWords-- ;

    // indicate OK status
    return 1 ;
  }

  // print the sentence -------------------------------------------------------
  void print( FILE *fpOut ) {
    for( int i = 0 ; i < numWords ; i++ ) fprintf( fpOut, "%s ", utt[i] ) ;
    fprintf( fpOut, "\n" ) ;
  }

  // get the number of words --------------------------------------------------
  int getNum() { return numWords ; }

  // get the line number ------------------------------------------------------
  int getLineNo() { return lineNo ; }

  // get a word pointer -------------------------------------------------------
  char *getWord( int i ) { return utt[i] ; }

  // get a souce file name
  char *getFileName() { return fileName; }

} ;

#endif
